Package: genoPlotR
Type: Package
Title: Plot Publication-Grade Gene and Genome Maps
Version: 0.8.11
Date: 2021-01-05
Author: Lionel Guy <lionel.guy@imbim.uu.se>
URL: http://genoplotr.r-forge.r-project.org/
Depends: R (>= 2.10.0), ade4, grid
Imports: methods
Maintainer: Lionel Guy <lionel.guy@imbim.uu.se>
Description: Draws gene or genome maps and comparisons between these, in a 
   publication-grade manner. Starting from simple, common files, it will 
   draw postscript or PDF files that can be sent as such to journals.
License: GPL (>= 2)
LazyLoad: yes
NeedsCompilation: no
Packaged: 2021-01-07 13:23:56 UTC; lionel
Repository: CRAN
Date/Publication: 2021-01-07 15:00:02 UTC
Built: R 4.6.0; ; 2026-04-27 09:39:41 UTC; unix

import("grid")
import("ade4")
import("methods")
importFrom("grDevices", "grey", "rainbow", "rgb")
importFrom("stats", "median", "optim", "weighted.mean")
importFrom("utils", "read.table")
export(
  plot_gene_map,

  dna_seg,
  is.dna_seg,
  as.dna_seg,
  c.dna_seg,
  range.dna_seg,
       
  comparison,
  is.comparison,
  as.comparison,
  range.comparison,
  
  annotation,
  is.annotation,
  as.annotation,
  range.annotation,

  seg_plot,
  is.seg_plot,
  as.seg_plot,
  trim.seg_plot,
       
  read_dna_seg_from_tab,
  read_dna_seg_from_file,
  read_dna_seg_from_embl,
  read_dna_seg_from_genbank,
  read_dna_seg_from_fasta,
  read_dna_seg_from_ptt,
  read_comparison_from_tab,
  read_comparison_from_blast,
  read_mauve_backbone,

  gene_types,
  
  human_nt,
  apply_color_scheme,
  middle,
  auto_annotate,

  artemisColors,
  
  trim,
  reverse
)
S3method(c, dna_seg)
S3method(range, dna_seg)
S3method(trim, dna_seg)
S3method(reverse, dna_seg)

S3method(range, comparison)
S3method(trim, comparison)
S3method(reverse, comparison)

S3method(range, annotation)
S3method(trim, annotation)

S3method(trim, seg_plot)

S3method(trim, default)
S3method(reverse, default)

################################
# Annotation class and methods
################################
# annotation is a set of text and one or two positions for each
# if one, other must be NA
annotation <- function(x1, x2=NA, text, rot=0, col="black"){
  if (missing(x1) | missing(text)) stop("Args x1 and text must be provided")
  if (!is.numeric(x1)) stop("x1 must be numeric")
  if (!(all(is.na(x2)) | is.numeric(x2))) stop("x2 must be numeric or NA")
  if (!is.character(text)) stop("text must be character")
  as.annotation(data.frame(x1=x1, x2=x2, text=text, stringsAsFactors=FALSE),
                rot=rot, col=col)
}
as.annotation <- function(df, x2=NA, rot=0, col="black"){
  if (is.annotation(df)) return(df)
  if (!all(c("x1", "text") %in% names(df)))
    stop("Data frame should have at least a x1 and text column")
  # attributes x2, col and arg to all rows if not defined
  if (is.null(df$x2)) df$x2 <- x2
  if (is.null(df$color)) df$color <- col
  if (is.null(df$rot)) df$rot <- rot
  class(df) <- c("annotation", "data.frame")
  df
}
is.annotation <- function(annotation){
  inherits(annotation, "annotation")  
}
range.annotation <- function(x, ...){
  annotation <- x
  range(annotation$x1, annotation$x2, na.rm=TRUE)
}
trim.annotation <- function(x, xlim=NULL, ...){
  annotation <- x
  xlim <- as.numeric(xlim)
  if (!is.null(xlim)){
    if (!is.numeric(xlim)) stop("xlim must be numeric")
    if (length(xlim) != 2) stop("xlim must be length 2")
    # to be accepted, x1 > xlim1 and, if x2=NA, xlim1 also < xlim1 or,
    # x2 < xlim2
    annotation <- annotation[annotation$x1 >= xlim[1] &
                             ((is.na(annotation$x2) &
                               annotation$x1 <= xlim[2]) |
                              (!is.na(annotation$x2) &
                               annotation$x2 <= xlim[2])),]
  }
  annotation
}

################################################################################
# File reading functions: read mauve backbone
################################################################################
read_mauve_backbone <- function(file, ref=1, gene_type="side_blocks",
                                header=TRUE, filter_low=0,
                                common_blocks_only=TRUE, ...){
  blocks <- read.table(file, stringsAsFactors=FALSE, header=header)
  n_orgs <- ncol(blocks)/2
  n_blocks <- nrow(blocks)
  # check ref parameter
  if (!(ref %in% 1:n_orgs))
    stop("Argument ref must be a positive integer =< row number")
  # ordering from ref: 
  for (i in 1:n_blocks){
    if (blocks[i,ref*2] < 0) blocks[i,] <- -blocks[i,] 
  }
  if (any(blocks[,ref*2] < 0))
      stop("Not all rows in ref columns are positive. Contact author.")
  blocks <- blocks[do.call(order, list(blocks[,ref*2])),]
  # filter, if needed
  sizes <- matrix(NA, nrow=n_blocks, ncol=n_orgs)
  for (i in 1:n_orgs){
    sizes[,i] <- abs(blocks[,2*i]) - abs(blocks[,2*i-1])
  }
  min_sizes <- apply(sizes, 1, min, na.rm=TRUE)
  rows_to_keep <- rep(TRUE, n_blocks)
  if (common_blocks_only){
    rows_to_keep <- (min_sizes > 0)
  }
  if (is.numeric(filter_low) && filter_low > 1){
    rows_to_keep <- rows_to_keep & (min_sizes >= filter_low)
  }
  blocks <- blocks[rows_to_keep,]
  n_blocks <- nrow(blocks)

  # check that there are rows remaining
  if (nrow(blocks) < 1){
    stop("No row left in data. Check data and/or lower filter_low argument")
  }
  # prepare objects
  dna_segs <- list()
  comparisons <- list()
  # colors: rainbow starting from the beginning
  col <- rainbow(n=n_blocks)
  # run through organisms
  for (i in 1:n_orgs){
    # prepare dna_seg
    s <- blocks[,2*i-1]
    e <- blocks[,2*i]
    strand <- sign(s)
    df <- data.frame(name=paste("block", 1:n_blocks, sep="_"),
                     start=abs(s), end=abs(e), strand=strand,
                     stringsAsFactors=FALSE)
    df$col <- col
    if (all(df$strand == 0)){
      stop("One or several of the columns is composed only of 0. Check data")
    }
    dna_segs[[i]] <- as.dna_seg(df[df$strand != 0,], gene_type=gene_type, ...)
    # prepare comparison (not with i=1)
    if (i > 1){
      s0 <- blocks[,2*(i-1)-1]
      e0 <- blocks[,2*(i-1)]
      strand0 <- sign(s0)
      df <- data.frame(start1=ifelse(strand0 == 1, abs(s0), abs(e0)),
                       end1=ifelse(strand0 == 1, abs(e0), abs(s0)),
                       start2=ifelse(strand == 1, abs(s), abs(e)),
                       end2=ifelse(strand == 1, abs(e), abs(s)))
      comparison <- as.comparison(df[df$start1 != 0 & df$start2 != 0,])
      # apply red_blue color scheme
      #if(i == n_orgs) browser()
      comparison$col <- apply_color_scheme(x=NULL,
                                           direction=comparison$direction,
                                           color_scheme="red_blue")
      comparisons[[i-1]] <- comparison
    }
  }
  list(dna_segs=dna_segs, comparisons=comparisons)
}

################################
# Similarity class and methods
################################
# deprecated
## similarity <- function(start1, end1, start2, end2, col=NULL){
##   if (missing(start1) | missing(end1) | missing(start2) | missing(end2))
##     stop("Both starts and ends must be provided")
##   similarity <- list(start1=start1, end1=end1, start2=start2, end2=end2,
##                      col=col)
##   warning("Similarity is deprecated. Use 1-line comparison instead")
##   as.similarity(similarity)
## }
## as.similarity <- function(similarity){
##   # return self is already of the right class
##   if (is.similarity(similarity)) {
##     return(similarity)
##   }
##   # check for presence of arguments
##   if (any(is.null(c(similarity$start1, similarity$end1,
##                     similarity$start2, similarity$end2))))
##     stop("One start or end missing")
##   if (is.null(similarity$col)) similarity$col <- "grey"
##   # check for correct argument types
##   if (any(!is.numeric(c(similarity$start1, similarity$end1,
##                         similarity$start2, similarity$end2))))
##     stop("One start or end missing")
##   # class attribution, overriding precedent classes
##   class(similarity) <- c("similarity", "list")
##   warning("Similarity is deprecated. Use 1-line comparison instead")
##   similarity
## }
## is.similarity <- function(similarity){
##   warning("Similarity is deprecated. Use 1-line comparison instead")
##   inherits(similarity, "similarity")
## }

################################################################################
# File reading functions: read dna_seg
################################################################################
# SUPPORT FUNCTIONS FOR READ_DNA_SEG_FROM_GENBANK
# Get start and end position for a currentFeature Object
# Please note, the function makes sure that only start and end of the
# following two types are ok:
# 'XXX (numeric)..(numeric)' or 'XXX complement((numeric)..(numeric))',
# where XXX may contain a-Z, 0-9 and
# also ! " # $ % & ' ( ) * + , - . / : ; < = > ? @ [ \ ] ^ _ ` { | } ~.
get_start <- function(line){
  ifelse (length(grep("complement", line)) > 0,
          start <- as.numeric(gsub("_|[[:blank:]]|[[:alpha:]]|\\(|\\)|\\.\\..*", "",
                   grep("^[[:graph:]]+ complement\\([[:digit:]]+\\.\\.[[:digit:]]+\\)$",
                   line, value=TRUE))),
          start <- as.numeric(gsub("_|[[:blank:]]|[[:alpha:]]|\\(|\\)|\\.\\..*", "",
                   grep("^[[:graph:]]+ [[:digit:]]+\\.\\.[[:digit:]]+$", line, value=TRUE))))
  start
}
get_end <- function(line){
  ifelse(length(grep("complement", line)) > 0,
         end <-
         as.numeric(gsub("_|[[:blank:]]|[[:alpha:]]|\\(|\\)|.*\\.\\.", "",
                         grep("^[[:graph:]]+ complement\\([[:digit:]]+\\.\\.[[:digit:]]+\\)$",
                              line, value=TRUE))),
         
         end <- as.numeric(gsub("_|[[:blank:]]|[[:alpha:]]|\\(|\\)|.*\\.\\.", "",
                                grep("^[[:graph:]]+ [[:digit:]]+\\.\\.[[:digit:]]+$",
                                     line, value=TRUE))))
  end
}
# Extracts data from feature lines.
extract_data <- function(extract, cF){
  extract <- gsub(extract, "", grep(paste("^",extract,sep=""), cF, value=TRUE))
  if (length(extract) == 0) { extract <- "NA" }
  extract[1]
}
# Added for support, to run an embl file directly
read_dna_seg_from_embl <- function(file, tagsToParse=c("CDS"), ...){
  read_dna_seg_from_file(file, tagsToParse, fileType="embl", ...)
}
# Added for support, to run an genbank file directly
read_dna_seg_from_genbank <- function(file, tagsToParse=c("CDS"), ...){
  read_dna_seg_from_file(file, tagsToParse, fileType="genbank", ...)
}

# MAIN FUNCTION
# Read genes from a GenBank file
read_dna_seg_from_file <- function(file, tagsToParse=c("CDS"),
                                   fileType="detect", meta_lines=2,
                                   gene_type="auto", header=TRUE,
                                   extra_fields=NULL, ...){
  
  # Import data from file into variable
  importedData <- readLines(file)
  
  # Find file type
  TYPE <- "Unknown"
  if (fileType == "detect" || fileType == "Detect" || fileType == "DETECT") {
    if (length(grep(">", importedData[1]))) {TYPE <- "Fasta"}
    if (length(grep("^ID", importedData))) {TYPE <- "EMBL"}
    if (length(grep("^LOCUS", importedData))) {TYPE <- "Genbank"}
  }
  else if (fileType == "EMBL" || fileType == "embl" || fileType == "Embl") {
    TYPE <- "EMBL"
  }
  else if (fileType == "Genbank" || fileType == "GENBANK" || fileType == "genbank") {
    TYPE <- "Genbank"
  }
  else if (fileType == "Ptt" || fileType == "PTT" || fileType == "ptt") {
    TYPE <- "PTT"
  }
  else if (fileType == "Fasta" || fileType == "FASTA" || fileType == "fasta") {
    TYPE <- "Fasta"
  }
  if (TYPE == "Unknown") {
    stop("fileType has to be either 'detect', 'embl', 'genbank' or 'ptt'. Note if file type is .ptt, please specify this rather than using 'detect'.")
  }
  
  # If type is PTT, call already made function
  if(TYPE == "PTT") {
    dna_seg <- read_dna_seg_from_ptt(file, meta_lines, header, ...)
    return(dna_seg)
  }
  else if (TYPE == "Fasta"){
    dna_seg <- read_dna_seg_from_fasta(file)
    return(dna_seg)
  }
  
  # If type isn't PTT do everything else...
  else {
    
    # Extract and name main segments
    if(TYPE == "Genbank") {
      mainSegments <- grep("^[[:alnum:]]", importedData)
      names(mainSegments) <- gsub("*| .*", "", grep("^[[:alnum:]]",
                                                    importedData, value=TRUE))
    }
    
    # SIMPLE ERROR HANDLING
    if(TYPE == "Genbank") {
      if(length(grep("FEATURES|DEFINITION", names(mainSegments))) < 2) {
        stop("FEATURES or DEFINITION segment missing in GBK File.")
      }
      if(length(grep("LOCUS", names(mainSegments))) != 1) {
        stop("Number of LOCUS should be 1.")
      }
    }
    if(TYPE == "EMBL") {
      if(length(grep("^ID|^FH", importedData)) < 2){
        stop("ID or FH segment missing in EMBL File.")
      }
      if(length(grep("^ID", importedData)) != 1) {
        stop("Number of ID lines should be 1.")
      }
    }
    
    # Extract META data
    if(TYPE == "EMBL") {
      seg_name <- gsub("^DE[[:blank:]]+", "",
                       grep("^DE", importedData, value=T))
    }
    if(TYPE == "Genbank") {
      seg_name <- gsub("DEFINITION {1,}", "",
                       importedData[mainSegments["DEFINITION"]])
    }
    
    # Extract features only, handles whether FEATURES is the last (or not)
    # entry in GBK file
    if(TYPE == "Genbank") {
      ifelse(which(names(mainSegments) == "FEATURES") == length(mainSegments),
             dataFeatures <-
             importedData[mainSegments["FEATURES"]:
                          (length(importedData) - 1)],
             dataFeatures <-
             importedData[mainSegments["FEATURES"]:
                          (mainSegments[which(names(mainSegments) ==
                                              "FEATURES")+1] - 1)])
    }
    if(TYPE == "EMBL") {
      dataFeatures <- grep("^FT", importedData, value=T)
    }
    
    # SIMPLE ERROR HANDLING
    if(TYPE == "Genbank") {
      if(length(dataFeatures) < 2){ stop("No FEATURES in GBK file.") }
    }
    if(TYPE == "EMBL") {
      if(length(dataFeatures) < 1){ stop("No FEATURES in GBK file.") }
    }
    
    
    # Extract each start line for each feature
    if(TYPE == "Genbank") {
      startLineOfFeature <- c(1:length(dataFeatures))[- grep("^ {6,}",
                                                             dataFeatures)]
    }  
    if(TYPE == "EMBL") {
      startLineOfFeature <- grep("FT   [[:alnum:]]", dataFeatures)
    }
    startLineOfFeature <- c(startLineOfFeature, length(dataFeatures)+1)
    
    
    # Define variables for storage
    nF <- length(startLineOfFeature)-1
    name <- character()
    start <- numeric()
    end <- numeric()
    strand <- numeric()
    length <- numeric()
    pid <- character()
    gene <- character()
    synonym <- character()
    product <- character()
    color <- character()
    proteinid <- character()
    feature <- character()
    geneType <- character()
    extra <- list()
    for (tag in extra_fields){
      extra[[tag]] <- character()
    }
    
    # Loop over all features                     
    for(counter in 1:nF){
       
      # Get feature, normally 20ish lines.
      currentFeature <- (dataFeatures[ startLineOfFeature[counter] :
                                      (startLineOfFeature[counter+1]-1) ])
      
      # Clean up feature, decreases number of lines etc.
      if(TYPE == "Genbank") {
        currentFeature <-
          gsub("^ |:|\"| $", "",
               gsub("[[:blank:]]+|[[:space:]]+",
                    " ", strsplit(paste(currentFeature,
                                        collapse=""), "   /")[[1]]))
      }
      if(TYPE == "EMBL") {
        currentFeature <-
          gsub("^ |:|\"| $", "",
               gsub("[[:blank:]]+|[[:space:]]+", " ",
                    strsplit(paste(gsub("FT", "", currentFeature),
                                   collapse=""), "   /")[[1]]))
      }
      
      # If feature is of a type to parse. Default is only CDS tags.
      if(length(grep(gsub(" [[:print:]]+", "", currentFeature[1]),
                     tagsToParse)) > 0){
        # Create list with exons to parse
        tag <- gsub(" [[:graph:]]+", "", currentFeature[1])
        qualif <- gsub("[[:graph:]]+ ", "", currentFeature[1])
        exonVector <- strsplit(gsub("[[:alpha:]]|_| |\\(|\\)|", "",
                                   currentFeature[1]), ",")
        if (length(grep("complement", currentFeature[1])) > 0){
          exonVector <- paste(tag, " complement(", exonVector[[1]], ")",
                              sep="")
        }
        if (length(grep("complement", currentFeature[1])) == 0){
          exonVector <- paste(tag, " ", exonVector[[1]], sep="")
        }

        # If there are more than one exon, insert introns
        if (length(exonVector) > 1) {
          exonVector2 <- 0
          for (i in 1:(length(exonVector)-1)) {
            if (length(grep("complement", currentFeature[1])) > 0) {
              exonVector2 <-
                c(exonVector2,exonVector[i],
                  paste(tag, "_intron complement(", get_end(exonVector[i])+1,
                        "..", get_start(exonVector[i+1])-1, ")", sep=""))
            }
            if (length(grep("complement", currentFeature[1])) == 0) {
              exonVector2 <- c(exonVector2,exonVector[i],
                               paste(tag, "_intron ", get_end(exonVector[i])+1,
                                     "..", get_start(exonVector[i+1])-1,
                                     sep=""))
            }
          }
          exonVector2[length(exonVector)*2] <- exonVector[length(exonVector)]
          exonVector2 <- exonVector2[2:length(exonVector2)]
          exonVector <- exonVector2
        }
        
        # For each exon in currentFeature 
        for (currentExon in exonVector) {
          
          # Set currentExon to currentFeature line 1
          currentFeature[1] <- currentExon
        
          # SIMPLE ERROR HANDLING AND Only continue parsing if start and stop
          # is valid...
          # Extract gene name or ID AND start and end, THEN, check if it's ok.
          ifelse(length(grep("gene=", currentFeature)) > 0,
                 nameTEMP <- extract_data("gene=", currentFeature),
                 nameTEMP <- extract_data("locus_tag=", currentFeature))
          startTEMP <-get_start(currentFeature[1])
          endTEMP <- get_end(currentFeature[1])
          if(length(startTEMP) == 0 || length(endTEMP) == 0) {
            warning(paste("Start and stop position invalid for "),
                    nameTEMP, ". This entry has been excluded.", sep="") }
          
          # Continue if start and end is ok... Otherwise, skip this feature
          if (length(startTEMP) > 0 && length(endTEMP) > 0) {
            
            # Save name, start, end, length
            name <- c(name, nameTEMP)
            start <- c(start, startTEMP)
            end <- c(end, endTEMP)
            length <- c(length, (get_end(currentFeature[1]) -
                                 get_start(currentFeature[1]) + 1)/3 - 1)
            
            # Set strand to 1 or -1
            ifelse (length(grep("complement", currentFeature[1])) > 0,
                    strand <- c(strand, -1), strand <- c(strand, 1))
            
            # Extract PID
            if (TYPE == "Genbank") {
              pid <- c(pid, extract_data("db_xref=GI", currentFeature))
            }
            if (TYPE == "EMBL") {
              pidTEMP <- extract_data("db_xref=UniProtKB/Swiss-Prot",
                                      currentFeature)
              if (pidTEMP == "NA"){
                pidTEMP <- extract_data("db_xref=UniProtKB/TrEMBL",
                                        currentFeature)
              }
              pid <- c(pid, pidTEMP)
            }
            
            # Extract gene
            ifelse(length(grep("gene=", currentFeature)) > 0,
                   gene <- c(gene, extract_data("gene=", currentFeature)),
                   gene <- c(gene, "-"))
            
            # Extract synonym
            synonym <- c(synonym, extract_data("locus_tag=", currentFeature))
            
            # Extract protein ID
            proteinid <- c(proteinid, extract_data("protein_id=",
                                                   currentFeature))
            
            # Extract product
            product <- c(product, extract_data("product=", currentFeature))

            # Extract color
            color <- c(color, extract_data("(color|colour)=", currentFeature))

            # Extract extra
            for (tag in names(extra)){
              extra[[tag]] <- c(extra[[tag]],
                                extract_data(paste(tag, "=", sep=""),
                                             currentFeature))
            }
            
            # Set geneType
            if (length(grep("intron", currentFeature[1])) > 0){
              geneType <- c(geneType, "introns")
            }
            if (length(grep("intron", currentFeature[1])) == 0){
              geneType <- c(geneType, gene_type)
            }
            
            # Return tag feature info, with or without added _pseudo tag
            if (length(grep("^pseudo", currentFeature)) > 0) {
              feature <- c(feature,
                           paste(gsub(" [[:print:]]+", "",
                                      currentFeature[1]), "_pseudo", sep=""))
            }
            if (length(grep("^pseudo", currentFeature)) == 0) {
              feature <- c(feature, gsub(" [[:print:]]+", "",
                                         currentFeature[1]))
            }
            
            
          # end of parse if start and end ok
          }
          
        # End of exon loop
        }
        
      # End of CDS loop
      }
      
    # End of loop over all features
    }
    
    # SIMPLE ERROR HANDLING
    if(is.numeric(start) == FALSE) stop("Start is not numeric.")
    if(is.numeric(end) == FALSE) stop("End is not numeric.")
    if(is.numeric(length) == FALSE) stop("Length is not numeric.")
    if(is.numeric(strand) == FALSE) stop("Strand is not numeric.")
    if(is.character(pid) == FALSE) stop("PID is not character.")
    if(is.character(name) == FALSE) stop("Name is not character.")
    if(is.character(gene) == FALSE) stop("Gene is not character.")
    if(is.character(synonym) == FALSE) {
      stop("Synonym is not character.")
    }
    if(is.character(product) == FALSE) {
      stop("Product is not character.")
    }
    if(is.character(proteinid) == FALSE) {
      stop("Protein ID is not character.")
    }
    if(is.character(feature) == FALSE) {
      stop("Feature is not character.")
    }
    # Check color
    # Eventually, change Artemis colors to their RGB equivalent
    artCol <- artemisColors()
    if (length(color) > 0 && all(color %in% c("NA", artCol$n))) {
      for (i in 1:length(color)){
        if (color[i] != "NA") color[i] <- artCol$colors[artCol$n == color[i]]
      }
    }

    # If gene_type is auto, change form arrows to blocks, otherwise arrows
    if (length(grep("intron", geneType))>=1 && gene_type == "auto") {
      geneType[geneType== "auto"] <- "exons"
    }
    if (length(grep("intron", geneType))==0 && gene_type == "auto") {
      geneType[geneType== "auto"] <- "bars"
    }
    
    # Cut table to include only added features    
    table <- data.frame(name=name, start=start, end=end, strand=strand,
                        length=length, pid=pid, gene=gene, synonym=synonym,
                        product=product, proteinid=proteinid, feature=feature,
                        gene_type=geneType, stringsAsFactors=FALSE)
    if (!all(color == "NA")){
      color[color == "NA"] <- "blue"
      table$col <- color
    }
    ## Adding extra fields
    for (tag in extra_fields){
      table[[tag]] <- extra[[tag]]
    }
    # SIMPLE ERROR HANDLING
    if (dim(table)[1] == 0)
      return (NULL)
      #stop("Nothing to return in table. I.e. no features extracted.")
    
    # Go to next function
    .read_dna_seg(table, seg_name, ...)
    
  # End of not PTT
  }
  
# End of genbank to dna_seg
}
read_dna_seg_from_fasta <- function(file, ...){
  # read data
  data <- readLines(file)
  title <- data[1]
  if (length(grep("^>\\w+", title, perl=TRUE)) < 1){
    stop(paste(file, "does not seem like a valid fasta file"))
  }
  name <- unlist(strsplit(title, " ", fixed=TRUE))[1]
  name <- substr(name, 2, nchar(name))
  seq <- data[-1]
  len <- sum(nchar(seq))
  dna_seg <- as.dna_seg(data.frame(name=name, start=1, end=len, strand=1,
                                   stringsAsFactors=FALSE), ...)
  return(dna_seg)
}
# reading genes from a file. Use source=tab or ptt to specify type
read_dna_seg_from_ptt <- function(file, meta_lines=2, header=TRUE, ...){
  # reads meta info
  seg_name <- readLines(file, n=1)
  seg_name <- strsplit(seg_name, "/,|-/", fixed=TRUE)[[1]][1]
  # reads ptt table
  ptt <- read.table(file, skip=meta_lines, as.is=TRUE, header=header,
                    sep="\t", quote="")
  if (header){
    names(ptt) <- tolower(names(ptt))
  }
  else {
    names(ptt) <- c("location", "strand", "length", "pid", "gene",
                    "synonym", "code", "cog", "product")
  }
  # parse location
  location <- strsplit(ptt$location, "..", fixed=TRUE)
  start <- as.numeric(sapply(location, function(x) x[[1]]))
  end <- as.numeric(sapply(location, function(x) x[[2]]))
  # parse strand
  strand <- ptt$strand
  strand[strand=="-"] <- -1
  strand[strand=="+"] <- 1
  strand <- as.numeric(strand)
  # parse gene name from name or synonym if not present
  name <- ifelse(ptt$gene == "-", ptt$synonym, ptt$gene)
  table <- data.frame(name=name, start=start, end=end, strand=strand,
                      length=ptt$length, pid=ptt$pid, gene=ptt$gene,
                      synonym=ptt$synonym, code=ptt$code, cog=ptt$cog,
                      product=ptt$product,
                      stringsAsFactors=FALSE)
  .read_dna_seg(table, seg_name, ...)
}
read_dna_seg_from_tab <- function(file, header=TRUE, ...) {
  table <- read.table(file, as.is=TRUE, header=header, sep="\t", quote="")
  if (ncol(table) < 4) stop("Insufficent number of columns in table")
  col_names <-  c("name", "start", "end", "strand")
  names(table)[1:length(col_names)] <- col_names
  # parse name from file name by default
  seg_name <- basename(file)
  .read_dna_seg(table, seg_name, ...)
}
.read_dna_seg <- function(table, seg_name, reverse=FALSE, xlim=NULL, ...){
  # check args
  if (ncol(table) < 4) stop("Insufficent number of columns in table")
  if (nrow(table) < 1) stop("No lines in table")
  col_names <-  c("name", "start", "end", "strand")
  if (!all(col_names %in% names(table)))
    stop("Table should contain at least columns name, start, end and strand")
  # make dna_seg object, set seg_name attribute
  dna_seg <- as.dna_seg(table, ...)
  dna_seg <- trim.dna_seg(dna_seg, xlim)
  if (reverse) dna_seg <- reverse.dna_seg(dna_seg)
  attr(dna_seg, "seg_name") <- seg_name
  dna_seg
}

## For a dna_seg, auto-annotates:
## Takes gene name if possible, nothing or locus_tag for the rest.
## For operons or sequences of genes, creates spanning annotations
## Gene names have to be consecutive and end with a number or a capital letter
auto_annotate <- function(dna_seg, locus_tag_pattern=NULL, names=dna_seg$gene,
                          keep_genes_only=TRUE, ...){
  nr <- nrow(dna_seg)
  if (is.null(names) || length(names) != nr)
    stop("names should be of the same length as dna_seg")
  idx <- rep(TRUE, nr)
  x1 <- middle(dna_seg)
  x2 <- rep(NA, nr)
  last_num <- numeric(nr)
  last_char_idx <- numeric(nr)
  prefix <- character(nr)
  for (i in 1:nr){
    ## If gene is not named
    if (names[i] == "-" || names[i] == ""){
      if (keep_genes_only){
        idx[i] <- FALSE
      } else if (!is.null(locus_tag_pattern)
                 && is.character(locus_tag_pattern)) {
        names[i] <- gsub(paste("^", locus_tag_pattern, "0+(.*)", sep=""),
                         "\\1", dna_seg$name[i], ignore.case=TRUE)
      } else {
        names[i] <- dna_seg$name[i]
      }
    } else {
      ## Else, which "type" is the name
      type <- NA
      if (length(grep("[A-Z]$", names[i])) > 0){
        last_char_idx[i] <- which(LETTERS == gsub(".*(.$)", "\\1", names[i]))
        prefix[i] <- gsub("^(.*).$", "\\1", names[i])
      } else if (length(grep("\\d+$", names[i])) > 0){
        ln <- gsub("^\\D*(\\d+$)", "\\1", names[i])
        last_num[i] <- if (ln != names[i]) ln else 0
        prefix[i] <- gsub("^(\\D+)\\d+$", "\\1", names[i])
      }
    }
  }
  #browser()
  ## Make sure last_* are numeric
  last_num <- as.numeric(last_num)
  ## Letter series
  series_idx <- numeric(0)
  series_dir <- 0
  series_type <- "" # num or char
  for (i in 1:(nr+1)){
    break_me <- FALSE
    ## Has pref + last_num/last_char?
    if (i <= nr && nchar(prefix[i]) > 0 &&
          (last_char_idx[i] > 0 || last_num[i] > 0)){
      valid <- TRUE
      curr_type <- if (last_num[i] > 0) "num" else "char"
      ## Is series started
      if (length(series_idx) > 0){
        ## Is type + prefix compatible?
        if (prefix[i] == prefix[series_idx[1]] && curr_type == series_type){
          ## Has series direction?
          if (series_dir != 0){
            ## Can I increment?
            if ((series_type == "num" &&
                 last_num[i-1] + series_dir == last_num[i]) ||
                (series_type == "char" &&
                 last_char_idx[i-1] + series_dir == last_char_idx[i])){
              ## Increment (do nothing)
            } else {
              ## Else break
              break_me <- TRUE
            }
          } else {
            ## No direction. Can I give one?
            dif <- ifelse(series_type == "num", last_num[i] - last_num[i-1],
                          last_char_idx[i] - last_char_idx[i-1])
            if (abs(dif) == 1){
              series_dir <- dif
            } else {
              ## Else break
              break_me <- TRUE
            }
          }
        } else {
          ## Type + prefix not compatible
          break_me <- TRUE
        }
      } else {
        ## Series not started, start it
        series_type <- if (last_num[i] > 0) "num" else "char"
      }
    } else {
      ## No pref + last_num/last_char
      break_me <- TRUE
      valid <- FALSE
    }
    ## Break or increment series
    if (break_me || i > nr){
      if (length(series_idx) > 1){
        ## Record series
        idx[series_idx[2:length(series_idx)]] <- FALSE
        x1[series_idx[1]] <- dna_seg$start[series_idx[1]]
        x2[series_idx[1]] <- dna_seg$end[series_idx[length(series_idx)]]
        suffices <- if(series_type == "num") {
          last_num[series_idx]
        } else {
          LETTERS[last_char_idx[series_idx]]
        }
        names[series_idx[1]] <- paste(prefix[series_idx[1]], suffices[1], "-",
                                      suffices[length(suffices)], sep="")
      } else {
        ## No series to record, thus do nothing
      } 
      ## Reset series
      series_dir <- 0
      ## Start a new one at once if valid
      if (valid){
        series_idx <- i 
        series_type <- curr_type
      } else {
        series_idx <- numeric(0)
        series_type <- ""
      }
    } else {
      ## Else increment
      series_idx <- c(series_idx, i)
    }
  }
  annots <- annotation(x1=x1, x2=x2, text=names, ...)
  annots[idx,]
}

################################################################################
# Plot helpers
################################################################################
# calculate arrow coordinates from gene coordinates
arrow_coord <- function(x1, x2, y=0.5, strand=NULL, width=1, head_len=100){
  # take care of strand, to get x1 as bottom and x2 as tip of arrow
  if (!is.null(strand) && strand == -1){
    x_temp <- x2
    x2 <- x1
    x1 <- x_temp
  }
  w2 <- width/4
  # if the head of the arrow is larger than half of the gene, reduce to half
  if (head_len > abs(x1-x2)/2){
    head_len <- abs(x1-x2)/2
  }
  # calculate xi, x "internal"
  if (x2 > x1){
    xi <- x2-head_len
  } else {
    xi <- x2+head_len
  }
  list(x=c(x1,   xi,   xi,     x2, xi,     xi,   x1),
       y=c(y-w2, y-w2, y-w2*2, y,  y+w2*2, y+w2, y+w2)
       )
}
# coords for a block
block_coord <- function(start, end, strand, y=0.5){
  x <- c(rep(start, 2), rep(end, 2))
  y <- c(y, y + strand/2, y + strand/2, y)
  list(x=x, y=y)
}

# exon coord
exon_coord <- function(start, end, strand){
  x <- c(rep(start, 2), rep(end, 2))
  if (strand == 0 ){ y <- c(0.2, 0.8, 0.8, 0.2) }
  if (strand == 1 ){ y <- c(0.5, 0.8, 0.8, 0.5) }
  if (strand == -1 ){ y <- c(0.2, 0.5, 0.5, 0.2) }
  list(x=x, y=y)
}

# coords for a zone annotation
bracket_coord <- function(start, end, y=0, w=0.1){
  x <- c(rep(start, 2), rep(end, 2))
  y <- c(y, rep(y+w, 2), y)
  list(x=x, y=y)
}
# axis coords
yaxis_coords <- function(at, x0=0, x1=0.5){
  n <- length(at)
  list(x0 = c(rep(x0, n), x1),
       x1 = rep(x1, n+1),
       y0 = c(at, at[1]),
       y1 = c(at, at[n]))
}
# human readable coordinates
human_nt <- function(nt, signif=FALSE){
  tag <- "nt"
  mult <- 1
  med <- median(nt)
  if (med >= 1e9){
    nt <- nt/1e9
    tag <- "Gb"
    mult <- 1e9
  } else if (med >= 1e6){
    nt <- nt/1e6
    tag <- "Mb"
    mult <- 1e6
  } else if (med >= 1e3){
    nt <- nt/1e3
    tag <- "kb"
    mult <- 1e3
  }
  if (signif) nt <- signif(nt, signif)
  list(n=nt, tag=tag, mult=mult, text=paste(nt, tag))
}
# calculate comparison coordinates
calc_comp_coor <- function(gap, xlim, comp, side){
  if (length(gap) != nrow(xlim))
    stop("gap should have the same length as xlim")
  if (side < 1 || side > 2) stop("side should be 1 or 2")
  # x is the moving cursor
  x <- 0
  old_start <- if (side==1) comp$start1 else comp$start2 
  old_end <- if (side==1) comp$end1 else comp$end2
  start <- old_start
  end <- old_end
  for (i in 1:nrow(xlim)){
    # increment by the gap length
    x <- x + gap[i]
    # select comps
    idx <- old_start >= xlim$x0[i] & old_end <= xlim$x1[i]
    # re-number by substracting the xlim and adding x
    if (xlim$strand[i] == 1){
      start[idx] <- old_start[idx] - xlim$x0[i] + x
      end[idx] <- old_end[idx] - xlim$x0[i] + x
    } else {
      start[idx] <- xlim$x1[i] - old_start[idx] + x
      end[idx] <- xlim$x1[i] - old_end[idx] + x
    }
    # increment x by the length of the segment
    x <- x + xlim$length[i]
  }
  # reattribute start and stop
  if (side==1) comp$start1 <- start else comp$start2 <- start
  if (side==1) comp$end1 <- end else comp$end2 <- end
  # return the modified comp
  comp
}
middle <- function(dna_seg){
  if (!is.dna_seg(dna_seg)) stop("argument should be a dna_seg object")
  apply(dna_seg[,c("start", "end")], 1, mean)
}
## Emulate artemis colors
## 0  white          (RGB values: 255 255 255)
## 1  dark grey      (RGB values: 100 100 100)
## 2  red            (RGB values: 255   0   0)
## 3  green          (RGB values:   0 255   0)
## 4  blue           (RGB values:   0   0 255)
## 5  cyan           (RGB values:   0 255 255)
## 6  magenta        (RGB values: 255   0 255)
## 7  yellow         (RGB values: 255 255   0)
## 8  pale green     (RGB values: 152 251 152)
## 9  light sky blue (RGB values: 135 206 250)
## 10 orange         (RGB values: 255 165   0)
## 11 brown          (RGB values: 200 150 100)
## 12 pale pink      (RGB values: 255 200 200)
## 13 light grey     (RGB values: 170 170 170)
## 14 black          (RGB values:   0   0   0)
## 15 mid red:       (RGB values: 255  63  63)
## 16 light red      (RGB values: 255 127 127)
## 17 pink           (RGB values: 255 191 191)
artemisColors <- function(){
  names <- c("white", "dark grey", "red", "green",
             "blue", "cyan", "magenta", "yellow", "pale green",
             "light sky blue", "orange", "brown", "pale pink",
             "light grey", "black", "mid red", "light red", "pink")
  numbers <- 0:(length(names)-1)
  r <- c(255, 100, 255, 0, 0, 0, 255, 255, 152, 135, 255, 200, 255,
         170, 0, 255, 255, 255)
  g <- c(255, 100, 0, 255, 0, 255, 0, 255, 251, 206, 165, 150, 200,
         170, 0, 63, 127, 191)
  b <- c(255, 100, 0, 0, 255, 255, 255, 0, 152, 250, 0, 100, 200,
         170, 0, 63, 127, 191)
  colors <- rgb(r, g, b, maxColorValue=255)
  data.frame(n=numbers, names=names, colors=colors, r=r, g=g, b=b,
             stringsAsFactors=FALSE)
}

################################################################################
# Minimize comparison sizes
################################################################################
# from a set of comparisons and lengths, determine the best possible
# arrangement of maps
minimize_comps <- function(comparisons, xlims, lengths, prel_offsets,
                           fixed_gap_length=FALSE){
  # function to minimize. Calculates the mean of the absolute differences
  # between starts and ends for direct hits only
  mean_w_fixed_gaps <- function(first_offset, fixed_offsets, ...){
    mean_w_gaps(c(first_offset, fixed_offsets), ...)
  }
  mean_w_gaps <- function(offsets, offsets_ref, xlim, xlim_ref, comp, side_ref){
    if (nrow(comp) == 0) return(0)
    side_test <- if (side_ref == 2) 1 else 2
    # recalc for ref side
    comp <- calc_comp_coor(gap=offsets_ref, xlim=xlim_ref, comp=comp,
                           side=side_ref)
    # test
    comp <- calc_comp_coor(gap=offsets, xlim=xlim, comp=comp, side=side_test)
    direction <- sign(comp$end1-comp$start1)*sign(comp$end2-comp$start2)
    dists <- c(abs(comp$start1-comp$start2),
             abs(comp$end1-comp$end2))#[direction > 0]
    lengths <- abs(comp$end1-comp$start1) + abs(comp$end2-comp$start2)
    lengths[lengths==0] <- 1
    weighted.mean(dists, c(lengths, lengths))
  }
  
  n_org <- length(lengths)
  offsets <- prel_offsets
  if (length(comparisons) < 1) return(offsets)
  idx_ref <- which.max(lengths)
  max_len <- max(lengths)

  # go up from ref
  if (idx_ref > 1){
    # comp i is between org i and i+1
    for (i in (idx_ref-1):1){
      # optimise
      # if fixed_gap_length, optimse only on the first gap
      if (fixed_gap_length && length(offsets[[i]]) > 1){
        fixed_offsets <- offsets[[i]][2:length(offsets[[i]])]
        opt <- optim(par=offsets[[i]][1], fn=mean_w_fixed_gaps,
                     method="L-BFGS-B",
                     lower=offsets[[i]][1],
                     fixed_offsets=fixed_offsets, offsets_ref=offsets[[i+1]],
                     xlim=xlims[[i]], xlim_ref=xlims[[i+1]],
                     comp=comparisons[[i]], side_ref=2)
        offsets[[i]][1] <- opt$par
      }
      # else optimise on all offsets
      else {
        opt <- optim(par=offsets[[i]], fn=mean_w_gaps,
                     method="L-BFGS-B",
                     lower=offsets[[i]],
                     offsets_ref=offsets[[i+1]],
                     xlim=xlims[[i]], xlim_ref=xlims[[i+1]],
                     comp=comparisons[[i]], side_ref=2)
        offsets[[i]] <- opt$par
      }
    }
  }
  # go down
  if (idx_ref < n_org){
    for (i in idx_ref:(n_org-1)){
      # optimise
      # if fixed_gap_length, optimse only on the first gap
      if (fixed_gap_length && length(offsets[[i]]) > 1){
        fixed_offsets <- offsets[[i]][2:length(offsets[[i]])]
        opt <- optim(par=offsets[[i+1]][1], fn=mean_w_fixed_gaps,
                     method="L-BFGS-B",
                     lower=offsets[[i+1]][1],
                     fixed_offsets=fixed_offsets, offsets_ref=offsets[[i]],
                     xlim=xlims[[i+1]], xlim_ref=xlims[[i]],
                     comp=comparisons[[i]], side_ref=1)
        offsets[[i+1]][1] <- opt$par
      }
      # else optimise on all offsets
      else {
        opt <- optim(par=offsets[[i+1]], fn=mean_w_gaps,
                     method="L-BFGS-B",
                     lower=offsets[[i+1]],
                     offsets_ref=offsets[[i]],
                     xlim=xlims[[i+1]], xlim_ref=xlims[[i]],
                     comp=comparisons[[i]], side_ref=1)
        offsets[[i+1]] <- opt$par
      }
    }
  }
  offsets
}

################################################################################
# Grobs
################################################################################
# create gene grobs
gene_grob <- function(gene, head_len=200, i=0, ...){
  if (!is.dna_seg(gene)) stop("A dna_seg object is required")
  if (nrow(gene) > 1) stop ("gene must be single-row")
  mid <- (gene$start + gene$end)/2
  name <- paste("seg.", i, ".", gene$name, sep="")
  color <- gene$col
  fill <- gene$fill
  if (is.null(fill)) fill <- color
  if (is.null(color)) color <- fill
  # arrows
  if (gene$gene_type == "arrows" || gene$gene_type == "headless_arrows"){
    if (gene$gene_type == "arrows"){
      arrow <- arrow_coord(x1=gene$start, x2=gene$end,
                           y=0.5, strand=gene$strand, head_len=head_len)
    }
    else {
      arrow <- block_coord(gene$start, gene$end, strand=1, y=0.25)
    }
    grob <- polygonGrob(arrow$x, arrow$y, name=name,
                        gp=gpar(col=color, fill=fill, lty=gene$lty,
                                lwd=gene$lwd),
                        default.units="native")
  }
  # blocks
  else if (gene$gene_type == "blocks" || gene$gene_type == "side_blocks"){
    if (gene$gene_type == "side_blocks"){
      block <- block_coord(gene$start, gene$end, strand=gene$strand, y=0.5)
    }
    else {
      block <- block_coord(gene$start, gene$end, strand=2, y=0)
    }
    grob <- polygonGrob(block$x, block$y, name=name,
                        gp=gpar(col=color, fill=fill, lty=gene$lty,
                                lwd=gene$lwd),
                        default.units="native")  
  }
  # lines
  else if (gene$gene_type == "lines" || gene$gene_type == "side_lines"){
    x <- c(gene$start, gene$end)
    if (gene$gene_type == "side_lines") {
      y <- gene$strand/4 + 0.5
    }
    else {
      y <- 0.5
    }
    grob <- segmentsGrob(x0=gene$start, y0=y, x1=gene$end, y1=y, name=name,
                         gp=gpar(col=color, fill=fill, lwd=gene$lwd,
                                 lty=gene$lty),
                         default.units="native")
  }
  # exons
  else if (gene$gene_type == "exons" || gene$gene_type == "side_exons"){
    if (gene$gene_type == "side_exons"){
      block <- exon_coord(gene$start, gene$end, gene$strand)
    }
    else {
      block <- exon_coord(gene$start, gene$end, 0)
    }
    grob <- polygonGrob(block$x, block$y, name=name,
                        gp=gpar(fill=fill, col=color, lty=gene$lty,
                                lwd=gene$lwd),
                        default.units="native")  
  }
  
  # bars
  else if (gene$gene_type == "bars" || gene$gene_type == "side_bars") {
    if (gene$gene_type == "side_bars"){
      y0 <- 0.5; y1 <- 0.5+gene$strand/2
    }
    else {
      y0 <- 0; y1 <- 1
    }
    grob <- segmentsGrob(x0=mid, y0=y0, x1=mid, y1=y1, name=name,
                         gp=gpar(col=color, fill=fill,lwd=gene$lwd,
                                 lty=gene$lty),
                         default.units="native")
  }

  # introns
  else if (gene$gene_type == "introns") {
    intron <- list(x0=c(gene$start, mid), x1=c(mid, gene$end),
                   y0=c(0.3, 0.5)*gene$strand + 0.5,
                   y1=c(0.5, 0.3)*gene$strand + 0.5)
    grob <- segmentsGrob(x0=intron$x0, y0=intron$y0,
                         x1=intron$x1, y1=intron$y1, name=name,
                         gp=gpar(lty=gene$lty, lwd=gene$lwd),
                         default.units="native")
  }
  # points
  else if (gene$gene_type == "points" || gene$gene_type == "side_points") {
    if (gene$gene_type == "side_points"){
      y <- 0.5+gene$strand/4
    }
    else {
      y <- 0.5
    }
    ## For pch with filled symbols (21-25), "col" refers to fill
    ## whereas col becomes
    grob <- pointsGrob(x=mid, y=y, name=name,
                       pch=gene$pch, size=unit(gene$cex/2, "char"),
                       gp=gpar(col=color, fill=fill),
                       default.units="native")
  }
  # text
  else if (gene$gene_type == "text" || gene$gene_type == "side_text") {
    if (gene$gene_type == "side_text"){
      just <- c("centre", c("top", "bottom")[gene$strand/2 + 1.5])
    }
    else {
      just <- c("centre", "centre")
    }
    grob <- textGrob(label=gene$name, x=mid, y=0.5, name=name,
                     just=just, gp=gpar(col=color, cex=gene$cex),
                     default.units="native")
  }
  else {
    grob <- try(do.call(gene$gene_type, list(gene, ...)), silent=FALSE)
    if (!(is.grob(grob) || all(sapply(grob, is.grob))))
      stop(paste(gene$gene_type, "is an invalid gene_type or",
                 "does not return a grob"))
  }
  grob
}
# create dna_seg grobs
dna_seg_grob <- function(dna_seg, ...){
  if(!is.dna_seg(dna_seg)) stop("A dna_seg object is required")
  grob_list <- gList()
  if (nrow(dna_seg) < 1) return(grob_list)
  for (i in 1:nrow(dna_seg)){
    gene <- as.dna_seg(dna_seg[i,])
    grob_list[[i]] <- gene_grob(gene, ...)
  }
  grob_list
}
# create similarity grobs
similarity_grob <- function(similarity, i){
  if (!is.comparison(similarity)) stop("A comparison object is required")  
  if (nrow(similarity) > 1) stop ("gene must be single-row")
  if (is.null(similarity$col)) similarity$col <- grey(0.5)
  x1 <- c(similarity$start1, similarity$end1)
  x2 <- c(similarity$end2, similarity$start2)
  polygonGrob(x=c(x1, x2), y=c(1, 1, 0, 0),
              name=paste("comp.", i, ".",
                similarity$start1, "-", similarity$end1, "_",
                similarity$start2, "-", similarity$end2, sep=""),
              gp=gpar(fill=similarity$col, col=similarity$col, lwd=0.1),
              default.units="native")
}
# create comparisons grobs
comparison_grob <- function(comparison, ...){
  if (!is.comparison(comparison)) stop("A comparison object is required")
  grob_list <- gList()
  if (nrow(comparison) < 1) return(grob_list)
  # go in the reverse order to plot strongest comparison last
  for (i in 1:nrow(comparison)){
    grob_list[[i]] <- similarity_grob(comparison[i,], ...)
  }
  grob_list
}
# create annot grob
label_grob <- function(label, cex=0.8){
  y <- 0
  w <- 0.1
  if (!is.annotation(label)) stop("An annotation object is required")
  if (nrow(label) > 1) stop("A single-line annotation is required")
  grob_list <- gList()
  # range
  if (!is.na(label$x2)){
    bracket_coord <- bracket_coord(label$x1, label$x2, y=y, w=w)
    grob_list[[2]] <- linesGrob(x=bracket_coord$x, y=bracket_coord$y,
                                name=paste("annot", "line",
                                  gsub(" ", "_", label$text), sep="."),
                                default.units="native",
                                gp=gpar(col=label$col))
    x <- mean(c(label$x1, label$x2))
    w <- w*2
  } else {
    x <- label$x1
  }
  if (label$rot == 0){
    just <- c(0.5, 0)
  } else {
    just <- c(-0.1, 0.5)
  }
  grob_list[[1]] <- textGrob(label$text, x=x, y=y+w, just=just,
                             name=paste("annot", "label",
                               gsub(" ", "_", label$text), sep="."),
                             rot=label$rot,
                             default.units="native",
                             gp=gpar(col=label$col, cex=cex))
  grob_list
}
# create annotation grob
annotation_grob <- function(annotation, ...){
  if (!is.annotation(annotation)) stop("An annotation object is required")
  grob_list <- gList()
  if (nrow(annotation) < 1) return(grob_list)
  for (i in 1:nrow(annotation)){
    label <- as.annotation(annotation[i,])
    grob_list[[i]] <- label_grob(label, ...)
  }
  grob_list
}
# create tree grob
dna_seg_label_grob <- function(labels, cex, col){
  n_label <- length(labels)
  y <- seq(1, 0, len=n_label)
  labelGrobs <- gList()
  for (i in 1:n_label) {
    labelGrobs[[i]] <-
      textGrob(x=0, y=y[i], name=paste("label", i, sep="."),
               label=labels[i], just="left", gp=gpar(cex=cex, col=col[i]),
               default.units="native")
  }
  width <- unit(1, "grobwidth", labelGrobs[[which.max(nchar(labels))]])
  labelTree <- gTree(children=labelGrobs,
                     vp=viewport(xscale=c(0, 1), yscale=c(0, 1),
                       width=width, name="labels"), name="labelsTree")
  list(grob=labelTree, width=width)
}
# create scale grob
scale_grob <- function(max_length){
  rng <- diff(pretty(c(0, max_length), n=8))[1]
  gList(segmentsGrob(x0=max_length, y0=0, x1=max_length-rng, y1=0,
                     name="scale.lines", default.units="native"),
        textGrob(label=human_nt(rng)$text, x=max_length-rng/2, y=0.5,
                 name="scale.text", default.units="native")
        )
}
# create dna_seg scale grob
dna_seg_scale_grob <- function(range, cex=0.6, unit, i, j){
  range <- as.numeric(range)
  if (length(range) != 2 && !is.numeric(range))
    stop("range must be numeric and length 2")
  x0 <- ceiling(range[1]/unit)*unit
  x1 <- floor(range[2]/unit)*unit
  if (x1 < x0) x1 <- x0
  ticks <- seq(x0, x1, by=unit)
  labels <- human_nt(ticks)
  gList(segmentsGrob(x0=ticks, x1=ticks, y0=0, y1=1,
                     gp=gpar(col=grey(0.3)),
                     name=paste("dna_seg_scale", i, j, "lines", sep="."),
                     default.units="native"),
        textGrob(labels$text, x=ticks, y=0.5, hjust=-0.05,
                 gp=gpar(col=grey(0.3), cex=cex),
                 name=paste("dna_seg_scale", i, j, "labels", sep="."),
                 default.units="native")
        )
}
# create gap grob
gap_grob <- function(w, m, i, j){
  segmentsGrob(x0=c(m-w/4, m-w/8),
               x1=c(m+w/8, m+w/4),
               y0=c(0.2,0.2),
               y1=c(0.8,0.8),
               gp=gpar(col=grey(0.3)),
               default.units="native",
               name=paste("gap", i, j, sep="."))
}
## yaxis grob
yaxis_grob <- function(ylim=c(0, 1), cex=0.6, n=3, i){
  at <- pretty(ylim, n=n)
  at <- at[at >= ylim[1] & at <= ylim[2]]
  coords <- yaxis_coords(at, x0=0, x1=0.5*cex)
  gList(segmentsGrob(x0=unit(coords$x0, "lines"), x1=unit(coords$x1, "lines"),
                     y0=unit(coords$y0, "native"), y1=unit(coords$y1, "native"),
                     name=paste("yaxis.segments", i, sep="."),
                     default.units="native"),
        textGrob(at, x=unit(1, "lines"), y=unit(at, "native"),
                 gp=gpar(cex=cex), just=c("left", "centre"),
                 name=paste("yaxis.labels", i, sep="."),
                 default.units="native")
        )
}

################################################################################
# Color schemes
################################################################################
# apply color scheme to a numeric vector
# decreasing: relationship gets stronger with decreasing values?
# TRUE for, typically, e-values, gaps, mism, ... 190
# FALSE for bit scores, per_id, aln_length
apply_color_scheme <- function(x, direction=NULL, color_scheme="grey",
                               decreasing=FALSE, rng=NULL, transparency=0.5){
  # check arguments
  # if length is 0, return length 0
  if (!is.null(direction) && length(direction) == 0) return (character(0))
  # if x is null and direction is not, get x to 1s (mainly for blue/red)
  if (is.null(x) && !is.null(direction)) {
    x <- rep(1, length(direction))
  }
  if (!is.numeric(x)) stop("Color column is not numeric")
  if (is.null(rng)) rng <- range(x)
  if (!(is.logical(transparency) && !(transparency))
      && !is.numeric(transparency))
    stop ("transparency should be FALSE or numeric")
  col <- rep(grey(0.5), length(x))
  # red blue
  if (any(color_scheme %in% c("red_blue", "blue_red"))){
    if (is.null(direction)) direction <- rep(1, length(x))
    blues <- c("#F7FBFF", "#DEEBF7", "#C6DBEF", "#9ECAE1", "#6BAED6",
               "#4292C6", "#2171B5", "#08519C", "#08306B")
    reds  <- c("#FFF5F0", "#FEE0D2", "#FCBBA1", "#FC9272", "#FB6A4A",
               "#EF3B2C", "#CB181D", "#A50F15", "#67000D")
    # case: only one value:
    if (diff(rng) == 0){
      level <- rep(9, length(x))
    } else { # case: several values
      level <- round(((x-rng[1])/diff(rng))*8+1)
    }
    if (decreasing) level <- -level + 10
    col[direction==1] <- reds[level[direction==1]] 
    col[direction==-1] <- blues[level[direction==-1]]
  }
  # grey: between 0.25 and 0.75
  else if (any(color_scheme %in% c("grey", "gray", "grays", "greys")))
    # case: only one value:
    if (diff(rng) == 0){
      col <- rep(grey(0.5), length(x))
    } else {
      level <- 0.75-((x-rng[1])/diff(rng))*0.5
      if (decreasing) level <- -level+1
      col <- grey(level)
    }
  else {
    stop("Color scheme name invalid, choose between red_blue or grey")
  }
  if (transparency && transparency != 1){
    # Convert ratio into hexadecimal
    if (transparency > 1 || transparency < 0)
      stop("transparency should be between 0 and 1")
    tpc <- floor(transparency*256)
    tpc <- sprintf("%X", tpc)
    if (nchar(tpc) == 1) tpc <- paste("0", tpc, sep="")
    col <- paste(col, tpc, sep="")
  }
  col
}

################################
## Plot class and methods
################################
## plot is a list that contains at least three named elements:
## - a function (func)
## - a list of arguments (arguments) to that function
## - an 'xargs' argument designating which variable(s) give x coordinates
## - a  'yargs' argument designating which variable(y) give y coordinates
## - a ylim argument, giving the boundaries on the y axis. Derived
##   automatically from yargs if not set.
seg_plot <- function(func, args=NULL, xargs=c("x", "x0", "x1", "x2", "v"),
                     yargs=c("y", "y0", "y1", "y2", "h"),
                     ylim=NULL){
  as.seg_plot(list(func=func, args=args, xargs=xargs, yargs=yargs, ylim=ylim))
}
as.seg_plot <- function(seg_plot){
  ## Check  that args is a list and that it has func and args
  if (!is.list(seg_plot))
    stop("list argument should be a list object")
  if (!all(c("func", "args") %in% names(seg_plot)))
    stop("list should have elements func, args")
  ## Check that function is a function
  if (!is.function(seg_plot$func))
    stop("func argument should be a function")
  ## Check args
  if (!(is.null(seg_plot$args) || is.list(seg_plot$args)))
    stop("args should be NULL or a list")
  ## Check that by default, the default.units are "native" and not "npc"
  if (is.null(seg_plot$args$default.units))
    seg_plot$args$default.units = "native"
  ## Check xargs and yargs
  if (is.null(seg_plot$xargs)) {
    seg_plot$xargs <- c("x", "x0", "x1", "x2", "v")
  } else if (!is.character(seg_plot$xargs)){
    stop("xargs should be NULL or a character")
  }
  if (is.null(seg_plot$yargs)) {
    seg_plot$yargs <- c("y", "y0", "y1", "y2", "h")
  } else if (!is.character(seg_plot$yargs)){
    stop("yargs should be NULL or a character")
  }
  ## Check ylim
  if (!is.null(seg_plot$ylim) && !is.numeric(seg_plot$ylim))
    stop("ylim should be NULL or numeric")
  ## Check that columns designated by xargs and yargs are numeric or null
  if (!all(sapply(c(seg_plot$xargs, seg_plot$yargs), function(x)
                  is.numeric(seg_plot$args[[x]]) ||
                  is.null(seg_plot$args[[x]]))))
    stop("All arguments designated by xargs & yargs should be numeric or null")
  ## Set class and return
  class(seg_plot) <- c("seg_plot", "list")
  seg_plot
}
is.seg_plot <- function(seg_plot){
  inherits(seg_plot, "seg_plot")
}
## Tries to trim correctly the seg_plot objects...
## Made difficult because of the complexity of the objects
trim.seg_plot <- function(x, xlim=NULL, ...){
  ## Check that we are in the right class
  if (!is.seg_plot(x))
    stop("x should be a seg_plot object.")
  xlim <- as.numeric(xlim)
  ## If xlim is null, return whole object. Else, trim
  if (!is.null(xlim)){
    if (!is.numeric(xlim)) stop("xlim must be numeric")
    if (length(xlim) != 2) stop("xlim must be length 2")
    mainargs <- names(x$args)
    gpargs <- NULL
    ## Define arguments to filter
    if ("gp" %in% mainargs && inherits(x$args$gp, "gpar")){
      mainargs <- mainargs[mainargs != "gp"]
      gpargs <- names(x$args$gp)
    }
    ##std_xargs <- c("x", "x0", "x1", "x2", "v")
    xargs <- mainargs[mainargs %in% x$xargs]
    if (length(xargs) > 0){
      ## Check that all xargs have the same length
      nrows <- length(x$args[[xargs[1]]])
      if (length(xargs) > 1){
        sapply(xargs[2:length(xargs)], function(sp) {
          if(length(x$args[[sp]]) != nrows)
            stop(paste("Argument", sp, "has a different length as argument",
                       xargs[1]))
        })
      }
      ## Get the correct indexes
      idx <- rep(TRUE, nrows)
      for (i in 1:length(xargs)){
        idx <- idx & x$args[[xargs[i]]] >=
          xlim[1] & x$args[[xargs[i]]] <= xlim[2] 
      }
      ## Filter everything...
      for (arg in mainargs){
        if (length(x$args[[arg]]) == nrows)
          x$args[[arg]] <- x$args[[arg]][idx]
      }
      if (!is.null(gpargs)){
        for (arg in gpargs){
          if (length(x$args$gp[[arg]]) == nrows)
            x$args$gp[[arg]] <- x$args$gp[[arg]][idx]
        }
      }
    }
  }
  x
}
## Get a range from a seg_plot object, allowing margin on each side
nice_ylim.seg_plot <- function(seg_plot, margin=0.05){
  if (!is.seg_plot(seg_plot))
    stop("seg_plot should be a seg_plot object")
  if (!is.numeric(margin))
    stop("margin should be numeric")
  if (length(seg_plot$yargs %in% names(seg_plot$args)) < 1)
    stop("No arguments in seg_plot$args matches seg_plot$yargs")
  flock <- unlist(seg_plot$args[seg_plot$yargs])
  rng <- range(flock)
  span <- diff(rng)
  c(rng[1]-span*margin, rng[2]+span*margin)
}

################################
# gene class and methods
################################
# deprecated
## gene <- function(name, start, end, strand, col=NULL, lwd=1, lty=1, pch=8, cex=1,
##                  gene_type="arrows"){
##   if (missing(name)) stop("Name must be provided")
##   if (missing(start)) stop ("Start must be provided")
##   if (missing(end)) stop ("End must be provided")
##   if (missing(strand)) stop ("Strand must be provided")
##   gene <- data.frame(name=name, start=start, end=end, strand=strand, col=col,
##                      lwd=lwd, lty=lty, pch=pch, cex=cex, gene_type=gene_type,
##                      stringsAsFactors=FALSE)
##   as.gene(gene)
## }
## as.gene <- function(gene, col="blue", lwd=1, lty=1, pch=8, cex=1,
##                     gene_type="arrows"){
##   # return self is already of the right class
##   if (is.gene(gene)) {
##     return(gene)
##   }
##   # check for presence of arguments
##   if (is.null(gene$name)) stop("No gene name")
##   if (is.null(gene$start)) stop("No start in gene")
##   if (is.null(gene$end)) stop("No end in gene")
##   if (is.null(gene$strand)) stop("No strand in gene")
##   if (is.null(gene$col)) gene$col <- col
##   if (is.null(gene$lwd)) gene$lwd <- lwd
##   if (is.null(gene$lty)) gene$lty <- lty
##   if (is.null(gene$pch)) gene$pch <- pch
##   if (is.null(gene$cex)) gene$cex <- cex
##   # gene_type: not given in gene
##   if (is.null(gene$gene_type)){
##     if (!is.character(gene_type) || !(gene_type %in% gene_types()))
##       stop(paste("gene_type muste be one of:",
##                  paste(gene_types(), collapse=", ")))
##     if (gene_type == "auto") gene_type <- "arrows"
##     gene$gene_type <- gene_type
##   }
##   # given in gene
##   else {
##     if (!is.character(gene$gene_type) || !all(gene_type %in% gene_types()))
##       stop(paste("gene_type muste be one of:",
##                  paste(gene_types(), collapse=", ")))   
##   }
##   # check for correct argument types
##   if (!is.character(gene$name)) stop("Non-character name")
##   if (!(is.numeric(gene$start) && is.numeric(gene$end)))
##     stop("Non-numeric start or end")
##   if (!all(is.numeric(c(gene$lwd, gene$lty, gene$pch, gene$cex))))
##     stop("lwd, lty, pch and cex must be numeric")
##   # care about strand
##   if (gene$strand == 1 | gene$strand == "+") {
##     gene$strand <- 1
##   } else if (gene$strand == -1 | gene$strand == "-") {
##     gene$strand <- -1
##   } else {
##     stop("Invalid gene strand: should be 1, -1, + or -")
##   }
##   # care about start/end
##   if (gene$end < gene$start) stop("End smaller than start. Use strand instead")
##   # no color check for the moment
##   # convert to df
##   gene <- as.data.frame(gene, stringsAsFactors=FALSE)
##   # class attribution, no check for other at the moment
##   class(gene) <- c("gene", "data.frame")
##   gene
## }
## is.gene <- function(gene){ 
##   inherits(gene, "gene")
## }
## # calculates x range
## rang.gene <- function(gene){
##   range(c(gene$start, gene$end))
## }
## rang.list <- rang.gene

################################################################################
# S3 method: generic functions
################################################################################
## trim generic function
trim <- function(x, ...){
  UseMethod("trim")
}
# slightly weird: given a xlim, returns rows for which the first numeric column
# is > xlim[1] and the second numeric column is < xlim[2] 
trim.default <- function(x, xlim=NULL, ...){
  if (!is.numeric(xlim)) stop("xlim must be numeric")
  if (length(xlim) != 2) stop("xlim must be length 2")
  num_col <- which(sapply(x, is.numeric))
  if (length(num_col) < 2) stop ("x should have at least 2 numeric columns")
  x <- x[x[,num_col[1]] >= xlim[1] & x[,num_col[2]] <= xlim[2]]
  x
}

## reverse generic function
reverse <- function(x, ...){
  UseMethod("reverse")
}
# default method
reverse.default <- function(x, ...){
  num_col <- which(sapply(x, is.numeric))
  if (length(num_col) < 2) stop ("x should have at least 2 numeric columns")
  tmp <- -x[,num_col[2]]
  x[,num_col[2]] <- -x[,num_col[1]]
  x[,num_col[1]] <- tmp
  x
}

################################
# Comparison class and methods
################################
# A comparison is mostly a bunch of similarities
comparison <- function(x){
  # if data.frame
  if (is.data.frame(x)){
    return(as.comparison(x))
  } else if (is.list(x)){
    return(as.comparison(as.data.frame(x, stringsAsFactors=FALSE)))
  } else {
    stop(paste("Cannot coerce class", class(x), "to comparison"))
  }
}
as.comparison <- function(df){
  # check for class comparison, list, df
  if (is.comparison(comparison)) return(df)
  if (is.list(df) && !is.data.frame(df)) df <- comparison(df)
  if (is.data.frame(df)) {
    # check for columns
    names <- c("start1", "end1", "start2", "end2")
    if (!identical(names(df)[1:4], names))
      stop("Col names should start with start1, end1, start2 and end2")
    if (!all(sapply(df, function(x) all(!is.null(x)))))
      stop("NULL values not allowed in data.frame")
    if (!is.numeric(df$start1) | !is.numeric(df$end1) |
        !is.numeric(df$start2) | !is.numeric(df$end2)){
      stop("Starts and ends must be numeric")
    }
    # add direction column
    df$direction <- ifelse(sign(df$start1-df$end1)
                           * sign(df$start2-df$end2) > 0, 1, -1)
    # check color (character and default)
    if (is.factor(df$col)) df$col <- as.character(df$col)
  }
  else {
    stop("Unable to handle this format")
  }
  class(df) <- c("comparison", "data.frame")
  df
}
is.comparison <- function(comparison){
  inherits(comparison, "comparison")
}
# gives the range of a comparison
range.comparison <- function(x, overall=TRUE, ...){
  comparison <- x
  if (overall){
    range <- range(comparison$start1, comparison$end1,
                   comparison$start2, comparison$end2, na.rm=FALSE)
  } else {
    xlim1 <- range(comparison$start1, comparison$end1, na.rm=FALSE)
    xlim2 <- range(comparison$start2, comparison$end2, na.rm=FALSE)
    range <- data.frame(xlim1=xlim1, xlim2=xlim2)
  }
  range
}
# trim comparison given x limits
trim.comparison <- function(x, xlim1=c(-Inf, Inf), xlim2=c(-Inf, Inf), ...){
  comparison <- x
  if (!is.null(xlim1) && !is.null(xlim2)){
    if (!is.numeric(xlim1) || !is.numeric(xlim2)) stop("xlims must be numeric")
    if (length(xlim1) != 2 || length(xlim2) != 2) stop("xlims must be length 2")
    ## testing to include overlapping comps
    ## direction 1
    comparison$start1[comparison$start1 < xlim1[1] &
                      comparison$end1 > xlim1[1]] <- xlim1[1]
    comparison$end1[comparison$start1 < xlim1[2] &
                    comparison$end1 > xlim1[2]] <- xlim1[2]
    comparison$start2[comparison$start2 < xlim2[1] &
                      comparison$end2 > xlim2[1]] <- xlim2[1]
    comparison$end2[comparison$start2 < xlim2[2] &
                    comparison$end2 > xlim2[2]] <- xlim2[2]
    ## direction -1
    comparison$start1[comparison$start1 > xlim1[2] &
                      comparison$end1 < xlim1[2]] <- xlim1[2]
    comparison$end1[comparison$start1 > xlim1[1] &
                    comparison$end1 < xlim1[1]] <- xlim1[1]
    comparison$start2[comparison$start2 > xlim2[2] &
                      comparison$end2 < xlim2[2]] <- xlim2[2]
    comparison$end2[comparison$start2 > xlim2[1] &
                    comparison$end2 < xlim2[1]] <- xlim2[1]
    comparison <-
      comparison[comparison$start1 >= xlim1[1] & comparison$end1 <= xlim1[2] &
                 comparison$start2 >= xlim2[1] & comparison$end2 <= xlim2[2],]
  }
  comparison
}
# reverses a comparison. side <1 for no sides, 1 for first,
# 2 for second, >2 for bth
reverse.comparison <- function(x, side=0, ...){
  comparison <- x
  if (side > 1){
    comparison$start2 <- -comparison$start2
    comparison$end2 <- -comparison$end2
  }
  if (side == 1 || side >2){
    comparison$start1 <- -comparison$start1
    comparison$end1 <- -comparison$end1    
  }
  # if only one side changed, change direction
  if (side == 1 || side == 2){
    comparison$direction <- -comparison$direction
  }
  comparison
}

###############################################################################
# PLOT GENE MAPS                                                              #
###############################################################################
# A R framework to plot comparison of gene stretches, a la Artemis, but
# with production-like graphics, and a static interface
################################################################################
# Plotting function
################################################################################
# plot frame, makes space for tree and legend if necessary
plot_gene_map <- function(dna_segs,
                          comparisons=NULL,
                          tree=NULL,
                          tree_width=NULL, # in inches
                          tree_branch_labels_cex=NULL,
                          tree_scale=FALSE, # plot tree scale?
                          legend=NULL, # unimplemented
                          annotations=NULL,
                          annotation_height=1, # height of annot line
                          annotation_cex=0.8, # size of annotations
                          seg_plots=NULL,    # user-defined plots
                          seg_plot_height=3, # height of plots (in lines)
                          seg_plot_height_unit="lines", # unit of preceding
                          seg_plot_yaxis=3, # if non-null or non false, ticks
                          seg_plot_yaxis_cex=scale_cex,
                          xlims=NULL,
                          offsets=NULL, # regulates manually alignment of segs
                          minimum_gap_size=0.05,
                          fixed_gap_length=FALSE,
                          limit_to_longest_dna_seg=TRUE,
                          main=NULL, # main title
                          main_pos="centre", # centre, left, right
                          dna_seg_labels=NULL, # labels on dna_segs
                          dna_seg_label_cex=1, # size of these
                          dna_seg_label_col="black", # color of these
                          gene_type=NULL, # if not null, resets gene_type
                          arrow_head_len=200, # force arrow head length
                          dna_seg_line=TRUE,  # draw a line on each dna_seg
                          scale=TRUE,         # scale in the bottom right
                          dna_seg_scale=!scale, # scale on each dna_seg
                          n_scale_ticks=7,    # number of tick marks for these
                          scale_cex=0.6,      # size of text on scale
                          global_color_scheme=c("auto", "auto",
                            "blue_red", 0.5),
                          override_color_schemes=FALSE,
                          plot_new=TRUE, # FALSE to integrate on a bigger plot
                          debug = 0,
                          ...){
  #----------------------------------------------------------------------------#
  # check arguments
  #----------------------------------------------------------------------------#
  ### objects ###
  # dna_segs
  if (missing(dna_segs)) stop("Argument dna_segs must be provided")
  if (!is.list(dna_segs) || !all(sapply(dna_segs, is.dna_seg)))
    stop("Argument dna_segs must be a list of dna_seg objects")
  n_dna_segs <- length(dna_segs)
  n_rows <- 3*n_dna_segs-1
  
  # comparisons
  n_comparisons <- length(comparisons)
  if (n_comparisons > 1){
    if (!is.list(comparisons) || !all(sapply(comparisons, is.comparison)))
      stop("Argument comparisons must be a list of comparison objects")
  }
  # check that there are enough comparisons compared to dna segments
  if (n_comparisons > 0 && !(n_dna_segs - n_comparisons == 1))
    stop("Number of comparisons not correct")
  # if dna_seg is a named list, attribute names to dna_seg_labels
  if (is.null(dna_seg_labels) && !is.null(dna_segs)){
    dna_seg_labels <- names(dna_segs)
  }
  
  # check length of labels
  if (!is.null(dna_seg_labels) && !(length(dna_seg_labels) == n_dna_segs))
    stop("Argument dna_seg_labels doesn't have the same length as dna_segs")

  # check dna_seg_label colors
  if (length(dna_seg_label_col) == 1){
    dna_seg_label_col <- rep(dna_seg_label_col, n_dna_segs)
  }
  else if (!length(dna_seg_label_col) == n_dna_segs){
    stop("Length of argument dna_seg_label_col must be 1 or as dna_segs")
  }
  
  # check tree
  if (!is.null(tree)){
    if (!inherits(tree, "phylog"))
      stop("Argument tree should be of class phylog (ade4)")    
    # check correspondence between names (given by names(comparison) or
    # dna_seg_labels) and tree leaves
    if (is.null(dna_seg_labels))
      stop("If tree is given, label names should be provided via named list dna_segs or dna_seg_labels")
    # check that number of leaves corresponds to number of segs
    if (length(tree$leaves) != n_dna_segs)
      stop("Number of leaves in the tree not equal to number of dna segs")
    if (!all(dna_seg_labels %in% names(tree$leaves)))
      stop("Tree leaves not corresponding to dna_seg labels")
    # check whether nodes have added labels
    if (is.null(tree_branch_labels_cex)){
      if (length(grep("^[^I]", names(tree$nodes)))) {
        tree_branch_labels_cex <- 0.8
      } else {
        tree_branch_labels_cex <- 0
      }
    }
  }

  # check seg_plots (user-defined plots)
  if (!is.null(seg_plots)){
    # if there is only one annotation, put it on the top
    if (is.seg_plot(seg_plots)){
      s_plot <- seg_plots
      seg_plots <- c(list(s_plot), rep(list(NULL), n_dna_segs-1))
    } else if (length(seg_plots) == n_dna_segs){
      if (!all(sapply(seg_plots, function(x) is.seg_plot(x) || is.null(x))))
        stop("All elements of seg_plots should be NULL or seg_plots objects")
    } else stop ("seg_plots must be of same length as dna_segs")
    seg_plot_h <- ifelse(sapply(seg_plots, is.null), 0, seg_plot_height)
  } else {
    seg_plot_h <- rep(0, n_dna_segs)
  }
  # check seg_plot_yaxis
  if (is.null(seg_plot_yaxis) || !is.numeric(seg_plot_yaxis) ||
      is.null(seg_plots))
    seg_plot_yaxis <- 0
  
  # check annotation
  if (!is.null(annotations)){
    # if there is only one annotation, put it on the top
    if (is.annotation(annotations)){
      annot <- annotations
      annotations <- c(list(annot), rep(list(NULL), n_dna_segs-1))
    } else if (length(annotations) == n_dna_segs){
      if (!all(sapply(annotations, function(x) is.annotation(x) || is.null(x))))
        stop("All elements of annotations should be NULL or annotation objects")
    } else stop ("annotation must be of same length as dna_segs")
    annot_h <- ifelse(sapply(annotations, is.null), 0, annotation_height)
  } else {
    annot_h <- rep(0, n_dna_segs)
  }
  
  ### graphical arguments ###
  # check xlims
  if (!is.null(xlims)){
    if (!is.list(xlims))
      stop("xlims must be a list")
    if (length(xlims) != n_dna_segs)
      stop("xlims must be of the same length as dna_segs")
    if (!all(sapply(xlims, function(x) (length(x) %% 2) == 0)))
      stop("All elements of xlims should have an even number of elements")
    for (i in 1:length(xlims)){
      xlim <- xlims[[i]]
      # check numeric, replace null and inf values
      rng <- range(dna_segs[[i]])
      min <- rng[1] - 0.02*diff(rng)
      max <- rng[2] + 0.02*diff(rng)
      # replace NULL value by min-max
      if (is.null(xlim)) xlim <- c(min, max)
      # replace -Inf and Inf by min max
      xlim[xlim == -Inf] <- min
      xlim[xlim == Inf] <- max
      # check that all are numeric
      if (!is.numeric(xlim)) stop("All elements of xlims should be numeric")
      # transform to data.frame
      xlim <- data.frame(matrix(xlim, ncol=2, byrow=TRUE))
      names(xlim) <- c("x0", "x1")
      # check the strand
      xlim$strand <- ifelse(xlim$x0 < xlim$x1, 1, -1)
      # sort x0 x1
      for (j in 1:nrow(xlim)) xlim[j,1:2] <- sort(as.numeric(xlim[j,1:2]))
      xlims[[i]] <- xlim
    }
  } else {
    xlims <- list()
    for (i in 1:n_dna_segs){
      rng <- range(dna_segs[[i]])
      xlims[[i]] <- data.frame(x0=rng[1] - 0.02*diff(rng),
                               x1=rng[2] + 0.02*diff(rng),
                               strand=1)
    }
  }
 
  # check offsets
  if (!is.null(offsets) && length(offsets) != n_dna_segs){
    stop("Length of offsets not equal to number of dna_segs")
  }
  
  # check main_pos
  if (main_pos == "centre"){
    main_x <- 0.5
    main_just <- "centre"
  } else if (main_pos == "left"){
    main_x <- 0
    main_just <- "left"
  } else if (main_pos == "right"){
    main_x <- 1
    main_just <- "right"
  } else {
    stop("main_pos should be one of centre, left, right")
  }

  # check dna_seg_line
  if (is.logical(dna_seg_line)) {
    dna_seg_line <- as.character(dna_seg_line)
    dna_seg_line[dna_seg_line == "TRUE"] <- "black"
  }
  if (!is.character(dna_seg_line ))
    stop("dna_seg_line should be eiher a logical or character giving color")
  if (length(dna_seg_line) == 1){
    dna_seg_line <- rep(dna_seg_line, n_dna_segs)
  } else if (length(dna_seg_line) != n_dna_segs){
    stop("dna_seg_line should be of length 1 or same length as dna_segs")
  }
  
  # check gene_type
  if (!is.null(gene_type) && !(gene_type %in% gene_types()))
    stop(paste("gene_type muste be one of:",
               paste(gene_types(), collapse=", ")))

  # check scale. Must be logical
  if (is.logical(dna_seg_scale)){
    # if length 1, make dna_seg_scale the same length as dna_segs
    if (length(dna_seg_scale) == 1){
      dna_seg_scale <- rep(dna_seg_scale, n_dna_segs)
    }
    # if a different length, must be of the same length as n_dna_segs
    else if (length(dna_seg_scale) != n_dna_segs){
      stop("dna_seg_scale must be the same length dna_segs")
    }
  }
  else {
    stop("dna_seg_scale must be logical")
  }

  # check global_color_scheme
  if (length(global_color_scheme) != 4)
    stop ("global_color_scheme should be length 4")
  # accepted values for second value
  glob_col_sch_2_vals <- c("increasing", "decreasing", "auto")
  if (length(grep(global_color_scheme[2], glob_col_sch_2_vals)) != 1){
    stop(paste("Second argument to global_color_scheme should be one of",
               paste(glob_col_sch_2_vals, collapse=", ")))
  } else {
    global_color_scheme[2] <- grep(global_color_scheme[2],
                                   glob_col_sch_2_vals, value=TRUE)
  }
  
  #----------------------------------------------------------------------------#
  # plotting options
  #----------------------------------------------------------------------------#
  # dna_seg lines height. 1 line for the dna_seg, 0.5 in addition for
  # the dna_seg_scale, if needed. 1 null in between for comparisons
  # to rewrite sometimes
  h <- rep(1, n_rows)
  h[seq(2, n_rows, by=3)] <- 1 + scale_cex*dna_seg_scale + annot_h
  h[seq(1, n_rows, by=3)] <- seg_plot_h
  dna_seg_heights <- unit(h, c(rep(c(seg_plot_height_unit, "lines", "null"),
                                   n_dna_segs), "lines"))
  
  # deal with resetting symbols
  if (!is.null(gene_type)){
    if (gene_type == "auto"){
      n_genes <- sapply(dna_segs, nrow)
      gene_type <- auto_gene_type(n_genes)
    }
    for (i in 1:n_dna_segs){
      dna_segs[[i]]$gene_type <- gene_type
    }
  }

  # deal with global color scheme: if no color defined or if override is
  # set, try to find a common numerical column in all comparisons
  if ((!any("col" %in% unlist(lapply(comparisons, names)))
           || override_color_schemes) && !is.null(comparisons)){
    # collect numerical columns
    num_cols <- lapply(comparisons, function(x) names(x)[sapply(x, is.numeric)])
    shared_num_cols <-
      names(which(table(unlist(num_cols)) == length(num_cols)))
    shared_num_cols <- shared_num_cols[!shared_num_cols %in%
                                       c("start1", "start2", "end1", "end2")]
    # global_color_scheme[1]: take color from?
    # take per_id if present, if not, evalue, else the first found in the
    # first comp
    if (global_color_scheme[1] == "auto"){
      names_comp_1 <- names(comparisons[[1]])
      global_color_scheme[1] <- if ("per_id" %in% shared_num_cols){
        "per_id"
      } else if ("e_value" %in% shared_num_cols) {
        "e_value"
      } else {
        names_comp_1[names_comp_1 %in% shared_num_cols][1]
      }
    } else if (! global_color_scheme[1] %in% shared_num_cols){
      stop("One or all columns don't have the indicated column for global color scheme")
    }
    # global_color_scheme[1]: incr/decr. Turn it to decreasing
    if (global_color_scheme[2] == "auto"){
      global_color_scheme[2] <-
        if (global_color_scheme[1] %in% c("mism", "gaps", "e_value"))
          TRUE else FALSE
    } else if (global_color_scheme[2] == "decreasing"){
      global_color_scheme[2] <- TRUE
    } else if (global_color_scheme[2] == "increasing"){
      global_color_scheme[2] <- FALSE
    } else {
      stop("Invalid value for global_color_scheme[2]")
    }
    # gather range of values from all comparisons
    range_col_from <- c(Inf, -Inf)
    for (i in 1:n_comparisons){
      if (nrow(comparisons[[i]]) > 0){
        range_col_from[1] <- min(c(range_col_from[1],
                                   comparisons[[i]][[global_color_scheme[1]]]))
        range_col_from[2] <- max(c(range_col_from[2],
                                   comparisons[[i]][[global_color_scheme[1]]]))
      }
    }
    # perform apply_color_scheme
    for (i in 1:n_comparisons){
      comparisons[[i]]$col <-
        apply_color_scheme(x=comparisons[[i]][[global_color_scheme[1]]],
                           direction=comparisons[[i]]$direction,
                           color_scheme=global_color_scheme[3],
                           decreasing=global_color_scheme[2],
                           rng=range_col_from,
                           transparency=as.numeric(global_color_scheme[4]))
    }
  }

  #----------------------------------------------------------------------------#
  # prepare plotting frame & filter objects 
  #----------------------------------------------------------------------------#
  # further calculations on xlims & gaps
  for (i in 1:n_dna_segs){
    xlims[[i]]$length <- xlims[[i]]$x1 - xlims[[i]]$x0
  }
  # default gap_length is a 20th of the max length
  def_gap_length <- max(sapply(xlims, function(x) sum(x$length)))*
    minimum_gap_size
  unpadded_lengths <- sapply(xlims, function(x)
                             sum(x$length) + (nrow(x)-1)*def_gap_length)
  longest_seg <- which.max(unpadded_lengths)
  max_length <- unpadded_lengths[longest_seg]
  scale_unit <- diff(pretty(c(0, max_length), n=n_scale_ticks+2)[1:2])
  
  ### trim dna_segs & seg_plot ###
  # initiate new object: create subsegments by trimming original dna_seg
  seg_subplots <- list()
  dna_subsegs <- list()                    
  for (i in 1:n_dna_segs){
    n_subsegs <- nrow(xlims[[i]])
    dna_subsegs[[i]] <- list()
    seg_subplots[[i]] <- list()
    for (j in 1:n_subsegs){
      dna_subsegs[[i]][[j]] <- trim.dna_seg(dna_segs[[i]], c(xlims[[i]]$x0[j],
                                                             xlims[[i]]$x1[j]))
      if (seg_plot_h[[i]] > 0){
        seg_subplots[[i]][[j]] <- trim.seg_plot(seg_plots[[i]],
                                                c(xlims[[i]]$x0[j],
                                                  xlims[[i]]$x1[j]))
      }
    }
  }
  ### trim comparisons ###
  if (n_comparisons > 0){
    for (i in 1:n_comparisons){
      # concatenate all possible combinations
      comp1 <- comparisons[[i]][0,]
      for (j in 1:nrow(xlims[[i]])){
        for (k in 1:nrow(xlims[[i+1]])){
          comp1 <-
            rbind(comp1,
                  trim.comparison(comparisons[[i]],
                                  xlim1=as.numeric(xlims[[i]][j,
                                    c("x0", "x1")]),
                                  xlim2=as.numeric(xlims[[i+1]][k,
                                    c("x0", "x1")])))
        }
      }
      comparisons[[i]] <- comp1
      ## # Trim from above first
      ## comp1 <- comparisons[[i]][0,]
      ## for (j in 1:nrow(xlims[[i]])){
      ##   comp1 <- rbind(comp1, trim.comparison(comparisons[[i]],
      ##                                         xlim1=as.numeric(xlims[[i]][j,
      ##                                           c("x0", "x1")])))
      ## }
      ## # trim from below then: trim from the previously obtained comp
      ## comp2 <- comparisons[[i]][0,]
      ## for (j in 1:nrow(xlims[[i+1]])){
      ##   comp2 <- rbind(comp2, trim.comparison(comp1,
      ##                                         xlim2=as.numeric(xlims[[i+1]][j,
      ##                                           c("x0", "x1")])))
      ## }
      ## comparisons[[i]] <- comp2
    }
  }
  
  ### calculate offsets ###
  if (is.null(offsets)){
    prel_offsets <- lapply(xlims, function(x)
                           c(0, rep(def_gap_length, nrow(x)-1)))
    offsets <- minimize_comps(comparisons, xlims, unpadded_lengths,
                              prel_offsets, fixed_gap_length)
  } else {
    offsets <- as.list(offsets)
    for (i in 1:n_dna_segs){
      if (length(offsets[[i]]) == 1){
        offsets[[i]] <- c(offsets[[i]], rep(def_gap_length, nrow(xlims[[i]])-1))
      } else if (length(offsets[[i]]) != nrow(xlims[[i]])){
        stop("The length of each element of offsets should be either one or equal to the number of subsegments in the corresponding segment.")
      }
    }
  }
  
  # check if total length doesn't exceed max length
  if (limit_to_longest_dna_seg){
    for (i in 1:n_dna_segs){
      tot_length <- sum(c(xlims[[i]]$length, offsets[[i]]))
      if (tot_length > max_length){
        excess <- tot_length - max_length
        # reduce offsets from the end
        for (j in length(offsets[[i]]):1){
          # decrease offsets or set them to min_gap
          if ((offsets[[i]][j] - excess) < def_gap_length){
            excess <- excess - offsets[[i]][j] + def_gap_length
            offsets[[i]][j] <- def_gap_length
          } else {
            offsets[[i]][j] <- offsets[[i]][j] - excess
            break
          }
        }
      }
    }
  } else {
    # recalculate lengths
    padded_lengths <- sapply(xlims, function(x) sum(x$length)) +
      sapply(offsets, sum)
    max_length <- max(padded_lengths)
  }
  
  # recalculate comps coordinates
  if (n_comparisons > 0){
    for (i in 1:n_comparisons){
      comparisons[[i]] <- calc_comp_coor(offsets[[i]], xlims[[i]],
                                         comparisons[[i]], side=1)
      comparisons[[i]] <- calc_comp_coor(offsets[[i+1]], xlims[[i+1]],
                                         comparisons[[i]], side=2)
    }
  }
  # recalculate lengths
  padded_lengths <- sapply(xlims, function(x) sum(x$length)) +
    sapply(offsets, sum)
  max_length <- max(padded_lengths)
  longest_segment <- which.max(padded_lengths)
  
  #----------------------------------------------------------------------------#
  # collect grobs
  #----------------------------------------------------------------------------#
  ### collect dna_seg, dna_seg_scale & seg_plot grobs ###
  dna_seg_grobs <- list()
  dna_seg_scale_grobs <- list()
  for (i in 1:n_dna_segs){
    dna_seg_grobs[[i]] <- list()
    dna_seg_scale_grobs[[i]] <- list()
    for (j in 1:length(dna_subsegs[[i]])){
      # debug
      if (debug > 0 && debug < nrow(dna_subsegs[[i]][[j]]))
        dna_subsegs[[i]][[j]] <- dna_subsegs[[i]][[j]][1:debug,]
      # end debug
      dna_seg_grobs[[i]][[j]] <- dna_seg_grob(dna_subsegs[[i]][[j]],
                                            arrow_head_len, i, ...)
      dna_seg_scale_grobs[[i]][[j]] <-
        if (dna_seg_scale[[i]])
          dna_seg_scale_grob(range=xlims[[i]][j,c("x0","x1")],
                             cex=scale_cex, unit=scale_unit, i=i, j=j)
        else NULL
    }
  }
  ### collect seg_plot_grobs & ylims
  seg_plot_grobs <- list()
  seg_plot_ylims <- list()
  for (i in 1:n_dna_segs){
    seg_plot_grobs[[i]] <- list()
    xl_sg <- c(Inf, -Inf)
    for (j in 1:length(dna_subsegs[[i]])){
      if (length(seg_plots[[i]]) > 0){
        grb <- do.call(seg_subplots[[i]][[j]]$func, seg_subplots[[i]][[j]]$args)
        rng <- nice_ylim.seg_plot(seg_subplots[[i]][[j]])
        xl_sg[1] <- min(xl_sg[1], rng[1])
        xl_sg[2] <- max(xl_sg[2], rng[2])
        seg_plot_grobs[[i]][[j]] <- grb
      }
      else {
        seg_plot_grobs[[i]] <- NULL
      }
    }
    seg_plot_ylims[[i]] <- if (is.null(seg_plots[[i]]$ylim))
      xl_sg else seg_plots[[i]]$ylim
  }
  ### collect seg_plot_yaxis_grobs
  seg_plot_yaxis_grobs <- list()
  for (i in 1:n_dna_segs){
    seg_plot_yaxis_grobs[[i]] <-
      if (length(seg_plots[[i]]) > 0 && seg_plot_yaxis > 0)
        yaxis_grob(seg_plot_ylims[[i]], cex=seg_plot_yaxis_cex,
                 n=seg_plot_yaxis, i)
      else NULL
  }
  ### collect comparison grobs ###
  comparison_grobs <- list()
  if (n_comparisons > 0){
    for (i in 1:n_comparisons){
      # debug
      if (debug > 0 && debug < nrow(comparisons[[i]]))
        comparisons[[i]] <- comparisons[[i]][1:debug,]
      # end debug
      comparison_grobs[[i]] <- comparison_grob(comparisons[[i]], i)
    }
  }
  
  ### annotations ###
  if (!is.null(annotations)){
    annotation_grobs <- list()
    for (i in 1:n_dna_segs){
      if (is.null(annotations[[i]])){
        annotation_grobs[[i]] <- list(NULL)
      } else {
        annotation_grobs[[i]] <- list()
        for (j in 1:length(dna_subsegs[[i]])){
          annot <- trim.annotation(annotations[[i]],
                                   xlims[[i]][j, c("x0", "x1")])
          annotation_grobs[[i]][[j]] <- annotation_grob(annot,
                                                        annotation_cex)
        }
      }
    }
  }
  
  ### scale ###
  if (scale){
    scale_grob <- scale_grob(max_length)
    scale_h <- 1
  } else {
    scale_h <- 0
  }

  ### main title ###
  if (!is.null(main)){
    main_grob <- textGrob(x=main_x, label=main,
                          gp=gpar(cex=1.2), just=main_just)
    main_h <- 1.8
  } else {
    main_h <- 0
  }

  ### tree ###
  if (!is.null(tree)){
    # tree
    #phylog <- newick2phylog(tree)
    # check that a nice permutation is OK, return ys
    y <- permute_tree(tree, dna_seg_labels)
    # feed tree grob with permutation transformed as y coords
    tree_grob <- phylog_grob(tree, 1-((y-1)/(n_dna_segs-1)),
                             clabel.leaves=dna_seg_label_cex,
                             clabel.nodes=tree_branch_labels_cex,
                             tree.scale=tree_scale)
    #tree_w <- unit(0.20, "npc")
    tree_w <- unit(0.1, "npc") + tree_grob$width
  } else if(!is.null(dna_seg_labels)){
    # just labels
    tree_grob <- dna_seg_label_grob(dna_seg_labels, cex=dna_seg_label_cex,
                                    col=dna_seg_label_col)
    tree_w <- tree_grob$width
  } else {
    # nothing
    tree_grob <- NULL
    tree_w <- unit(0, "npc")
  }
  if (!is.null(tree_width)) tree_w <- unit(tree_width, "inches")
  # reset scale_h if tree_scale is true
  if (tree_scale) scale_h <- 1
  
  #----------------------------------------------------------------------------#
  # plotting
  #----------------------------------------------------------------------------#
  # overall frame
  if (plot_new) grid.newpage()
  pushViewport(viewport(width=unit(1, "npc")-unit(1, "lines"),
                        height=unit(1, "npc")-unit(1, "lines"),
                        name="oma"),
               viewport(layout=grid.layout(2, 1,
                          heights=unit(c(main_h, 1),
                            c("lines", "null"))),
                        name="oma_layout"))
  # main title
  if (!is.null(main)) {
    pushViewport(viewport(layout.pos.row=1, name="main"))
    grid.draw(main_grob)
    upViewport()
  }

  # frame: columns=tree,maps,legend rows=maps+tree+legend,scale
  seg_plot_yaxis_w <- if (seg_plot_yaxis > 0 &&
                          !is.null(seg_plots[[longest_segment]]))
    unit(3, "grobwidth", data=seg_plot_yaxis_grobs[[longest_segment]])
  else unit(0, "null")
  pushViewport(viewport(layout.pos.row=2,
                        layout=grid.layout(2, 3,
                          heights=unit(c(1, scale_h), c("null", "lines")),
                          widths=unit.c(tree_w, unit(1, "null"),
                            seg_plot_yaxis_w)),
                        name="frame"))
  # scale
  if (scale) {
    pushViewport(viewport(layout.pos.row=2, layout.pos.col=2,
                          xscale=c(0, max_length), name="scale"))
    grid.draw(scale_grob)
    upViewport()
  }

  # tree or labels. Height is 1-3 lines because margin is 2 in plotarea,
  # and 1 to center to the middle of each dna_seg (1/2 line top and bottom)
  if (!is.null(tree_grob)){
    # extra margin if there is an annotation in the first dna_seg
    annot_margin <- unit(if (is.null(annotations[[1]])) 0 else
                         annotation_height, "lines")
    # extra margin is there is a seg_plot in the first dna_seg
    seg_plot_margin <- unit(if (is.null(seg_plot_grobs[[1]][[1]])) 0 else
                            seg_plot_height, seg_plot_height_unit)
    # make a supplementary margin if there is a scale in the last
    # dna_seg to get labels facing the text. Hack.
    hli <- unit(0.5, "lines")
    dna_scale_margin <- unit(scale_cex * dna_seg_scale[[n_dna_segs]], "lines")
    pushViewport(viewport(layout.pos.row=1, layout.pos.col=1,
                          layout=grid.layout(6, 1,
                            heights=unit.c(seg_plot_margin, annot_margin, hli,
                              unit(n_dna_segs*(1+seg_plot_height), "null"),
                              hli, dna_scale_margin)),
                          name="tree_outer"))
    pushViewport(viewport(layout.pos.row=4,
                          width=unit(1, "npc")-unit(1, "lines"),
                          just=c("centre", "bottom"),
                          name="tree"))
    grid.draw(tree_grob$grob)
    upViewport(2) # up tree & tree_outer vp
  } 
  
  # plotting area
  pushViewport(viewport(layout.pos.row=1, layout.pos.col=2,
                        name="plotarea_outer", clip="on"),
               viewport(width=unit(1, "npc")-unit(1, "lines"),
                        height=unit(1, "npc")-unit(0, "lines"),
                        name="plotarea", clip="off"))

  # map grid
  pushViewport(viewport(layout=grid.layout(n_rows, 1,
                          heights=dna_seg_heights), name="map"))
  ### comparisons ###
  if (n_comparisons > 0){
    for (i in 1:n_comparisons){
      pushViewport(viewport(layout.pos.row=3*i,
                            yscale=c(0,1),
                            xscale=c(0, max_length),
                            #clip="on",
                            name = paste("comparison", i, sep=".")))
      # draw comparison grobs
      grid.draw(comparison_grobs[[i]])
      upViewport() # pop comparisons[[i]] vp
    }
  }
  ### seg_plots, dna_segs, annotations, scales ###
  for (i in 1:n_dna_segs){
    n_dna_subsegs <- length(dna_subsegs[[i]])
    n_cols <- n_dna_subsegs*2+1
    widths <- numeric(n_cols)
    widths[1:n_dna_subsegs*2] <- xlims[[i]]$length
    widths[1:n_dna_subsegs*2-1] <- offsets[[i]]
    widths_units <- unit(widths, rep("native", n_cols))
    heights <- unit(c(annot_h[i], 1, scale_cex*dna_seg_scale[i]),
                    c("lines", "null", "lines"))
    # push seg_plot grid
    pushViewport(viewport(layout.pos.row=3*i-2,
                          layout=grid.layout(1, n_cols,
                            widths=widths_units,
                            just=c("left", "centre")),
                          #clip="on",
                          xscale=c(0,max_length),
                          name=paste("seg_plot", i, sep=".")))
    for (j in 1:n_dna_subsegs){
      idx <- if (xlims[[i]]$strand[j] == 1) c("x0", "x1") else c("x1", "x0")
      xscale <- as.numeric(xlims[[i]][j,idx])
      if (!is.null(seg_plots[[i]])){
        pushViewport(viewport(layout.pos.col=j*2,
                              yscale=seg_plot_ylims[[i]],
                              xscale=xscale,
                              just=c("left", "centre"),
                              name=paste("seg_subplot", i, j, sep=".")))
        grid.draw(seg_plot_grobs[[i]][[j]])
        upViewport() # up seg_subplot vp
        
      }
    }
    ## Draw y axis
    if (!is.null(seg_plots[[i]]) && seg_plot_yaxis > 0){
      pushViewport(viewport(layout.pos.col=n_cols,
                            yscale=seg_plot_ylims[[i]],
                            width=unit(1, "grobwidth",
                              data=seg_plot_yaxis_grobs[[i]]),
                            just=c("left", "centre"),
                            name=paste("seg_plot_yaxis", i, sep=".")))
      grid.draw(seg_plot_yaxis_grobs[[i]])
      upViewport() # up seg plot yaxis
    }
    upViewport() # up seg_plot vp
    # push dna_seg grid (subsegments in cols, annotations, genes and
    # scales in rows)
    pushViewport(viewport(layout.pos.row=3*i-1,
                          layout=grid.layout(3, n_cols,
                            heights=heights,
                            widths=widths_units,
                            just=c("left", "centre")),
                          #clip="on",
                          xscale=c(0,max_length),
                          name=paste("scale_and_dna_seg", i, sep=".")))
    for (j in 1:n_dna_subsegs){
      # calculate xscale
      idx <- if (xlims[[i]]$strand[j] == 1) c("x0", "x1") else c("x1", "x0")
      xscale <- as.numeric(xlims[[i]][j,idx])
      # annotation
      if (!is.null(annotations[[i]])){
        pushViewport(viewport(layout.pos.row=1,
                              layout.pos.col=j*2,
                              yscale=c(0,1),
                              xscale=xscale,
                              just=c("left", "centre"),
                              name=paste("annotation", i, j, sep=".")))
        grid.draw(annotation_grobs[[i]][[j]])
        upViewport() # up annotation vp
      }
      # dna_seg_scale
      if (dna_seg_scale[i]){
        pushViewport(viewport(layout.pos.row=3,
                              layout.pos.col=j*2,
                              yscale=c(0,1),
                              xscale=xscale,
                              just=c("left", "centre"),
                              name=paste("dna_seg_scale", i, j, sep=".")))
        grid.draw(dna_seg_scale_grobs[[i]][[j]])
        upViewport() # up dna_seg_scale vp
      }
      # dna_seg itself
      pushViewport(viewport(layout.pos.row=2,
                            layout.pos.col=j*2,
                            yscale=c(0,1),
                            xscale=xscale,
                            just=c("left", "centre"),
                            name = paste("dna_seg", i, j, sep=".")))
      # draw segment line
      if (!dna_seg_line[i]=="FALSE"){
        grid.segments(x0=unit(xlims[[i]]$x0[j], "native"),
                      y0=unit(0.5, "native"),
                      x1=unit(xlims[[i]]$x1[j], "native"),
                      y1=unit(0.5, "native"),
                      name=paste("dna_seg_line", i, j, sep="."),
                      gp=gpar(col=dna_seg_line[i]))
      }
      # draw dna_seg grobs
      grid.draw(dna_seg_grobs[[i]][[j]])
      upViewport() # up dna_seg
      # draw gap, but not at pos 1
      if (j > 1){
        pushViewport(viewport(layout.pos.row=2,
                              layout.pos.col=j*2-1,
                              yscale=c(0,1),
                              xscale=c(0, widths[2*j-1]),
                              just=c("centre", "centre"),
                              name=paste("gap", i, j, sep=".")))
        grid.draw(gap_grob(w=def_gap_length, m=widths[2*j-1]/2, i, j))
        upViewport() # up gap vp
      }
    }
    upViewport() # up scale_and_dna_seg vp
  }
  upViewport(2) # pop map viewports
  upViewport(2) # pop plotarea viewport
  upViewport(2) # pop frame+oma viewport
}

################################################################################
# Phylo tree
################################################################################
# a function to plot phylogenetic trees ported to grid from ade4.
# to be rewritten...
phylog_grob <-
  function (x, y=NULL, f.phylog = 1, cleaves = 0, cnodes = 0, 
            labels.leaves = names(x$leaves), clabel.leaves = 1,
            labels.nodes = names(x$nodes), 
            clabel.nodes = 0.8,
            leave.seg.col = grey(0.7),
            tree.scale = FALSE,
            sub = "", csub = 1.25, possub = "bottomleft", 
            draw.box = FALSE, ...) 
{
  if (!inherits(x, "phylog")) 
    stop("Non convenient data")
  leaves.number <- length(x$leaves)
  leaves.names <- names(x$leaves)
  nodes.number <- length(x$nodes)
  nodes.names <- names(x$nodes)
  if (length(labels.leaves) != leaves.number) 
    labels.leaves <- names(x$leaves)
  if (length(labels.nodes) != nodes.number) 
    labels.nodes <- names(x$nodes)
  leaves.car <- gsub("[_]", " ", labels.leaves)
  nodes.car <- gsub("[_]", " ", labels.nodes)
  if (f.phylog < 0.05) 
    f.phylog <- 0.05
  if (f.phylog > 0.95) 
    f.phylog <- 0.95
  maxx <- max(x$droot)
  x.leaves <- x$droot[leaves.names]
  x.nodes <- x$droot[nodes.names]
  annot_nodes <- grep("^[^I]", names(x.nodes), value=TRUE)
  if (!is.null(y)){
    # check that it is constrained between 0 and 1
    if (!all(range(y) %in% c(0,1)))
      stop("y should be constrained between 0 and 1")
  } else {
    y <- seq(1, 0, len=leaves.number)
  }
  names(y) <- leaves.names
  xcar <- maxx * 1.05
  xx <- c(x.leaves, x.nodes)
  # prepare grobs
  labelGrobs <- gList()
  labelSegGrobs <- gList()
  branchLabelGrobs <- gList()
  branchesGrobs <- gList()
  # leaves labels and segments leading to it
  if (clabel.leaves > 0) {
    for (i in 1:leaves.number) {
      labelGrobs[[i]] <-
        textGrob(x=0, y=y[i], label=leaves.car[i], just="left",
                 gp=gpar(cex=clabel.leaves), default.units="native",
                 name=paste("tree.leave.label", i, sep="."))
      labelSegGrobs[[i]] <-
        segmentsGrob(xcar, y[i], xx[i], y[i],
                     gp=gpar(col=leave.seg.col), default.units="native",
                     name=paste("tree.leave.segment", i, sep="."))
   }
  }
  yleaves <- y[1:leaves.number]
  xleaves <- xx[1:leaves.number]
  ## if (cleaves > 0) {
  ##     for (i in 1:leaves.number) {
  ##         points(xx[i], y[i], pch = 21, bg = 1, cex = par("cex") * 
  ##             cleaves)
  ##     }
  ## }
  yn <- rep(0, nodes.number)
  names(yn) <- nodes.names
  y <- c(y, yn)
  # plot tree
  for (i in 1:length(x$parts)) {
    w <- x$parts[[i]]
    but <- names(x$parts)[i]
    y[but] <- mean(y[w])
    b <- range(y[w])
    # vertical branches
    branchesGrobs[[i]] <-
      segmentsGrob(xx[but], b[1], xx[but], b[2], default.units="native",
                   name=paste("tree.branch.vert.segment", i, sep="."))
    x1 <- xx[w]
    y1 <- y[w]
    x2 <- rep(xx[but], length(w))
    # horizontal branches
    branchesGrobs[[i+length(x$parts)]] <-
      segmentsGrob(x1, y1, x2, y1, default.units="native",
                   name=paste("tree.branch.horiz.segment", i, sep="."))
    idx_node_label <- names(x1) %in% annot_nodes
    if (clabel.nodes > 0 && sum(idx_node_label) > 0) {
      label <- names(x1)[idx_node_label]
      label <- gsub("^X?([^_]+)(_[0-9]+)?$", "\\1", label, perl=TRUE)
      branchLabelGrobs[[i]] <- 
        textGrob(label=label,
                 x=(x1[idx_node_label] + x2[idx_node_label])/2,
                 y=unit(y1[idx_node_label], "native") + unit(0.5, "line"),
                 gp=gpar(cex=clabel.nodes), default.units="native",
                 name=paste("tree.branch.label", i, sep="."))
        
    }
  }
  # scale
  if (tree.scale){
    rng <- diff(pretty(c(0, maxx), n=2))[1]
    scaleGrob <- segmentsGrob(maxx, unit(-1.5, "lines"),
                              maxx-rng,  unit(-1.5, "lines"),
                              default.units="native",
                              name=("tree.scale.seg"))
    scaleLabelGrob <- textGrob(label=rng, x=maxx-(rng/2), y=unit(-1, "lines"),
                               default.units="native",
                               name=("tree.scale.label"))
  } else {
    scaleGrob <- nullGrob()
    scaleLabelGrob <- nullGrob()
  }
  ## if (cnodes > 0) {
  ##     for (i in nodes.names) {
  ##         points(xx[i], y[i], pch = 21, bg = "white", cex = cnodes)
  ##     }
  ## }
  ## if (clabel.nodes > 0) {
  ##     scatterutil.eti(xx[names(x.nodes)], y[names(x.nodes)], 
  ##         nodes.car, clabel.nodes)
  ## }
  x <- x.leaves
  y <- y[leaves.names]
  xbase <- xcar
  ## if (csub > 0) 
  ##     scatterutil.sub(sub, csub = csub, possub = possub)
  ## if (draw.box) 
  ##     box()
  ## if (cleaves > 0) 
  ##     points(xleaves, yleaves, pch = 21, bg = 1, cex = par("cex") * 
  ##         cleaves)
  # creating gTree for branches
  branchesTree <- gTree(children=gList(branchesGrobs, labelSegGrobs,
                          branchLabelGrobs, scaleGrob, scaleLabelGrob),
                        vp=viewport(xscale=c(0, xcar), yscale=c(0, 1),
                          name="tree.branches"),
                        name="tree.branchesTree")
  labelTree <- gTree(children=labelGrobs,
                     vp=viewport(xscale=c(0, 1), yscale=c(0, 1),
                       name="tree.labels"),
                     name="tree.labelsTree")
  # finally plotting
  label_width <- unit(1, "grobwidth",
                      labelGrobs[[which.max(nchar(leaves.car))]])
  layout <- grid.layout(1, 2, widths=unit.c(unit(1, "null"), label_width))
  fg <- frameGrob(layout=layout, name="treeFrameGrob",
                  vp=viewport(name="treeFrame"))
  fg <- placeGrob(fg, branchesTree, col=1)
  fg <- placeGrob(fg, labelTree, col=2)
  return(invisible(list(xy = data.frame(x = x, y = y), xbase = xbase, 
                        cleaves = cleaves, grob=fg, width=label_width)))
}


# permute tree leaves to match labels
permute_tree <- function(tree, labels, no.over=100000){
  ref <- names(tree$leaves)
  n <- length(ref)
  wanted_permut <- rep(NA, n)
  for (i in 1:n){
    idx <- which(ref[i] == labels)
    if(length(idx) != 1) stop("Non-unique or non-matching labels")
    wanted_permut[i] <- idx
  }
  if (identical(as.numeric(1:n), as.numeric(wanted_permut))){
    res <- 1:n
    names(res) <- labels
    return(res)
  }
  permuts <- enum.phylog(tree, no.over)
  if (is.null(permuts)) stop("Number of permutations too large, use a tree that has labels in the same order as dna_segs")
  equals <- apply(permuts, 1, function(x)
                  identical(as.numeric(x), as.numeric(wanted_permut)))
  if (!any(equals))
    stop("No tree permutation compatible with label order. Change input order")
  if (!sum(equals))
    stop("Several permutations matching. Something went wrong, contact the author")
  return(permuts[equals,])
}

################################################################################
# File reading functions: read comparison
################################################################################
# read comparison file. Use source=tab or blast to specify type
read_comparison_from_blast <- function(file, sort_by="per_id",
                                       filt_high_evalue=NULL,
                                       filt_low_per_id=NULL,
                                       filt_length=NULL,
                                       color_scheme=NULL, ...){
  # read table anyway
  table <- read.table(file, as.is=TRUE, header=FALSE, sep="\t", quote="")
  # from blast output
  col_names <- c("name1", "name2", "per_id", "aln_len", "mism", "gaps",
                 "start1", "end1", "start2", "end2", "e_value", "bit_score")
  names(table) <- col_names
  # sort?
  if (!is.null(sort_by)){
    if (!sort_by %in% col_names) stop("Argument sort_by not in col names")
    if (!is.numeric(table[[sort_by]]))
      stop("Argument sort_by must designate a numeric column")
    decr <-
      if (sort_by %in% c("per_id", "aln_len", "bit_score")) FALSE else TRUE
    # reorder from weakest to strongest
    table <- table[do.call(order, list(table[[sort_by]], decreasing=decr)),]
  }
  # filter evalue
  if (!is.null(filt_high_evalue)){
    if (!is.numeric(filt_high_evalue)) stop("filt_high_evalue must be numeric")
    table <- table[table$e_value <= filt_high_evalue,]
  }
  # filter per id
  if (!is.null(filt_low_per_id)){
    if (!is.numeric(filt_low_per_id)) stop("filt_low_per_id must be numeric")
    table <- table[table$per_id >= filt_low_per_id,]
  }
  # reorder columns
  table <- table[,c(col_names[c(7:10,1:6,11:12)])]
  
  # send further
  comparison <- .read_comparison(table, filt_length=filt_length, ...)
  # apply color scheme if requested
  if (!is.null(color_scheme)){
    comparison$col <- apply_color_scheme(comparison$per_id,
                                         direction=comparison$direction,
                                         color_scheme=color_scheme)
  }
  comparison
}
read_comparison_from_tab <- function(file, header=TRUE, ...){
  col_names <-  c("start1", "end1", "start2", "end2", "col")
  # from tabular data
  table <- read.table(file, as.is=TRUE, header=header, sep="\t", quote="")
  if (ncol(table) < 4) {
    stop("Insufficent number of columns in table")
  } else if (!header){
    if (ncol(table) == 4) {
      names(table) <- col_names[1:4]
    } else {
      names(table)[1:length(col_names)] <- col_names[1:length(col_names)]
    }
  }
  .read_comparison(table, ...)
}
.read_comparison <- function(table, filt_length=NULL, reverse=0,
                             color_scheme=NULL, ...){
  # check arguments
  if (ncol(table) < 4) stop("Insufficent number of columns in table")
  if (!all(c("start1", "end1", "start2", "end2") %in% names(table)))
    stop("Table must contain columns start1, end1, start2, end2")
  
  # filter length
  if (!is.null(filt_length)){
    if (!is.numeric(filt_length)) stop("filt_length must be numeric")
    av_len <- apply(cbind(abs(table$end1-table$start1),
                          abs(table$end2-table$start2)), 1, mean)
    table <- table[av_len >= filt_length,]
  }
  # reverse
  if (is.numeric(reverse) && reverse > 0){
    table <- reverse.comparison(table, side=reverse)
  }
  # make comparison object
  as.comparison(table)
}

################################################################################
# Plot helpers: list and determine gene "types"
################################################################################
gene_types <- function(auto=TRUE){
  types <- c("arrows", "headless_arrows",
             "blocks", "bars", "points", "text", "lines", 
             "side_blocks", "side_bars", "side_points", "side_text",
             "side_lines",
             "introns", "exons", "side_exons")
  if (auto) types <- c("auto", types)
  types
}
auto_gene_type <- function(n_genes){
  if (max(n_genes) > 1000){
    gene_type <- "side_bars"
  } else if (max(n_genes) > 100) {
    gene_type <- "side_blocks"
  } else {
    gene_type <- "side_bars"
  }
  gene_type
}

################################
# dna_seg class and methods
################################
dna_seg <- function(x, ...){
  # support for list ?
  if (is.data.frame(x)){
    return(as.dna_seg(x, ...))
  } else if (is.list(x)) {
    dna_seg(as.data.frame(x, stringsAsFactors=FALSE))
  } else {
    stop(paste("Cannot coerce class", class(x), "to dna_seg"))
  }
}
# convert to dna_seg format. 
as.dna_seg <- function(df, col="blue", fill="blue", lty=1, lwd=1, pch=8,
                       cex=1, gene_type="arrows"){
  # check for class dna_seg, list, df
  if (is.dna_seg(dna_seg)) return(df)
  #if (is.list(df) && !is.data.frame(df)) df <- dna_seg(df)
  if (is.data.frame(df)) {
    # check that it has rows
    if (nrow(df) < 1){
      stop("Number of rows is 0, check data input")
    }
    # check for columns
    names <- c("name", "start", "end", "strand")
    if (!identical(names(df)[1:4], names))
      stop("Col names should start with name, start, end, strand")
    if (!all(sapply(df, function(x) all(!is.null(x)))))
      stop("NULL values not allowed in data.frame")
    if (!is.numeric(df$start) | !is.numeric(df$end)){
      stop("Start and end must be numeric")
    }
    if (is.factor(df$name)) df$name <- as.character(df$name)
    if (is.factor(df$strand)) df$strand <- as.character(df$strand)
    if (is.factor(df$col)) df$col <- as.character(df$col)
    if (is.factor(df$fill)) df$fill <- as.character(df$fill)
    if (is.factor(df$gene_type)) df$gene_type <- as.character(df$gene_type)
    # care for strand
    if (is.character(df$strand)) {
      df$strand[df$strand=="+"] <- 1
      df$strand[df$strand=="-"] <- -1
      df$strand <- as.numeric(df$strand)
    }
    if (!all(df$strand %in% c(-1, 1)))
      stop("Strand vector must be composed of 1, -1, - and +, uniquely")
    # col
    if (is.null(df$col)) df$col <- col
    if (is.null(df$fill)) df$fill <- fill
    # lwd & lty
    if (is.null(df$lty)) df$lty <- lty
    if (is.null(df$lwd)) df$lwd <- lwd
    if (is.null(df$pch)) df$pch <- pch
    if (is.null(df$cex)) df$cex <- cex
    # gene_type: not given in gene
    if (is.null(df$gene_type)){
      if (gene_type == "auto") gene_type <- auto_gene_type(nrow(df))
      df$gene_type <- gene_type
    }
    if (is.null(df$gene_type)) df$gene_type <- gene_type
    # check for correct argument types
    if (!is.character(df$name)) stop("Non-character name")
    if (!(is.numeric(df$start) && is.numeric(df$end)))
      stop("Non-numeric start or end")
    if (!all(is.numeric(c(df$lwd, df$lty, df$pch, df$cex))))
      stop("lwd, lty, pch and cex must be numeric")
    if (!is.character(df$gene_type))
      stop(paste("gene_type must be a character vector, made of:",
                 paste(gene_types(), collapse=", "), "or a function name"))
  }
  else {
    stop("Unable to handle this format")
  }
  class(df) <- c("dna_seg", "data.frame")
  return(df)
}
is.dna_seg <- function(dna_seg){
  inherits(dna_seg, "dna_seg")
}
# calculates dna_seg range
range.dna_seg <- function(x, ...){
  dna_seg <- x
  range(dna_seg$start, dna_seg$end, na.rm=FALSE)
}
#range.data.frame <- range.dna_seg
# trim dna_seg given x limits
trim.dna_seg <- function(x, xlim=NULL, ...){
  dna_seg <- x
  if (!is.null(xlim)){
    if (!is.numeric(xlim)) stop("xlim must be numeric")
    if (length(xlim) != 2) stop("xlim must be length 2")
    dna_seg <- dna_seg[dna_seg$start >= xlim[1] & dna_seg$end <= xlim[2],]
  }
  dna_seg
}
# reverse dna_seg
reverse.dna_seg <- function(x, ...){
  dna_seg <- x
  start <- -dna_seg$end
  dna_seg$end <- -dna_seg$start
  dna_seg$start <- start
  dna_seg$strand <- -dna_seg$strand
  dna_seg
}
# merges two dna_segs. So far returns the minimal common set of columns, maybe
# changed in future
c.dna_seg <- function(...){
  # a little helper function to grab colnames
  #fold <- function(f, x, L) (for(e in L) x <- f(x, e))
  fold <- function(x, fun) {
    if (length(x) == 1) return(fun(x))
    accumulator <- fun(x[[1]], x[[2]])
    if (length(x) == 2) return(accumulator)
    for(i in 3:length(x)) {
      accumulator <- fun(accumulator, x[[i]])
    }
    accumulator
  }
  # parse args
  x <- list(...)
  n <- length(x)
  if (!all(sapply(x, is.dna_seg))) 
    stop("All elements must be of class dna_seg")
  if (n == 1) return(x)
  cols <- lapply(x, names)
  com_cols <- fold(cols, intersect)
  dna_seg <- x[[1]][com_cols]
  for (i in 2:n){
    dna_seg <- rbind(dna_seg, x[[i]][com_cols])
  }
  dna_seg
}

# download fasta & gbk
FTP=ftp://ftp.ncbi.nih.gov/genomes/Bacteria/
for EXT in ptt rnt fna; do
    wget $FTP/Bartonella_bacilliformis_KC583/NC_008783.$EXT -O BB.$EXT
    wget $FTP/Bartonella_grahamii_as4aup/NC_012846.$EXT -O BG.$EXT
    wget $FTP/Bartonella_henselae_Houston-1/NC_005956.$EXT -O BH.$EXT
    wget $FTP/Bartonella_quintana_Toulouse/NC_005955.$EXT -O BQ.$EXT
done

# make databases; run blasts with BLAST+ 2.2.22
for ORG in BB BG BH BQ; do
    makeblastdb -in $ORG.fna -dbtype nucl -parse_seqids
done
blastn -query BB.fna -db BG.fna -evalue 1e-3 -outfmt 6 > BB_vs_BG.blastn.tab
blastn -query BG.fna -db BH.fna -evalue 1e-3 -outfmt 6 > BG_vs_BH.blastn.tab
blastn -query BH.fna -db BQ.fna -evalue 1e-3 -outfmt 6 > BH_vs_BQ.blastn.tab

# download gbk files for 4 Bartonella genomes
FTP=ftp://ftp.ncbi.nih.gov/genomes/Bacteria/
wget $FTP/Bartonella_bacilliformis_KC583/NC_008783.gbk -O BB.gbk
wget $FTP/Bartonella_grahamii_as4aup/NC_012846.gbk -O BG.gbk
wget $FTP/Bartonella_henselae_Houston-1/NC_005956.gbk -O BH.gbk
wget $FTP/Bartonella_quintana_Toulouse/NC_005955.gbk -O BQ.gbk

# run mauve: this supposes that you have progressiveMauve available in 
# your path. Mauve version used was 2.3.1
progressiveMauve --output=barto.xmfa --output-guide-tree=barto.guide_tree --backbone-output=barto.backbone --hmm-identity=0.8 --island-gap-size=10000 BB.gbk BG.gbk BH.gbk BQ.gbk

\documentclass[a4paper]{article}
\usepackage{url} % For \url{}
%\VignetteIndexEntry{Getting started with genoPlotR}
%\VignettePackage{genoPlotR}
\newcommand{\code}[1]{\texttt{#1}}
\newcommand{\pkg}[1]{{\normalfont\fontseries{b}\selectfont #1}}
\newcommand{\genoPlotR}{\pkg{genoPlotR}}
\newcommand{\R}{{\sffamily R}}

\title{Getting started with \genoPlotR{}}
\author{Lionel Guy}

\begin{document}

\maketitle

\tableofcontents

\section*{Introduction}

The \genoPlotR{} package is intended to produce publication-grade graphics 
of gene and genome maps. With the amazing speed of data production of new 
DNA sequencing techniques and the increase in the number of software 
available to compare these sequences, there is a great need to graphically 
represent these sequences and their comparisons. A number of packages 
already exist (Artemis, ACT, mauve), but none of them produces easily 
reproducible, publication-grade graphics. The goal of this package is 
to fill in that gap. 

This document provides an introduction to \genoPlotR{}, providing the user 
with examples of increasing complexity. It is not meant as a comprehensive
guide to all the functions and options of the package, but rather as 
a first approach to the package.

To load the library in a R session, type:

<<results=hide>>=
library(genoPlotR)
@ 
\section{Quick start}

Loading the simplest dataset, applying a color scheme and some limits to the 
plotting aread, adding a tree and some annotations. For more details about
that plot, refer to the first of the examples developed below.
<<>>=
data(three_genes)
comparisons[[1]]$col <- apply_color_scheme(c(0.6, 0.4, 0.5), "grey")
names <- c("Huey", "Dewey", "Louie")
names(dna_segs) <- names
tree <- newick2phylog("(((Huey:4.2,Dewey:3.9):3.1,Louie:7.3):1);")
mid_pos <- middle(dna_segs[[1]])
xlims <- list(c(Inf, -Inf), c(-Inf, Inf), c(1850, 2800))
annot <- annotation(x1=c(mid_pos[1], dna_segs[[1]]$end[2]),
                     x2=c(NA, dna_segs[[1]]$end[3]),
                     text=c(dna_segs[[1]]$name[1], "region1"),
                     rot=c(30, 0), col=c("blue", "black"))
@

Now plotting these three segments:

<<quick_plot>>=
plot_gene_map(dna_segs=dna_segs, comparisons=comparisons,
              annotations=annot, annotation_height=1.3,
              tree=tree, tree_width=2,
              xlims=xlims,
              main="Comparison of Huey, Dewey and Louie")

@ 
\begin{center}
<<fig=TRUE, width=6, height=2.5, echo=FALSE, results=hide>>=
<<quick_plot>>
@ 
\end{center}

\section{Getting help}

There are various ways to get help with \genoPlotR{}. First, you can start 
the general help in a web browser, and then click on ``packages'' and 
find \genoPlotR{} in the list. It will provide a list of the functions 
available. The first link in the list leads to a very general description of
the package.
<<eval=FALSE, results=hide>>=
help.start()
@ 
Another way of obtaining the list of functions present in the package is to run 
<<eval=FALSE>>=
library(help=genoPlotR)
@ 
A lot of examples and help are available in the main functions, i.e. the 
reading functions (the various \code{read\_dna\_seg\_from}* and 
\code{read\_comparison\_from}* functions) and the main plotting 
function, \code{plot\_gene\_map}.
<<eval=FALSE>>=
help("read_functions")
help("plot_gene_map")
@ 
Finally, the web page on R-forge (\url{http://genoplotr.r-forge.r-project.org/})
provides ways to get in touch with the \genoPlotR{} community and to submit 
bugs and feature requests.

\section{Objects in \genoPlotR{}}

This section will give an overview of the different types of \R{} objects in
\genoPlotR{}.

\subsection{\code{dna\_seg}}

A \code{dna\_seg} object is a collection of genes or elements along a 
genome, to be represented on a map. 

\code{dna\_seg} objects need to have 4 columns, \code{name}, \code{start}, 
\code{end} and \code{strand}. Extra columns with names \code{col}, 
\code{lty}, \code{lwd}, \code{pch}, \code{cex}, \code{gene\_type} will 
be used in the  plotting process. Other extra columns will be kept in 
the object, but not used.

<<>>=
names1 <- c("feat1", "feat2", "feat3")
starts1 <- c(2, 1000, 1050)
ends1 <- c(600, 800, 1345)
strands1 <- c("-", -1, 1)
cols1 <- c("blue", "grey", "red")
df1 <- data.frame(name=names1, start=starts1, end=ends1,
                  strand=strands1, col=cols1)

dna_seg1 <- dna_seg(df1)
str(dna_seg1)
@

\subsection{\code{comparison}}

A \code{comparison} is a collection of similarities, representing the
comparison between two DNA segments. 

Objects (either data frames or lists) should have at least named
elements \code{start1}, \code{end1}, \code{start2} and \code{end2}. 
In addition, it can use a \code{col}or column, that will give
the color of each comparison. Additional numeric columns can be used 
for automatic color-coding (via \code{apply\_color\_scheme}.

<<>>=
starts1 <- c(2, 1000, 1050)
ends1 <- c(600, 800, 1345)
starts2 <- c(50, 800, 1200)
ends2 <- c(900, 1100, 1322)
comparison1 <- as.comparison(data.frame(start1=starts1, end1=ends1,
                                        start2=starts2, end2=ends2))
str(comparison1)
@ 

\subsection{\code{annotation}}

An \code{annotation} object is used to annotate a DNA segment. It has 
labels attached to positions. Each label can be attached to a single 
position or to a range. 

<<>>=
mid_pos <- middle(dna_segs[[1]])
annot1 <- annotation(x1=mid_pos, text=dna_segs[[1]]$name)
str(annot1)
@ 

\subsection{\code{tree}}

A tree description in Newick format can ba parsed using \pkg{ade4} package.
<<>>=
tree <- newick2phylog("(((A_aaa:4.2,B_bbb:3.9):3.1,C_ccc:7.3):1);")
str(tree$leaves)
@ 

\section{Reading data}

\subsection{DNA segments}

Several formats can be read by \genoPlotR{} to produce \code{dna\_seg} objects:
\begin{itemize}
  \item EMBL files (\code{read\_dna\_seg\_from\_embl})
  \item Genbank files (\code{read\_dna\_seg\_from\_genbank})
  \item PTT (protein table) files, tabular versions of Genbank 
    files)(\code{read\_dna\_seg\_from\_ptt})
  \item User generated tabular files (\code{read\_dna\_seg\_from\_ptt})
\end{itemize}
The function \code{read\_dna\_seg\_from\_file} is a wrapper function, that
will attempt to guess the correct format of the file.

The first three files are common biological formats and can be downloaded 
from major databases, such as the NCBI (\url{http://www.ncbi.nlm.nih.gov/}) 
and the ENA (\url{http://www.ebi.ac.uk/ena}). The 
definition of EMBL and Genbank files can be found at 
\url{ftp://ftp.ebi.ac.uk/pub/databases/embl/doc/FT_current.html}.

\subsection{Comparisons}

\genoPlotR{} can read tabular files, either user-generated tab files 
(\code{read\_comparison\_from\_tab}), or from BLAST output
(\code{read\_comparison\_from\_blast}). To produce files that are readable by 
\genoPlotR{}, the \code{-m 8} or \code{9} option should be used in 
\code{blastall}, or \code{-outfmt 6} or {7} with the BLAST+ suite.

\subsection{Mauve output}

The backbone output of the Mauve genome aligner 
(\url{http://darlinglab.org/mauve/mauve.html}) can be parsed using 
\code{read\_comparison\_from\_blast}\footnote{Tested with Mauve 2.3.1}.

The function will return a list consisting of a list of \code{dna\_seg} objects 
and the corresponding \code{comparisons}.

\section{Plotting data}

There is only one plotting function in \genoPlotR{}, \code{plot\_gene\_map}.
Many arguments are available, but here is a list of the most important. Check
the documentation for a more thorough description.
\begin{itemize}
  \item[\code{dna\_segs}] A list of DNA segment objects.
  \item[\code{comparisons}] A list of comparisons. Should containt one element
    less than the previous.
  \item[\code{tree}] An eventual phylogenetic tree to be plotted at the
    left of the figure.
  \item[\code{annotations}] An annotation object, or a list of annotations. 
    Will display annotations to the first, or to all DNA segments, respectively.
  \item[\code{xlims}] A list of even-numbered numeric vectors, giving the
    borders of sub-segments to be plotted. The vector \code{c(1,5000,8000,6000)}
    will display two sub-segments (1 to 5000 and 6000 to 8000), the second 
    being in reverse orientation.
  \item[\code{main}] A title to the plot.
  \item[\code{scale}] Should a scale be displayed at the bottom right of the
    plot?
  \item[\code{dna\_seg\_scale}] Allows to control the addition of scales
    to each segments. If simply \code{TRUE}, will display a scale on each
    segment. If a vector, a scale will be displayed for the corresponding 
    \code{TRUE} element.
  \item[\code{global\_color\_scheme}] Allows to recalculate the colors of 
    the comparisons, to have colors corresponding to the same scale for all 
    comparisons.
  \item[\code{plot\_new}] Turn off to avoid creating a new plot. Escpecially
    useful to integrate a \genoPlotR{} plot in a larger figure.
\end{itemize}

\section{Other useful functions}

\begin{itemize}
  \item[\code{apply\_color\_scheme}] Allows to apply a gray scale or shades
    of red and blue to a comparison.
  \item[\code{middle}] Useful to get the middle of a gene, especially to
    create annotations.
  \item[Datasets] Type \code{data(package="genoPlotR")} to get the full list.
\end{itemize}

\section{Examples}

This section gives step-by-step examples, of gradual complexity. The last
one shows how to combine several plots and to annotate already generated 
plots.

For all examples, the first step is to load the library.

<<loadLib>>=
library(genoPlotR)
@ 

\subsection{Example 1: A very simple example}

The data used in this example is a dataset included in \genoPlotR{}, but
for the sake of demonstration, it will be recreated from scratch. To retrieve
data from the package, use \code{data(three\_genes)}.

\subsubsection{Data generation and a very simple plot}

First, three \code{dna\_segs} are generated, from data frames with columns 
name, start, end, strand and col.

<<ex1_create_dna_segs>>=
df1 <- data.frame(name=c("feat1", "feat2", "feat3"),
                  start=c(2, 1000, 1050),
                  end=c(600, 800, 1345),
                  strand=c(-1, -1, 1),
                  col=c("blue", "grey", "red"))
dna_seg1 <- dna_seg(df1)
df2 <- data.frame(name=c("feat1", "feat2", "feat3"),
                  start=c(50, 800, 1200),
                  end=c(900, 1100, 1322),
                  strand=c(-1, 1, 1),
                  col=c("blue", "grey", "red"))
dna_seg2 <- dna_seg(df2)
df3 <- data.frame(name=c("feat1", "feat2", "feat3"),
                  start=c(1899, 2108, 2803),
                  end=c(2034, 2732, 3620),
                  strand=c(-1, -1, 1),
                  col=rep("blue", 3))
dna_seg3 <- dna_seg(df3)
dna_segs <- list(dna_seg1, dna_seg2, dna_seg3)
@ 

Then, create two \code{comparisons} objects from data frames with columns
start1, end1, start2, end2 and col. In the first comparison, the starts
and ends correspond to the genes created above.

<<ex1_create_comps>>=
df4 <- data.frame(start1=dna_seg1$start,
                  end1=dna_seg1$end,
                  start2=dna_seg2$start,
                  end2=dna_seg2$end)
comparison1 <- comparison(df4)
df5 <- data.frame(start1=c(50, 800),
                  end1=c(500, 1100),
                  start2=c(1899, 2732),
                  end2=c(2034, 2508),
                  col=c("#67000D", "#08306B"))
comparison2 <- comparison(df5)
comparisons <- list(comparison1, comparison2)
@

This constitutes all the material required to create a basic plot.

<<ex1_raw_plot>>=
plot_gene_map(dna_segs=dna_segs, comparisons=comparisons)
@ 
\begin{center}
<<fig=TRUE, width=6, height=2.5, echo=FALSE, results=hide>>=
<<ex1_raw_plot>>
@ 
\end{center}
  
\subsubsection{A more elaborate plot}

Many options can be added to this simple plot. To begin with, the first 
comparison will be colored in grey scale, using an arbitrary scale passed 
to the function \code{apply\_color\_scheme}.

<<ex1_col_scheme>>=
comparisons[[1]]$col <- apply_color_scheme(c(0.6, 0.4, 0.5), "grey")
@ 

Second, names and a phylogenetic tree are prepared. The function 
\code{newick2phylog} from the package \pkg{ade4} is used to prepare an
object suitable for \genoPlotR{}. The elements of the list of 
\code{dna\_segs} must be named and correspond to the labels of the
tree object, to avoid confusion.

<<ex1_tree>>=
names <- c("Huey", "Dewey", "Louie")
names(dna_segs) <- names
tree_HDL <- newick2phylog("(((Huey:4.2,Dewey:3.9):3.1,Louie:7.3):1);")
@ 

Third, annotations to the first segment are added. The first gene is annotated
in its middle (thus the use of \code{middle} function), while a region 
comprising the second and third gene are annotated with a square bracket.
Note the use of a numeric value for \code{x1} and \code{NA} for the first
gene, and the use of two numeric values for the second region.

<<ex1_annots>>=
mid_pos <- middle(dna_segs[[1]])
annot <- annotation(x1=c(mid_pos[1], dna_segs[[1]]$end[2]),
                    x2=c(NA, dna_segs[[1]]$end[3]),
                    text=c(dna_segs[[1]]$name[1], "region1"),
                    rot=c(30, 0), col=c("grey", "black"))
@ 

Finally, the plot is drawn, addind a title, allowing 2 inches for the tree
width (instead of the default 20\% of the plot), and changing the annotation
space to 1.3 lines (as opposed to 1 by default). The different options can be
checked by removing/adding the different arguments to \code{plot\_gene\_map}. 

<<ex1_adv_plot>>=
plot_gene_map(dna_segs=dna_segs, comparisons=comparisons,
              annotations=annot, annotation_height=1.3,
              tree=tree_HDL, tree_width=2,
              main="Comparison of Huey, Dewey and Louie")
@

To explore the different options, new windows can be opened by using 
\code{x11()} or \code{window()}, depending on the OS, or saved to files 
using for example \code{png()} or \code{pdf()}. To finish the plot, the 
function \code{dev.off()} should be called.


\begin{center}
<<fig=TRUE, width=6, height=2.5, echo=FALSE, results=hide>>=
<<ex1_adv_plot>>
@ 
\end{center}
The plot can be saved by adding, in front of the call to 
\code{plot\_gene\_map}, a call to one of the graphical device functions of 
\R{} (e.g. \code{pdf()}, \code{png()}, \code{jpeg()}). To finish the plot,
the function \code{dev.off()} should be called.

\subsection{Example 2: Generating data online}

This section will give an example of how data can be retrieved and generated
online, using the resources available at the NCBI
\url{http://www.ncbi.nlm.nih.gov/}. This example was devised on 5/3/2010, and
given the relatively rapid rate of changes in public databases, it is 
possible that part or all of this example becomes obsolete. Please contact
the author of this document (lionel.guy@ebc.uu.se) in such a case.

In this example, the genomes of \emph{Bartonella henselae} 
(RefSeq accession NC\_005956) and \emph{Bartonella quintana} (RefSeq 
accession NC\_005955) will be compared.

\subsubsection{Retrieving \code{dna\_seg} data}

To retrieve Genbank files, the Entrez Nucleotide Database can be directly 
queried, provided that the accession number is known. In such a case, the
database can be accessed from the home page of NCBI 
(\url{http://www.ncbi.nlm.nih.gov/}), by entering the accession number 
(e.g. NC\_005956) in the search field and by selecting ``Nucleotide'' in 
the ``Search'' list. The Genbank file is then displayed. It can be saved it in 
a suitable format by clicking on the ``Download'' link and by selecting 
``GenBank''. 

If the accession number is unknown, the nucleotide database can be queried
for the name of the organism, or accessed via a list 
(\url{https://www.ncbi.nlm.nih.gov/genome/browse/}). In that list, the files
can be directly retrieved from the ftp site of the NCBI (by clicking the letter
``F'' in the corresponding row). Alternatively, it is possible to click 
on the accession number, which leads to a page summarizing information on
the queried genome. Access to the page described above is achieved by clicking
on the accession number linked under the title ``Refseq''. 

Repeat the procedure for the other genome (NC\_005955).

\subsubsection{Performing web-based BLAST and retrieving result}

The two previously retrieved genomes can be compared with BLAST, more
precisely using the \code{bl2seq} feature of BLAST.

Access to the BLAST section of the NCBI website is achieved by clicking on
BLAST in the ``Popular resources'' section of the NCBI home page, or by using
a direct link (\url{http://blast.ncbi.nlm.nih.gov/Blast.cgi}). A link to
the ``bl2seq'' resource is presented in the ``Specialized BLAST'' section.

There, the genomes can be aligned by entering the two genome accession 
numbers. The other arguments to BLAST can then be modified at will to suit 
the needs of the comparison. Once the BLAST search is run, the hit table 
(which can be read by \genoPlotR{} can be downloaded by clicking on
``Download'' and ``Hit Table(text)'' in the result page.

\subsubsection{Plotting}

Provided that the Genbank files and their comparison have been saved
under the correct names in the directory where R was started, 
they can be parsed directly by \genoPlotR{}.

<<ex_2_load_fake, results=hide, eval=FALSE>>=
BH <- try(read_dna_seg_from_file("NC_005956.gbk"))
BQ <- try(read_dna_seg_from_file("NC_005955.gbk"))
BH_vs_BQ <- try(read_comparison_from_blast("NC_005956_vs_NC_005955.blast"))
@ 

<<ex2_load_silent, echo=FALSE, results=hide>>=
data(barto)
BH <- barto$dna_segs[[3]]
BQ <- barto$dna_segs[[4]]
BH_vs_BQ <- barto$comparisons[[3]]
@ 

The data, or a sample thereof (using \code{xlims}) can now be plotted.

<<ex2_plot>>=
xlims <- list(c(1,50000), c(1,50000))
plot_gene_map(dna_segs=list(BH, BQ),
              comparisons=list(BH_vs_BQ),
              xlims=xlims,
              main="BH vs BQ, comparison of the first 50 kb",
              gene_type="side_blocks",
              dna_seg_scale=TRUE, scale=FALSE)

@ 
\begin{center}
<<fig=TRUE, width=6, height=1.8, echo=FALSE, results=hide>>=
<<ex2_plot>>
@ 
\end{center}
\subsection{Example 3: Mauve alignment of four \emph{Bartonella} genomes}

\genoPlotR{} is able to parse the backbone file produced by Mauve (for details
on the format, see \url{http://darlinglab.org/mauve/user-guide/files.html}).

The elements of the DNA segments are no longer genes, but Mauve blocks. 
Similarly, the comparison reflects the correspondances between these blocks. 
The strand indicates the orientation of the block with respect to a reference,
which by default is the first genome in the comparison. A single list is 
returned, which contains two lists, one containing the \code{dna\_seg}s and
the other one the \code{comparison}s.

The genomes in the backbone file are not named, so it is advised to be 
cautious in the order of the names given.

In this example, 4 genomes of \emph{Bartonella} have been compared with 
Mauve 2.3.1. The smaller blocks (smaller than 10 kb) are filtered out, and
the second genome (which is the largest) is taken as the reference). 
The data set can also be accessed in the package by running
\code{data(mauve\_bbone)}. 

<<ex3_read_data>>=
bbone_file <- system.file('extdata/barto.backbone', package = 'genoPlotR')
bbone <- read_mauve_backbone(bbone_file, ref=2, filter_low=10000)
names <- c("B_bacilliformis", "B_grahamii", "B_henselae", "B_quintana")
names(bbone$dna_segs) <- names
@ 

Calculating the lengths by adding the length on both sides of the comparisons

<<ex3_length>>=
for (i in 1:length(bbone$comparisons)){
  cmp <- bbone$comparisons[[i]]
  bbone$comparisons[[i]]$length <- 
    abs(cmp$end1 - cmp$start1) + abs(cmp$end2 - cmp$start2)
}
@ 

Now plotting, using \code{global\_color\_scheme} to color the segments 
according to their lengths.

<<ex3_plot>>=
plot_gene_map(dna_segs=bbone$dna_segs, 
              comparisons=bbone$comparisons,
              global_color_scheme=c("length", "increasing", "red_blue", 0.7),
              override_color_schemes=TRUE)
@
\begin{center}
<<fig=TRUE, width=8, height=2, echo=FALSE, results=hide>>=
<<ex3_plot>>
@ 
\end{center}

\subsection{Example 4: Several sub-segments of four \emph{Bartonella} genomes}

This examples presents how to use the \code{xlims} argument to represent 
several sub-segments of the same segment, and how to show them in a reverse
orientation.

The data used here is also a comparison between four \emph{Bartonella} genomes
(see above), but the comparison has been performed using BLAST. The dataset
is available in the package (\code{barto}).

First, the data is loaded and a tree is created. 

<<ex4_load_tree>>=
data(barto)
tree_barto <- newick2phylog("(BB:2.5,(BG:1.8,(BH:1,BQ:0.8):1.9):3);")
@ 

The \code{xlims} argument is then created. It is a list with as many elements
as there are \code{dna\_seg}s. Each element is an even-numbered numeric 
vector, containing the left and right border of each sub-segment, 
consecutively. In the first DNA segment, two sub-segments are shown: between
1,415,000 and 1,445,000 in a reverse orientation, and between 1,380,000 and
1,412,000, in ``normal'' orientation. In the remaining segments, 3, 3, and 1 
subsegments are shown, respectively, all in normal orientation.

<<ex4_xlims>>=
xlims <- list(c(1445000, 1415000, 1380000, 1412000),
              c(  10000,   45000,   50000,   83000, 90000, 120000),
              c(  15000,   36000,   90000,  120000, 74000,  98000),
              c(   5000,   82000))
@

To complete the example, annotations are added. Genes that have a gene name 
(i.e. these for which the name is not the locus or synonym name) are used
to annotate the segment. Only every fourth gene is annotated, to avoid
overlapping the tags. The function \code{middle} is used to retrieve the 
middle of each element of the DNA segments.

<<ex4_annots>>=
annots <- lapply(barto$dna_segs, function(x){
  mid <- middle(x)
  annot <- annotation(x1=mid, text=x$name, rot=30)
  idx <- grep("^[^B]", annot$text, perl=TRUE)
  annot[idx[idx %% 4 == 0],] 
})
@ 

Finally, plot the result, using scales on each DNA segment, adding a title,
and not limiting the plotting area to the longest segment, to allow for a better
placement of the sub-segments.

<<ex4_plot>>=
plot_gene_map(barto$dna_segs, barto$comparisons, tree=tree_barto,
              annotations=annots,
              xlims=xlims,
              limit_to_longest_dna_seg=FALSE,
              dna_seg_scale=TRUE, scale=FALSE,
              main="Comparison of homologous segments in 4 Bartonella genomes")
@ 
\begin{center}
<<fig=TRUE, width=6, height=4, echo=FALSE, results=hide>>=
<<ex4_plot>>
@ 
\end{center}

\subsection{Example 5: Two segments of the Y chromsome in human and chimp}

In this example, the ability to plot introns and exons is demontrated.

First, data is loaded. The dataset included in the package is used, but any
Genbank file containing introns and exons can be used. Some annotations are
added: for each segment, the range of each gene is calculated, and 
corresponding annotations are created. 

<<ex5_load_annot>>=
data(chrY_subseg)
genes_homo <- unique(chrY_subseg$dna_segs[[1]]$gene)
x_homo <- sapply(genes_homo, function(x)
                 range(chrY_subseg$dna_segs[[1]]
                       [chrY_subseg$dna_segs[[1]]$gene == x,])
                 )
annot_homo <- annotation(x1=x_homo[1,], x2=x_homo[2,],
                         text=dimnames(x_homo)[[2]])
genes_pan <- unique(chrY_subseg$dna_segs[[2]]$gene)
x_pan <- sapply(genes_pan, function(x)
                range(chrY_subseg$dna_segs[[2]]
                      [chrY_subseg$dna_segs[[2]]$gene == x,])
                 )
annot_pan <- annotation(x1=x_pan[1,], x2=x_pan[2,],
                        text=dimnames(x_pan)[[2]])
@ 

The segments can be directly plotted, passing the annotations as a list.

<<ex5_plot>>=
main <- "Comparison of two subsegments in H. sapiens and P. troglodytes"
plot_gene_map(chrY_subseg$dna_segs, chrY_subseg$comparison,
              annotations=list(annot_homo, annot_pan),
              dna_seg_scale=TRUE,
              main=main,
              scale=FALSE)
@ 
\begin{center}
<<fig=TRUE, width=6, height=2, echo=FALSE, results=hide>>=
<<ex5_plot>>
@ 
\end{center}

\subsection{Example 6: Combining several \genoPlotR{} figures and annotating thefigure}

In this example, some of the previous plots are combined in a single, 
multi-panel figure. Using the tools present in the \pkg{grid} package, the
plot is annotated further. More information is available in the documentation
of the \pkg{grid} package.

The example here uses \R{} objects built in the previous examples, which 
should thus be run first.

First, a \code{viewport} that will contain all plots is pushed in a new page.
The plot is divided in three rows, of relative heights 1, 1.3 and 0.8.

<<ex6_startdevice, echo=FALSE, results=hide>>=
  pdf(file="ex6.pdf", width=6, height=8, onefile=TRUE)
@ 

<<ex6_vpext, results=hide>>=
pushViewport(viewport(layout=grid.layout(3,1,
                        heights=unit(c(1,1.3,0.8), rep("null", 3))),
                      name="overall_vp"))
@ 

The three panels A to C, containing the result of the above examples 4 to 6 
are pushed into viewports, calling \code{upViewport()} after each plot to 
come back to the main viewport. After the last plot, \code{upViewport(2)}
is called to go up 2 viewports, coming back to the root viewport. Each time,
the option \code{plot\_new=FALSE} is used, to avoid plotting in a new 
page every single panel.


<<ex6_plot, results=hide>>=
## Panel A
pushViewport(viewport(layout.pos.row=1, name="panelA"))
plot_gene_map(dna_segs=bbone$dna_segs, comparisons=bbone$comparisons,
              dna_seg_scale=c(FALSE, FALSE, FALSE, TRUE),
              scale=FALSE, main="A", main_pos="left", plot_new=FALSE)
upViewport()
## Panel B
pushViewport(viewport(layout.pos.row=2, name="panelB"))
plot_gene_map(barto$dna_segs, barto$comparisons, 
              annotations=annots, tree=tree_barto, xlims=xlims,
              limit_to_longest_dna_seg=FALSE, scale=FALSE,
              dna_seg_scale=TRUE, main="B", main_pos="left",
              annotation_height=0.6, annotation_cex=0.5, 
              plot_new=FALSE)
upViewport()
## Panel C
pushViewport(viewport(layout.pos.row=3, name="panelC"))
plot_gene_map(chrY_subseg$dna_segs, chrY_subseg$comparison,
              annotations=list(annot_homo, annot_pan),
              dna_seg_scale=TRUE, scale=FALSE, main="C", main_pos="left",
              plot_new=FALSE)
upViewport(0)
@ 

The commands \code{current.vpTree()} and \code{grid.ls()} can be used to see the current 
viewport structure, and to see all object names.
<<ex6_currentvp>>=
grid_list <- grid.ls(grob=TRUE, viewports=TRUE, print=FALSE)
str(grid_list)
current.vpTree()
@ 

Despite the complexity of the viewport structure, it is possible to identify
the previously defined panel A, B and C. Using the \code{downViewport} 
commands, it will be possible to find and modify elements of the plots. 
First, the labels of panel A will be modified to have the names in italics
and to replace the underscores by a dot and a space. The second label is 
also removed. Note the use of \code{grid.edit} and \code{grid.remove}. 
Warning: the whole figure is redrawn for each use of the edition 
function, it can thus become relatively long.

<<ex6_mod_panela, results=hide>>=
downViewport("panelA")
for (i in 1:length(names)){
  new_label <- sub("_", ". ", names[[i]])
  grid.edit(paste("label", i, sep="."), label=new_label, 
            gp=gpar(fontface="italic"))
}
grid.remove("label.2")
upViewport(0)
@ 

In a second time, a red rectangle will be placed around one subsegment of
panel B.

<<ex6_mod_panelb>>=
downViewport("panelB")
downViewport("dna_seg.3.2")
grid.rect(height = unit(2.2, "npc"), gp=gpar(col="red", lwd=2, fill=0))
upViewport(0)
@ 
<<ex6_devoff echo=FALSE, results=hide>>=
dev.off()
@ 
\begin{center}
\includegraphics[page=6]{ex6} 
\end{center}

\subsection{Example 7: Using user-defined \code{gene\_type} and \code{seg\_plot}}
In this example, the use of personalized \code{gene\_type}s and of 
\code{seg\_plots} to show data next to DNA segments is demonstrated. The 
same data as in example 4 is used.

A graphical function is defined, to represent triangles instead of arrows 
for genes. The triangles are oriented in function of their strand and the 
transparency is set differently according to a length cut-off, passed with 
the function. As required, the function has only two arguments, a \code{gene} 
which is a row of a \code{dna\_seg}, and the dots (...). Any extra argument 
passed to \code{plot\_gene\_map} will be passed in turn to the 
\code{gene\_type}-defining function.

<<ex7_starGrob, results=hide>>=
starGrob <- function(gene, ...){
  ## Coordinates for the star
  x <- sin(((0:5)/2.5)*pi)*(gene$end-gene$start)/2 + (gene$end+gene$start)/2
  y <- cos(((0:5)/2.5)*pi)*gene$strand*0.5 + 0.5
  idx <- c(1, 3, 5, 2, 4, 1)
  ## Attribute line_col only if present in the gene
  line_col <- if (!is.null(gene$line_col)) gene$line_col else gene$col
  ## Having a conditional transparency, depending on a length cut-off
  ## passed via dots
  length_cutoff <- list(...)$length_cutoff
  if (!is.null(length_cutoff)){
    alpha <- if ((gene$end-gene$start) < length_cutoff)  0.3 else  0.8
  } else alpha <- 1
  
  ## Grobs
  g <- polygonGrob(x[idx], y[idx], gp=gpar(fill=gene$col, col=line_col,
                                     lty=gene$lty, lwd=gene$lwd, alpha=alpha),
                   default.units="native")
  t <- textGrob(label="***", x=(gene$end+gene$start)/2, y=0.5,
                default.units="native")
  gList(g, t)
}
@ 

To make use of that, the \code{gene\_type} column of the \code{dna\_segs} must be replaced. To make use of the supplementary argument \code{line\_col}, a column is added to the \code{dna\_seg} object. Finally, plot the result.

<<ex7_replace_geneType>>=
barto$dna_segs[[2]]$gene_type <- "starGrob"
barto$dna_segs[[4]]$gene_type <- "starGrob"
line_col <- rep(1:20, (nrow(barto$dna_segs[[3]]) %% 20)+1)
barto$dna_segs[[2]]$line_col <- line_col[1:nrow(barto$dna_segs[[2]])]
plot_gene_map(barto$dna_segs, barto$comparisons, tree = tree_barto,
              annotations = annots,
              xlims = xlims, 
              dna_seg_scale = TRUE,
              length_cutoff = 600)
@ 
\begin{center}
<<fig=TRUE, width=7, height=6, echo=FALSE, results=hide>>=
<<ex7_replace_geneType>>
@ 
\end{center}

Now, let's plot some random data along with the DNA segment. A standard 
\code{grid} function returning a grob is used, but it is possible to use 
personalized functions, as long as they return \code{grob} or \code{gList} 
objects. For example, the function \code{starGrob} defined in the previous 
exercise could be modified to be used here.

The \code{seg\_plot} objects are first defined, with the following structure:
<<ex7_definesegplots>>=
seg_plots <- lapply(barto$dna_segs, function(ds){
  x <- seq(1, range(ds)[2], by=1000)
  y <- jitter(seq(100, 300, length=length(x)), amount=50)
  seg_plot(func=linesGrob, args=list(x=x, y=y, gp=gpar(col=grey(0.3), lty=2)))
})
str(seg_plots[[1]])
@ 

Each \code{seg\_plot} element has a \code{func} argument, giving the function 
to use to plot the data, here the \code{grid} function \code{linesGrob}. The
arguments that are passed to this function are contained in the \code{args} 
argument, a list. Notice the use of \code{gp=gpar(...)}. 

The result can then be plotted, and modulated using the arguments 
\code{seg\_plot\_height}, \code{seg\_plot\_height\_unit}, \code{seg\_plot\_yaxis},
and \code{seg\_plot\_yaxis\_cex}.

<<ex7_plotwithsegplots, results=hide>>=
plot_gene_map(barto$dna_segs, barto$comparisons, tree = tree_barto,
              annotations = annots,
              xlims = xlims,
              seg_plots = seg_plots,
              seg_plot_height = 0.5,
              seg_plot_height_unit = "null",
              seg_plot_yaxis = 2,
              seg_plot_yaxis_cex = 0.7)
@ 
<<fig=TRUE, width=7, height=6, echo=FALSE, results=hide>>=
<<ex7_plotwithsegplots>>
@ 

\end{document}

\name{genoPlotR-package}
\alias{genoPlotR-package}
\alias{genoPlotR}
\docType{package}
\title{
\packageTitle{genoPlotR}
}
\description{
\packageDescription{genoPlotR}
}
\details{
\packageDESCRIPTION{genoPlotR}
\packageIndices{genoPlotR}
  The only plotting function is \code{\link{plot_gene_map}}, which
  produces \code{link[grid]{grid}} graphics. Data is composed mainly of
  DNA segments (\code{\link{dna_seg}}) objects, which represent
  collections of genes or segments of genomes, and of
  \code{\link{comparison}} objects, which are the pairwise comparisons
  between the \code{dna_seg}s. Data can be read from files (see
  \code{\link{read_functions}}) or from R objects like
  \code{data.frame}s or \code{list}s, with \code{\link{dna_seg}} and
  \code{\link{comparison}} conversion functions. 
}
\author{
\packageAuthor{genoPlotR}

Maintainer: \packageMaintainer{genoPlotR}
}
\references{
Guy, L., Roat Kultima, J, and Andersson, S.G.E. (2010). genoPlotR:
comparative gene and genome visualization in R. Bioinformatics 26(18):2334-2335.
}
\keyword{ aplot }
\seealso{
  \code{\link{plot_gene_map}} for plotting. \code{\link{dna_seg}} and
  \code{\link{comparison}} for the base objects and conversion functions.
  \code{\link{read_dna_seg_from_tab}}, \code{\link{read_dna_seg_from_ptt}}, 
  \code{\link{read_comparison_from_tab}} and
  \code{\link{read_comparison_from_blast}} to read from files.
}
\examples{
## simple example
## dna segments
## data.frame with several genes
names1 <- c("feat1", "feat2", "feat3")
starts1 <- c(2, 1000, 1050)
ends1 <- c(600, 800, 1345)
strands1 <- c("-", -1, 1)
cols1 <- c("blue", "grey", "red")
df1 <- data.frame(name=names1, start=starts1, end=ends1,
                  strand=strands1, col=cols1)
dna_seg1 <- dna_seg(df1)
is.dna_seg(dna_seg1)

## with only one gene, or two, and merging
gene2a <- dna_seg(list(name="feat1", start=50, end=900, strand="-", col="blue"))
genes2b <- dna_seg(data.frame(name=c("feat2", "feat3"), start=c(800, 1200),
                              end=c(1100, 1322), strand=c("+", 1),
                              col=c("grey", "red")))
dna_seg2 <- c.dna_seg(gene2a, genes2b)
is.dna_seg(dna_seg2)

## reading from file
dna_seg3_file <- system.file('extdata/dna_seg3.tab', package = 'genoPlotR')
dna_seg3 <- read_dna_seg_from_tab(dna_seg3_file)
is.dna_seg(dna_seg3)

## comparison
## from a data.frame
comparison1 <- as.comparison(data.frame(start1=starts1, end1=ends1,
                                        start2=dna_seg2$start,
                                        end2=dna_seg2$end))
is.comparison(comparison1)

## from a file
comparison2_file <- system.file('extdata/comparison2.tab',
                                package = 'genoPlotR')
comparison2 <- read_comparison_from_tab(comparison2_file,
                                        color_scheme="red_blue")
is.comparison(comparison1)

## plot
plot_gene_map(dna_segs=list(dna_seg1, dna_seg2, dna_seg3),
              comparisons=list(comparison1, comparison2)) 
}

\name{artemisColors}
\alias{artemisColors}
\title{
  Artemis Colors
}
\description{
  Returns a data frame with the standard artemis colors.
}
\usage{
artemisColors()
}
\value{
  A \code{data.frame} with the following columns: \code{n}, \code{names},
  \code{colors}, \code{r}, \code{g} and \code{g}. The 3 first columns
  give the Artemis color number, its name, and its equivalent in R. The
  3 last give the r, g and b values. 
}
\author{
  Lionel Guy
}
\references{
  Artemis website: http://www.sanger.ac.uk/resources/software/artemis/
}
\examples{
artCol <- artemisColors()
plot(rep(1, nrow(artCol)), artCol$n, xlim=c(1, 2), type="n")
text(rep(1, nrow(artCol)), artCol$n, labels=artCol$n, col=artCol$colors)
text(rep(1, nrow(artCol)), artCol$n, labels=artCol$names, col=artCol$colors,
     pos=4, offset=1)

}
\keyword{ data }

\name{reverse}
\alias{reverse}
\alias{reverse.default}
\alias{reverse.dna_seg}
\alias{reverse.comparison}
\title{
  Reverse objects
}
\description{
  Reverse objects, mainly \code{dna_seg} and \code{comparison}
}
\usage{
reverse(x, ...)
\method{reverse}{default}(x, ...)
\method{reverse}{dna_seg}(x, ...)
\method{reverse}{comparison}(x, side = 0, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
    The object to reverse.
  }
  \item{\dots}{
    Unused.
  }
  \item{side}{
    In the case of comparisons, the side of the comparison that should
    be reversed. If \code{side=1}, the first side will be reversed. If
    \code{side=2}, the second side will be reversed. If \code{side<1},
    no side is reversed. If \code{side>2}, both sides are reversed.
  }
}
\value{
  The same object as input.
}
\author{
  Lionel Guy
}
\seealso{
  \code{\link{dna_seg}}, \code{\link{comparison}}
}
\examples{
## load data
data(three_genes)

## on dna_seg
dna_segs[[1]]
reverse(dna_segs[[1]])
## on comparison
reverse(comparisons[[2]], side=1)
reverse(comparisons[[2]], side=3)

## With mauve backbone
data(mauve_bbone)
## Plot
plot_gene_map(dna_segs=bbone$dna_segs, comparisons=bbone$comparisons)

## Reverse B_bacilliformis, and the corresponding comparison (first "side")
bbone$dna_segs[[1]] <- reverse(bbone$dna_segs[[1]])
bbone$comparisons[[1]] <- reverse(bbone$comparisons[[1]], 1)
plot_gene_map(dna_segs=bbone$dna_segs, comparisons=bbone$comparisons)

}
\keyword{ data }

\name{read_functions}
\alias{read_functions}
\alias{read_dna_seg_from_tab}
\alias{read_dna_seg_from_file}
\alias{read_dna_seg_from_embl}
\alias{read_dna_seg_from_genbank}
\alias{read_dna_seg_from_fasta}
\alias{read_dna_seg_from_ptt}
\alias{read_comparison_from_tab}
\alias{read_comparison_from_blast}
\alias{read_mauve_backbone}
\title{Reading functions}
\description{
  Functions to parse dna_seg objects from tab, embl, genbank, fasta, ptt
  files or from mauve backbone files, and comparison objects from tab or blast
  files. 
}
\usage{
read_dna_seg_from_tab(file, header = TRUE, ...)
read_dna_seg_from_file(file, tagsToParse=c("CDS"), fileType = "detect",
                       meta_lines = 2, gene_type = "auto", header = TRUE,
                       extra_fields = NULL, ...)
read_dna_seg_from_embl(file, tagsToParse=c("CDS"), ...)
read_dna_seg_from_genbank(file, tagsToParse=c("CDS"), ...)
read_dna_seg_from_fasta(file, ...)
read_dna_seg_from_ptt(file, meta_lines = 2, header = TRUE, ...)
read_comparison_from_tab(file, header = TRUE, ...)
read_comparison_from_blast(file, sort_by = "per_id",
                           filt_high_evalue = NULL,
                           filt_low_per_id = NULL,
                           filt_length = NULL,
                           color_scheme = NULL, ...)
read_mauve_backbone(file, ref = 1, gene_type = "side_blocks",
                    header = TRUE, filter_low = 0,
                    common_blocks_only = TRUE, ...)
}
\arguments{
  \item{file}{
    Path to file to load. URL are accepted.
  }
  \item{header}{
    Logical. Does the tab file has headers (column names)?
  }
  \item{tagsToParse}{
    Character vector. Tags to parse in embl or genbank files.
    Common tags are 'CDS', 'gene', 'misc_feature'.
  }
  \item{fileType}{
    Character string. Select file type, could be 'detect' for automatic
    detection, 'embl' for embl files, 'genbank' for genbank files or
    'ptt' for ptt files.
  }
  \item{meta_lines}{
    The number of lines in the ptt file that represent "meta" data, not
    counting the header lines. Standard for NCBI files is 2 (name and
    length, number of proteins. Default is also 2.
  }
  \item{gene_type}{
    Determines how genes are visualized. If 'auto' genes will appear as
    arrows in there are no introns and as blocks if there are introns.
    Can also be set to for example 'blocks' or 'arrows'. Do note, currently
    introns are not supported in the ptt file format.
    Default for mauve backbone is \code{side_blocks}.
    See \code{\link{gene_types}} page for more details, or use function
    \code{gene_types}. 
  }
  \item{extra_fields}{
    \code{NULL} by default. If a character vector, parses extra fields
    in the genbank or embl file that have corresponding keys and put
    them in the resulting \code{dna_seg}.
  }
  \item{sort_by}{
    In BLAST-like tabs, gives the name of the column that will be used
    to sort the comparisons. Accepted values are \code{per_id}
    (percent identity, default), \code{mism} (mismatches), \code{gaps}
    (gaps), \code{e_value} (E-value), \code{bit_score} (bit score). 
  }
  \item{filt_high_evalue}{
    A numerical, or \code{NULL} (default). Filters out all comparisons
    that have a e-value higher than this one.
  }
  \item{filt_low_per_id}{
    A numerical, or \code{NULL} (default). Filters out all comparisons
    that have a percent identity lower than this one.
  }
  \item{filt_length}{
    A numerical, or \code{NULL} (default). Filters out all comparisons
    that have alignments shorter than this value.
  }
  \item{color_scheme}{
    A color scheme to apply. See \code{apply_color_scheme} for more
    details. Possible values include \code{grey} and \code{red_blue}.
    \code{NULL} by default. Color schemes can be applied while running
    \code{plot_gene_map}.
  }
  \item{ref}{
    In mauve backbone, which of the dna segments will be the reference,
    i.e. which one will have its blocks in order.
  }
  \item{\dots}{
    Further arguments passed to generic reading functions and class
    conversion functions. See \code{\link{as.dna_seg}} and
    \code{\link{as.comparison}}.

    For \code{read_comparison*} functions, see details.
  }
  \item{filter_low}{
    A numeric. If larger than 0, all blocks smaller that this number
    will be filtered out. Defaults to 0.
  }
  \item{common_blocks_only}{
    A logical. If \code{TRUE} (by default), reads only common blocks
    (core blocks).
  }
}
\details{
  Tab files representing DNA segements should have at least the following
  columns: name, start, end, strand (in that order. Additionally, if the
  tab file has headers, more columns will be used to define, for
  example, the color, line width and type, pch and/or cex. See
  \code{\link{dna_seg}} for more information. An example:

  \tabular{lllll}{
    name   \tab start \tab end   \tab strand \tab col   \cr
    feat1A \tab 2     \tab 1345  \tab 1      \tab blue  \cr
    feat1B \tab 1399  \tab 2034  \tab 1      \tab red   \cr
    feat1C \tab 2101  \tab 2932  \tab -1     \tab grey  \cr
    feat1D \tab 2800  \tab 3120  \tab 1      \tab green \cr
  }

  Embl and Genbank files are two commonly used file types. These file types
  often contain a great variety of information. To properly extract data from
  these files, the user has to choose which features to extract. Commonly 'CDS'
  features are of interest, but other feature tags such as 'gene' or
  'misc_feature'  may be of interest. Should a feature contain an inner
  "pseudo" tag indicating this CDS or gene is a pseudo gene, this will
  be presented as a 'CDS_pseudo' or a 'gene_pseudo' feature type
  respectively in the resulting table. Certain constraints apply to
  these file types, of which some are: embl files must contain one and
  only one ID tag; genbank files may only contain one and only one locus
  tag. In these two files, the following tags are parsed (in addition to
  the regular name, start, end and strand): protein_id, product, color
  (or colour). In addition, extra tags can be parsed with the argument
  \code{extra_fields}. If there are more than one field with such a tag,
  only the first one is parsed.

  Fasta files are read as one gene, as long as there are nucleotides in
  the fasta file.

  Ptt (or protein table) files are a tabular format giving a bunch of
  information on each protein of a genome (or plasmid, or virus,
  etc). They are available for each published genome on the NCBI ftp
  site (ftp://ftp.ncbi.nlm.nih.gov/genomes/). As an example, look at
  ftp://ftp.ncbi.nlm.nih.gov/genomes/Bacteria/Bartonella_henselae_Houston-1/NC_005956.ptt.
  
  Tabular comparison files should have at least the following
  columns: start1, end1, start2, end2. If no header is specified, the
  fifth column is parsed as the color.

  \tabular{lllll}{
    start1 \tab end1   \tab start2 \tab end2  \tab  col  \cr
    2      \tab 1345   \tab 10     \tab 1210  \tab  red  \cr
    1399   \tab 2034   \tab 2700   \tab 1100  \tab  blue \cr
    500    \tab 800    \tab 3000   \tab 2500  \tab  blue \cr
  }

  BLAST tabular result files are produced either with blastall using -m8
  or -m9 parameter, or with any of the newer
  blastn/blastp/blastx/tblastx using -outfmt 6 or -outfmt 7.

  In the subsequent \code{plot_gene_map}, the comparisons are drawn in
  the order of the \code{comparison} object, i.e. the last rows of the
  comparison object are on the top in the plot. For comparisons read
  from BLAST output, the order can be modified by using the argument
  \code{sort_by}. In any case, the order of plotting can be modified by
  modifying the order of rows in the \code{comparison} object prior to
  plotting. 
  
  Mauve backbone is another tabular data file that summarizes the blocks
  that are similar between all compared genomes. Each genome gets two
  columns, one start and one end of the block. There is one row per
  block and eventually a header row. If named, columns have sequence
  numbers, not actual names, so be careful to input the same order in
  both Mauve and genoPlotR. See
  http://asap.ahabs.wisc.edu/mauve-aligner/mauve-user-guide/mauve-output-file-formats.html
  for more info on the file format. Normally, the function should be
  able to read both \code{progressiveMauve} and \code{mauveAligner}
  outputs. The function returns both the blocks as \code{dna_seg}s and
  the links between the blocks as \code{comparison}s.

}
\value{
  \code{read_dna_seg_from_tab}, \code{read_dna_seg_from_file}, \code{read_dna_seg_from_embl},
  \code{read_dna_seg_from_genbank} and \code{read_dna_seg_from_ptt} return
  \code{dna_seg} objects. \code{read_comparison_from_tab} and
  \code{read_comparison_from_blast} return \code{comparison}
  objects. \code{read_mauve_backbone} returns a list containing a list
  of \code{dna_seg}s and \code{comparison}s.
  objects. 
}
\references{
  For BLAST: http://www.ncbi.nlm.nih.gov/blast/
  For Mauve: http://asap.ahabs.wisc.edu/mauve/
}
\author{
  Lionel Guy, Jens Roat Kultima
}
\note{
  Formats are changing and it maybe that some functions are temporarily
  malfunctioning. Please report any bug to the author. Mauve examples
  were prepared with Mauve 2.3.1.
}
\seealso{
  \code{\link{comparison}}, \code{\link{dna_seg}},
  \code{\link{apply_color_scheme}}.
}
\examples{
##
## From tabs
##
## Read DNA segment from tab
dna_seg3_file <- system.file('extdata/dna_seg3.tab', package = 'genoPlotR')
dna_seg3 <- read_dna_seg_from_tab(dna_seg3_file)

## Read comparison from tab
comparison2_file <- system.file('extdata/comparison2.tab',
                                package = 'genoPlotR')
comparison2 <- read_comparison_from_tab(comparison2_file)

##
## Mauve backbone
##
## File: this is only to retrieve the file from the genoPlotR
## installation folder. 
bbone_file <- system.file('extdata/barto.backbone', package = 'genoPlotR')
## Read backbone
## To read your own backbone, run something like
## bbone_file <- "/path/to/my/file.bbone"
bbone <- read_mauve_backbone(bbone_file)
names <- c("B_bacilliformis", "B_grahamii", "B_henselae", "B_quintana")
names(bbone$dna_segs) <- names
## Plot
plot_gene_map(dna_segs=bbone$dna_segs, comparisons=bbone$comparisons)

## Using filter_low & changing reference sequence
bbone <- read_mauve_backbone(bbone_file, ref=2, filter_low=2000) 
names(bbone$dna_segs) <- names
plot_gene_map(dna_segs=bbone$dna_segs, comparisons=bbone$comparisons)

## Read guide tree
tree_file <- system.file('extdata/barto.guide_tree', package = 'genoPlotR')
tree_str <- readLines(tree_file)
for (i in 1:length(names)){
  tree_str <- gsub(paste("seq", i, sep=""), names[i], tree_str)
}
tree <- newick2phylog(tree_str)
## Plot
plot_gene_map(dna_segs=bbone$dna_segs, comparisons=bbone$comparisons,
              tree=tree)

##
## From embl file
##
bq_embl_file <- system.file('extdata/BG_plasmid.embl', package = 'genoPlotR')
bq <- read_dna_seg_from_embl(bq_embl_file)

##
## From genbank file
##
bq_genbank_file <- system.file('extdata/BG_plasmid.gbk', package = 'genoPlotR')
bq <- read_dna_seg_from_file(bq_genbank_file, fileType="detect")

## Parsing extra fields in the genbank file
bq <- read_dna_seg_from_file(bq_genbank_file,
                             extra_fields=c("db_xref", "transl_table"))
names(bq)


##
## From ptt files
##
## From a file
bq_ptt_file <- system.file('extdata/BQ.ptt', package = 'genoPlotR')
bq <- read_dna_seg_from_ptt(bq_ptt_file)
## Read directly from NCBI ftp site:
url <- "ftp://ftp.ncbi.nih.gov/genomes/Bacteria/Bartonella_henselae_Houston-1/NC_005956.ptt"
attempt <- 0
\dontrun{
while (attempt < 5){
  attempt <- attempt + 1
  bh <- try(read_dna_seg_from_ptt(url))
  if (!inherits(bh, "try-error")) {
    attempt <- 99
  } else {
    print(paste("Tried", attempt, "times, retrying in 5s"))
    Sys.sleep(5)
  }
}
}
## If attempt to connect to internet fails
if (!exists("bh")){
  data(barto)
  bh <- barto$dna_segs[[3]]
}

##
## Read from blast
##
bh_vs_bq_file <- system.file('extdata/BH_vs_BQ.blastn.tab',
                             package = 'genoPlotR')
bh_vs_bq <- read_comparison_from_blast(bh_vs_bq_file, color_scheme="grey")

## Plot
plot_gene_map(dna_segs=list(BH=bh, BQ=bq), comparisons=list(bh_vs_bq),
              xlims=list(c(1,50000), c(1, 50000)))


}
\keyword{file}
\keyword{IO}

\name{dna_seg}
\alias{dna_seg}
\alias{as.dna_seg}
\alias{is.dna_seg}
\title{
  DNA segment (dna_seg) class and class functions
}
\description{
  A DNA segment is a collection of genes or elements along a genome, to
  be represented on a map. These functions are class functions to
  create, convert and test dna_seg objects.
}
\usage{
dna_seg(x, ...)
as.dna_seg(df, col = "blue", fill = "blue", lty = 1, lwd = 1,
           pch = 8, cex = 1, gene_type = "arrows")
is.dna_seg(dna_seg)
}
\arguments{
  \item{x}{
    A \code{data.frame} or \code{list} that can be coerced to a data frame.
  }
  \item{\dots}{
    Arguments further passed to \code{as.dna_seg} (see below).
  }
  \item{df}{
    A data frame representing the \code{dna_seg} object. See details for
    necessary columns.
  }
  \item{col}{
    Either a color vector of the same length as \code{df} or of length
    one, to be applied to the whole object. Will be passed to
    corresponding \code{grob} object. Default to blue.
  }
  \item{fill}{
    Either a fill vector of the same length as \code{df} or of length
    one, to be applied to the whole object. Will be passed to
    corresponding \code{grob} object. Default to \code{blue}.
  }
  \item{lty}{
    A vector of the same length as \code{df} or of length one, giving
    the line type around the objects.
  }
  \item{lwd}{
    Same as \code{lty}, giving the line width.
  }
  \item{pch}{
    Same as \code{lty}, giving the character representing each
    object. Goes with \code{gene_type} points.
  }
  \item{cex}{
    Same as \code{lty}, giving the character size representing each
    object. Goes with \code{gene_type} points.
  }
  \item{gene_type}{
    Vector of the same length as \code{df} or of length one, giving the
    type of representation of each object.
  }
  \item{dna_seg}{
    Object to test.
  }
}
\details{
  Objects to be converted needs to have their first 4 columns named 
  \code{name}, \code{start}, \code{end} and \code{strand}. Extra columns
  with names \code{col}, \code{lty}, \code{lwd}, \code{pch}, \code{cex},
  \code{gene_type} will be used in the plotting process. Other extra
  columns will be kept in the object, but not used.

  \code{dna_seg} tries to build a dna_seg object from a data frame or a
  list. 

  \code{as.dna_seg} tries to build a dna_seg object from a data frame.
  
  Read functions such as \code{\link{read_dna_seg_from_tab}} and
  \code{\link{read_dna_seg_from_ptt}} also return dna_seg objects.  
}
\value{
  A comparison object for \code{comparison} and
  \code{as.comparison}. DNA seg objects are also of class
  \code{data.frame}. They contain the following columns: \code{name},
  \code{start}, \code{end}, \code{strand}, \code{col}, \code{lty}, \code{lwd},
  \code{pch}, \code{cex}, \code{gene_type}.

  A logical for \code{is.comparison}.
}
\author{
  Lionel Guy
}
\seealso{
  \code{\link{read_dna_seg_from_tab}},
  \code{\link{read_dna_seg_from_ptt}}, \code{\link{gene_types}}.
}
\examples{
## generate data
names1 <- c("feat1", "feat2", "feat3")
starts1 <- c(2, 1000, 1050)
ends1 <- c(600, 800, 1345)
strands1 <- c("-", -1, 1)
cols1 <- c("blue", "grey", "red")

## create data.frame
df1 <- data.frame(name=names1, start=starts1, end=ends1,
                  strand=strands1, col=cols1)

## with dna_seg
dna_seg1 <- dna_seg(df1)
dna_seg1
as.dna_seg(df1)

## test
is.dna_seg(dna_seg1)

## directly readable with read_dna_seg_from_tab
\dontrun{
write.table(x=dna_seg1, file="dna_seg1.tab", quote=FALSE,
            row.names=FALSE, sep="\t")
}

## with only one gene and with list, or two, and merging with c.dna_seg
gene2a <- dna_seg(list(name="feat1", start=50, end=900, strand="-", col="blue"))
genes2b <- dna_seg(data.frame(name=c("feat2", "feat3"), start=c(800, 1200),
                              end=c(1100, 1322), strand=c("+", 1),
                              col=c("grey", "red"),
                              fill=c("transparent", "grey"),
                              gene_type=c("arrows", "blocks")))
dna_seg2 <- c(gene2a, genes2b)

## test
is.dna_seg(dna_seg2)

## reading from file
dna_seg3_file <- system.file('extdata/dna_seg3.tab', package = 'genoPlotR')
dna_seg3 <- read_dna_seg_from_tab(dna_seg3_file)
is.dna_seg(dna_seg3)

}
\keyword{ data }

\name{range.dna_seg}
\alias{range.dna_seg}
\alias{range.comparison}
\alias{range.annotation}
\title{
  Range calculation
}
\description{
  Calculate the range of dna_seg and comparisons.
}
\usage{
\method{range}{dna_seg}(x, ...)
\method{range}{comparison}(x, overall=TRUE, ...)
\method{range}{annotation}(x, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
    Object to calculate the range from.
  }
  \item{overall}{
    Logical, \code{TRUE} by default. Should the range be calculated over
    the whole object? If \code{FALSE}, a range is calculated on each
    side of the comparison.
  }
  \item{\dots}{
    Unused.
  }
}
\details{
  Calculate the overall range of a \code{dna_seg}, \code{comparison} or
  an \code{annotation} object. 
}
\value{
  A numeric of length 2. For \code{comparison}, if \code{overall} is
  \code{FALSE}, a data frame with two rows and two columns, \code{xlim1}
  and \code{xlim2}.
}
\author{
  Lionel Guy
}
\seealso{
  \code{\link{dna_seg}}, \code{\link{comparison}}, \code{\link{trim}}
  for further examples.
}
\examples{
## Load data
data(three_genes)

## On dna_seg
dna_segs[[1]]
range(dna_segs[[1]])

## On comparison
comparisons[[2]]
range(comparisons[[2]])
range(comparisons[[2]], overall=FALSE)
}
\keyword{ data }

\name{middle}
\alias{middle}
\title{
  Middles of a dna_seg
}
\description{
  Returns a vector containing the middle of the genes of a
  dna_seg. Useful to prepare annotations, for example.
}
\usage{
middle(dna_seg)
}
\arguments{
  \item{dna_seg}{
    A \code{dna_seg} object.
  }
}
\value{
  A numeric vector.
}
\author{
  Lionel Guy
}
\seealso{
  \code{\link{annotation}}, \code{\link{dna_seg}}
}
\examples{

## Load data
data(barto)

## Get middles of the first dna_seg
mid <- middle(barto$dna_segs[[1]])

}
\keyword{ data }

\name{apply_color_scheme}
\alias{apply_color_scheme}
\title{
  Apply a color scheme
}
\description{
  Apply a color scheme to a numeric vector, eventually taking the
  direction into account.
}
\usage{
apply_color_scheme(x, direction = NULL, color_scheme = "grey",
decreasing = FALSE, rng = NULL, transparency = 0.5)
}
\arguments{
  \item{x}{
    A numeric, that will be used to apply a gradient of colors to a
    comparison. 
  }
  \item{direction}{
    If a red-blue scheme is choosen, the vector (composed of -1 and 1
    values and of same length as \code{x}) giving the direction of the
    comparison. 
  }
  \item{color_scheme}{
    Character. One of \code{red_blue}, \code{blue_red}, \code{grey},
    \code{gray}. 
  }
  \item{decreasing}{
    Logical. Are the values of the comparisons oriented such as the
    lower the value, the closer the relationship (e.g. e-values, gaps,
    mismatches, etc)? \code{FALSE} by default. 
  }
  \item{rng}{
    Numeric of length 2. Gives the higher and lower limit to apply a
    color scheme.  
  }
  \item{transparency}{
    Numeric of length 1, between 0 and 1, or FALSE. Should the color
    scheme use transparency, and if yes how much (ratio). 0.5 by
    default.
    Not supported on all devices.
  }
}
\details{
  A color scale is calculated, with the darker color corresponding to
  the highest values of \code{x}, or the contrary is \code{decreasing}
  is \code{TRUE}. For the moment, two schemes (red-blue and grey scale)
  are used.

  For the red-blue scale (as in ACT), the direct comparisons are
  colored in red hues, and the reversed ones in blue hues.

  This is especially useful to replace comparison values (such as BLAST
  percent identity values) by color hues.
}
\value{
  A character vector of same length as \code{x}, representing colors.
}
\references{
  Artemis Comparison Tool, \url{https://www.sanger.ac.uk/tool/artemis-comparison-tool-act/}
}
\author{
  Lionel Guy
}
\seealso{
  \code{\link{comparison}}
}
\examples{
## Load data
data(three_genes)

## Color schemes
## Greys
comparisons[[1]]$values <- c(70, 80, 90)
comparisons[[1]]$col <- apply_color_scheme(comparisons[[1]]$values,
                                           color_scheme="grey")
plot_gene_map(dna_segs=dna_segs, comparisons=comparisons)
## Red-blue
comparisons[[1]]$col <- apply_color_scheme(comparisons[[1]]$values,
                                           direction=comparisons[[1]]$direction,
                                           color_scheme="red_blue")
plot_gene_map(dna_segs=dna_segs, comparisons=comparisons)
## Decreasing
comparisons[[1]]$col <- apply_color_scheme(comparisons[[1]]$values,
                                           direction=comparisons[[1]]$direction,
                                           color_scheme="red_blue",
                                           decreasing=TRUE)
plot_gene_map(dna_segs=dna_segs, comparisons=comparisons)
## Range
comparisons[[1]]$col <- apply_color_scheme(comparisons[[1]]$values,
                                           direction=comparisons[[1]]$direction,
                                           color_scheme="red_blue",
                                           rng=c(30,100))
plot_gene_map(dna_segs=dna_segs, comparisons=comparisons)
## Transparency
x1 <- seq(100, 600, by=50)
x2 <- seq(1100, 700, by=-50)
comparisons[[2]] <- as.comparison(data.frame(start1=c(x1, x2),
                                             end1=c(x1+250, x2+300),
                                             start2=c(x1+150, x2-300)+2000,
                                             end2=c(x1+250, x2-500)+2000
                                             ))
comparisons[[2]]$col <- apply_color_scheme(1:nrow(comparisons[[2]]),
                                           comparisons[[2]]$direction,
                                           color_scheme="blue_red")
comparisons[[1]]$col <- apply_color_scheme(comparisons[[1]]$values,
                                           color_scheme="grey",
                                           transparency=0.8)
plot_gene_map(dna_segs=dna_segs, comparisons=comparisons)
comparisons[[1]]$col <- apply_color_scheme(comparisons[[1]]$values,
                                           color_scheme="grey",
                                           transparency=1)
comparisons[[2]]$col <- apply_color_scheme(1:nrow(comparisons[[2]]),
                                           comparisons[[2]]$direction,
                                           color_scheme="blue_red",
                                           transparency=0.2)
plot_gene_map(dna_segs=dna_segs, comparisons=comparisons)
}
\keyword{ data }

\name{auto_annotate}
\alias{auto_annotate}
\title{
  Auto-annotate dna_segs
}
\description{
  Annotate dna_segs in a smart way. This is especially designed for
  \code{dna_seg}s read from genbank or embl files, but can be extended for
  other uses. In short, it produces annotations from \code{dna_seg}s, grouping
  the tags for operons (atpA, atpB, atC) into one tag (atpA-C), and
  similarly for numbered genes (bep1-9).
}
\usage{
auto_annotate(dna_seg, locus_tag_pattern=NULL, names=dna_seg$gene,
              keep_genes_only=TRUE, ...)
}
\arguments{
  \item{dna_seg}{
    A \code{dna_seg} object.
  }
  \item{locus_tag_pattern}{
    \code{NULL} by default. A character giving a pattern, that is used
    to simplify names. Specially useful to transform long locus tags
    into numbers (e.g. Eco003456 becomes 3456).
  }
  \item{names}{
    A character vector with as many elements as there are rows in the
    \code{dna_seg}. By default, the \code{gene} column of the
    \code{dna_seg} is taken. Gives the names to be summarized.
  }
  \item{keep_genes_only}{
    A logical, \code{TRUE} by default. If set, the genes that have a
    name that is "-" or empty are not annotated.
  }
  \item{\dots}{
    Further arguments to be passed to \code{annotation} function, like
    \code{rot} or \code{color}.
  }
}
\value{
  An \code{annotation} object.
}
\author{
  Lionel Guy
}
\seealso{
  \code{\link{annotation}}, \code{\link{dna_seg}}.
}
\examples{
## Prepare dna_seg
names <- paste("Eco", sprintf("\%04d", 1:20), sep="")
gene <- c("-", "atpC", "atpB", "atpA", "atp2", 
          "-", "-", "cda1", "cda2", "cda3",
          "vcx23", "vcx22", "vcx21", "cde20",
          "-", "gfrU", "gfrT", "gfrY", "gfrX", "gfrW")
ds <- dna_seg(data.frame(name=names, start=(1:20)*3, end=(1:20)*3+2,
                         strand=rep(1, 20), gene=gene,
                         stringsAsFactors=FALSE))
## Original annotation
annot1 <- annotation(x1=middle(ds), text=ds$gene, rot=30)
## auto_annotate with various options
annot2 <- auto_annotate(ds)
annot3 <- auto_annotate(ds, keep_genes_only=FALSE, rot=45)
annot4 <- auto_annotate(ds, keep_genes_only=FALSE,
                        locus_tag_pattern="Eco", col="red")
## Plot
plot_gene_map(list(ds, ds, ds, ds),
              annotations=list(annot1, annot2, annot3, annot4))

}
\keyword{ data }

\name{comparison}
\alias{comparison}
\alias{as.comparison}
\alias{is.comparison}
\title{
  Comparison class and class functions
}
\description{
  A comparison is a collection of similarities, representing the
  comparison between two DNA segments. These functions are class
  functions to create, convert and test comparison objects.
}
\usage{
comparison(x)
as.comparison(df)
is.comparison(comparison)
}
\arguments{
  \item{x}{
    Can be a \code{list} or \code{data.frame} object. See the
    details for the columns in the \code{data.frame}.
  }
  \item{df}{
    A \code{data.frame} object. See details for the required columns.
  }
  \item{comparison}{
    Any object to test.
  }
}
\details{
  Objects (either data frames or lists) should have at least named
  elements \code{start1}, \code{end1}, \code{start2} and \code{end2}. In
  addition, it can take a \code{col}or column. Additional numeric columns can
  be used for color-coding (via \code{apply_color_scheme}.

  \code{comparison} tries to build a comparison object from either a
  data frame or a list, \code{as.comparison} accepts only data.frames.

  \code{is.comparison} returns \code{TRUE} if the object tested is a
  comparison object.
  
  Read functions such as \code{\link{read_comparison_from_tab}} and
  \code{\link{read_comparison_from_blast}} also return comparison objects.
}
\value{
  A comparison object for \code{comparison} and
  \code{as.comparison}. Comparison objects are also of class
  \code{data.frame}. They contain the columns  \code{start1},
  \code{end1}, \code{start2}, \code{end2}, \code{direction} and
  \code{col} (color).

  A logical for \code{is.comparison}.
}
\author{
  Lionel Guy
}
\seealso{
  \code{\link{dna_seg}}, \code{\link{read_comparison_from_tab}},
  \code{\link{read_comparison_from_blast}},
  \code{\link{trim.comparison}}, \code{\link{reverse.comparison}}.
}
\examples{
## Get some values
starts1 <- c(2, 1000, 1050)
ends1 <- c(600, 800, 1345)
starts2 <- c(50, 800, 1200)
ends2 <- c(900, 1100, 1322)

## From a data.frame
comparison1 <- as.comparison(data.frame(start1=starts1, end1=ends1,
                                        start2=starts2, end2=ends2))
comparison1
is.comparison(comparison1)
is.data.frame(comparison1)
comparison(data.frame(start1=starts1, end1=ends1,
                      start2=starts2, end2=ends2))
## From a list
comparison(list(start1=starts1, end1=ends1,
                start2=starts2, end2=ends2))

## From a file
comparison2_file <- system.file('extdata/comparison2.tab',
                                package = 'genoPlotR')
comparison2 <- read_comparison_from_tab(comparison2_file)

}
\keyword{ data }

\name{annotation}
\alias{annotation}
\alias{as.annotation}
\alias{is.annotation}
\title{
  Annotation class and class functions
}
\description{
  An annotation describes a DNA segment. It has labels attached to
  positions. Each label can be attached to a single position or to a
  range. 
}
\usage{
annotation(x1, x2 = NA, text, rot = 0, col = "black")
as.annotation(df, x2 = NA, rot = 0, col = "black")
is.annotation(annotation)
}
\arguments{
  \item{x1}{
    Numeric. A vector giving the first or only position of the
    label. Mandatory. 
  }
  \item{x2}{
    Numeric. A vector of the same length as \code{x1}. If a row (or the
    whole column is  \code{NA}, then the annotation(s) will be attached
    to \code{x0}. Else, the annotation will be attached to the range
    between both positions. \code{NA} by default.
  }
  \item{text}{
    Character of the same length as \code{x0}. Gives the text of the
    labels. Mandatory.
  }
  \item{rot}{
    Numeric of the same length as \code{x0}. Gives the rotation, in
    degrees, of the labels. 0 by default.
  }
  \item{col}{
    Vector of the same length as \code{x0}. The color of the
    labels. \code{black} by default.
  }
  \item{df}{
    A data frame to convert to an annotation object. Should have at
    least columns \code{x1} and \code{text}.
  }
  \item{annotation}{
    An object to test.
  }
}
\details{
  An \code{annotation} object is a data frame with columns \code{x0},
  \code{x1}, \code{text}, \code{col} and \code{rot}. They give,
  respectively, the first (or only) position, eventually the second
  position, the text, the color and the rotation of the annotation. When
  plotted with \code{plot_gene_map}, it will add an annotation row on
  top of the first \code{dna_seg}. Labels for which only one position is
  given will be centered on that position. Labels for which two
  positions are given are linked by an horizontal square bracket and the
  label is plotted in the middle of the positions.
}
\value{
  \code{annotation} and  \code{as.annotation} return an annotation object.
  \code{is.annotation} returns a logical.
}
\author{
  Lionel Guy
}
\seealso{
  \code{\link{plot_gene_map}}, \code{\link{middle}}.
}
\examples{
## loading data
data(three_genes)

## Calculating middle positions
mid_pos <- middle(dna_segs[[1]])

# Create first annotation
annot1 <- annotation(x1=mid_pos, text=dna_segs[[1]]$name)
plot_gene_map(dna_segs=dna_segs, comparisons=comparisons, annotations=annot1)

## Exploring options
annot2 <- annotation(x1=c(mid_pos[1], dna_segs[[1]]$end[2]),
                     x2=c(NA, dna_segs[[1]]$end[3]),
                     text=c(dna_segs[[1]]$name[1], "region1"),
                     rot=c(30, 0), col=c("grey", "black"))
plot_gene_map(dna_segs=dna_segs, comparisons=comparisons,
              annotations=annot2, annotation_height=1.3)

## Annotations on all the segments
annots <- lapply(dna_segs, function(x){
  mid <- middle(x)
  annot <- annotation(x1=mid, text=x$name, rot=30)
})
plot_gene_map(dna_segs=dna_segs, comparisons=comparisons,
              annotations=annots, annotation_height=1.8, annotation_cex=1)

##
## Using a bigger dataset from a 4-genome comparison
##
data(barto)
## Adding a tree
tree <- newick2phylog("(BB:2.5,(BG:1.8,(BH:1,BQ:0.8):1.9):3);")
## Showing several subsegments
xlims2 <- list(c(1445000, 1415000, 1380000, 1412000),
               c(  10000,   45000,   50000,   83000, 90000, 120000),
               c(  15000,   36000,   90000,  120000, 74000,  98000),
               c(   5000,    82000))
## Adding annotations for all genomes, allow segments to be placed out
## of the longest segment
annots <- lapply(barto$dna_segs, function(x){
  mid <- middle(x)
  annot <- annotation(x1=mid, text=x$name, rot=30)
  # removing gene names starting with "B" and keeping 1 in 4
  idx <- grep("^[^B]", annot$text, perl=TRUE)
  annot[idx[idx \%\% 4 == 0],] 
})
plot_gene_map(barto$dna_segs, barto$comparisons, tree=tree,
              annotations=annots,
              xlims=xlims2,
              limit_to_longest_dna_seg=FALSE,
              dna_seg_scale=TRUE)


}
\keyword{ data }

\name{mauve_bbone}
\docType{data}
\alias{mauve_bbone}
\alias{bbone}
\title{Mauve backbone of 4 Bartonella genomes}
\description{
  The result of a multiple genome alignment with Mauve. 
}
\usage{
data(mauve_bbone)
}
\format{
  \code{bbone}, a list of two dataframes, representing the regions which
  are conserved in at least two genomes:
  \itemize{
    \item{\code{dna_segs}}{which is a list of 4 \code{dna_seg}
      objects, containing the mauve blocks for each genome.}
    \item{\code{comparisons}}{which is a list of 3 \code{comparison}
      objects.}
  }
  A bash script to obtain the same file as in the data is
  available in the \code{extdata} folder of the package. Find its
  location by running 
  \code{system.file('extdata/mauve.sh', package = 'genoPlotR')}.
    
  The resulting backone file can then be read with
  \code{\link{read_mauve_backbone}}.
}
\references{
  Mauve: http://asap.ahabs.wisc.edu/mauve/
}

\examples{
data(mauve_bbone)
plot_gene_map(bbone$dna_segs, bbone$comparisons)
}
\keyword{datasets}

\name{chrY_subseg}
\docType{data}
\alias{chrY_subseg}
\title{Comparisons of subsegments of the Y chromosome in human and chimp}
\description{
  A subsegment of the Y chromosome in Homo sapiens and Pan troglodytes,
  to illustrate support for exons and introns.
}
\usage{
data(chrY_subseg)
}
\format{
  A list of two data frames, representing the Y segment in the two
  species, and containing:
  \itemize{
    \item{\code{dna_segs}}{which is a list of two \code{dna_seg}
      objects, containing each three rows (or genes).}
    \item{\code{comparison}}{which is a list of one \code{comparison}
      objects.}
  }
}
\details{
Header for the Homo sapiens genbank file:
LOCUS       NC_000023             220001 bp    DNA     linear   CON 10-JUN-2009
DEFINITION  Homo sapiens chromosome X, GRCh37 primary reference assembly.
ACCESSION   NC_000023 REGION: 2600000..2820000 GPC_000000047

Header for the Pan troglodytes file:
LOCUS       NC_006491             220001 bp    DNA     linear   CON 18-SEP-2006
DEFINITION  Pan troglodytes chromosome X, reference assembly (based on
            Pan_troglodytes-2.1).
ACCESSION   NC_006491 REGION: 2620000..2840000

}
\examples{
data(chrY_subseg)
plot_gene_map(chrY_subseg$dna_segs, chrY_subseg$comparison, dna_seg_scale=TRUE,
              scale=FALSE)
}
\keyword{datasets}

\name{human_nt}
\alias{human_nt}
\title{
  Human-readable nucleotide scale
}
\description{
  Return a human readable list from a nucleotide position or lenght. 
}
\usage{
human_nt(nt, signif = FALSE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{nt}{
    A nucleotide position
  }
  \item{signif}{
    Either a logical or an integer. If \code{FALSE} (default), \code{nt}
    is not rounded. Else, it returns \code{signif} significant digits.
  }
}
\details{
  Return a nucleotide value in nt, kb, Mb or Gb, according to the value given.
  This is particularly useful to display nice scales without too many 
  trailing zeros.
}
\value{
  Returns a list with 4 elements
  \item{n}{A numeric value corresponding to \code{nt} divided by
    \code{mult} (see below).}
  \item{tag}{A character, giving the multiplier used in text.}
  \item{mult}{The muliplier used, in numeric value.}
  \item{text}{A character, giving the value in a human readable format.}

}
\author{
  Lionel Guy
}
\examples{
human_nt(123456)
human_nt(123456, signif=2)
human_nt(123456890, signif=2)
}
\keyword{ data }

\name{seg_plot}
\alias{seg_plot}
\alias{as.seg_plot}
\alias{is.seg_plot}
\title{
  seg_plot class and class functions
}
\description{
  An seg_plot is an object to plot data associated to a \code{dna_seg}
  object. It is a \code{list} with mandatory and optional arguments. The
  main arguments are \code{func}, which is a function returning a
  \code{\link{grob}} or a \code{\link{gList}}, and \code{args}, which
  are arguments to be passed to this function.
}
\usage{
seg_plot(func,
         args = NULL,
         xargs = c("x", "x0", "x1", "x2", "v"),
         yargs = c("y", "y0", "y1", "y2", "h"),
         ylim = NULL)
as.seg_plot(seg_plot)
is.seg_plot(seg_plot)
}
\arguments{
  \item{func}{
    Mandatory, with no defaults. A function that returns a \code{grob}
    object. See \code{\link{grid}} documentation to find ready-made
    functions. User-defined functions work too.
  }
  \item{args}{
    A list, \code{NULL} by default. The arguments that will be passed to
    the function. It is recommended that all arguments are named.
  }
  \item{xargs}{
    A vector giving the names of which of the arguments in \code{args}
    are defining the x-axis. Used, among others, by the function
    \code{\link{trim.seg_plot}}. By default, gives the most common
    x-defining arguments of the \code{grid} functions (x, x0, x1, x2, v). 
  }
  \item{yargs}{
    A vector giving the names of which of the arguments in \code{args}
    are defining the y-axis. Used when plotting the graphs to define
    a sensible \code{ylim} if not defined. By default, gives the most common
    y-defining arguments of the \code{grid} functions (y, y0, y1, y2, h). 
  }
  \item{ylim}{
    A numeric vector of length 2, defining the range of the plot when
    drawn with \code{\link{plot_gene_map}}. Derived from \code{yargs} if
    not set.
  }
  \item{seg_plot}{
    In \code{as.seg_plot}, a \code{list} object to convert to
    \code{seg_plot}. See details below.

    In \code{is.seg_plot}, an object to test.
  }
}
\value{
  \code{seg_plot} and \code{as.seg_plot} return a \code{seg_plot} object.
  \code{is.seg_plot} returns a logical.
}
\details{
  A \code{seg_plot} object is an object describing how to plot data
  associated to a \code{dna_seg}. It is a list composed of a function,
  arguments to pass to this function, two arguments to define which of
  those define x and y, and an eventual \code{ylim} to limit the
  plotting to a certain range when plotting.

  The function \code{func} should return a \code{grob} object, or a
  \code{gList} list of \code{grob}s. The predefined functions of
  \code{grid}, such as \code{linesGrob}, \code{pointsGrob},
  \code{segmentsGrob}, \code{textGrob} or \code{polygonGrob} can be
  used, or user-defined functions can be defined.

  The arguments in \code{args} should correspond to arguments passed to
  \code{func}. For example, if \code{func = pointsGrob}, \code{args}
  could contain the elements \code{x = 10:1}, \code{y = 1:10}. It will
  often also contain a \code{gp} element, the result of a call to the
  \code{\link{gpar}} function, to control graphical aspects of the plot
  such as color, fill, line width and style, fonts, etc.
}
\author{
  Lionel Guy
}
\seealso{
  \code{\link{plot_gene_map}}, \code{\link{trim.seg_plot}}.
}
\examples{
## Using the existing pointsGrob
x <- 1:20
y <- rnorm(20)
sp <- seg_plot(func=pointsGrob, args=list(x=x, y=y,
                                  gp=gpar(col=1:20, cex=1:3)))
is.seg_plot(sp)
## Function seg_plot(...) is identical to as.seg_plot(list(...))
sp2 <- as.seg_plot(list(func=pointsGrob, args=list(x=x, y=y,
                                           gp=gpar(col=1:20, cex=1:3))))
identical(sp, sp2)
## For the show, plot the obtained result
grb <- do.call(sp$func, sp$args)
## Trim the seg_plot
sp_trim <- trim(sp, c(3, 10))
## Changing color and function "on the fly"
sp_trim$args$gp$col <- "blue"
sp_trim$func <- linesGrob
grb_trim <- do.call(sp_trim$func, sp_trim$args)
## Now plot
plot.new()
pushViewport(viewport(xscale=c(0,21), yscale=c(-4,4)))
grid.draw(grb)
grid.draw(grb_trim)

## Using home-made function
triangleGrob <- function(start, end, strand, col, ...) {
  x <- c(start, (start+end)/2, end)
  y1 <- 0.5 + 0.4*strand
  y <- c(y1, rep(0.5, length(y1)), y1)
  polygonGrob(x, y, gp=gpar(col=col), default.units="native",
              id=rep(1:7, 3))
}
start <- seq(1, 19, by=3)+rnorm(7)/3
end <- start + 1 + rnorm(7)
strand <- sign(rnorm(7))
sp_tr <- seg_plot(func=triangleGrob,
                  args=list(start=start, end=end, strand=strand,
                    col=1:length(start)), xargs=c("start", "end"))
grb_tr <- do.call(sp_tr$func, sp_tr$args)
plot.new()
pushViewport(viewport(xscale=c(1,22), yscale=c(-2,2)))
grid.draw(grb_tr)
## Trim
sp_tr_trim <- trim(sp_tr, xlim=c(5, 15))
str(sp_tr_trim)
## If the correct xargs are not indicated, trimming won't work
sp_tr$xargs <- c("x")
sp_tr_trim2 <- trim(sp_tr, xlim=c(5, 15))
identical(sp_tr_trim, sp_tr_trim2)

y1 <- convertY(grobY(grb_tr, "south"), "native")
y2 <- convertY(grobY(grb_tr, "north"), "native")
heightDetails(grb)
grb

## Applying it to plot_gene_maps
data(three_genes)

## Build data to plot
xs <- lapply(dna_segs, range)
colors <- c("red", "blue", "green")

seg_plots <- list()
for (i in 1:length(xs)){
  x <- seq(xs[[i]][1], xs[[i]][2], length=20)
  seg_plots[[i]] <- seg_plot(func=pointsGrob,
                             args=list(x=x, y=rnorm(20)+2*i,
                               default.units="native", pch=3,
                               gp=gpar(col=colors[i], cex=0.5)))
}
plot_gene_map(dna_segs, comparisons,
              seg_plots=seg_plots,
              seg_plot_height=0.5,
              seg_plot_height_unit="inches",
              dna_seg_scale=TRUE)

## A more complicated example
data(barto)
tree <- newick2phylog("(BB:2.5,(BG:1.8,(BH:1,BQ:0.8):1.9):3);")
## Showing several subsegments per genome
xlims2 <- list(c(1445000, 1415000, 1380000, 1412000),
               c(  10000,   45000,   50000,   83000, 90000, 120000),
               c(  15000,   36000,   90000,  120000, 74000,  98000),
               c(   5000,    82000))

## Adding fake data in 1kb windows
seg_plots <- lapply(barto$dna_segs, function(ds){
  x <- seq(1, range(ds)[2], by=1000)
  y <- jitter(seq(100, 300, length=length(x)), amount=50)
  seg_plot(func=linesGrob, args=list(x=x, y=y, gp=gpar(col=grey(0.3), lty=2)))
})
plot_gene_map(barto$dna_segs, barto$comparisons, tree=tree,
              seg_plots=seg_plots,
              seg_plot_height=0.5,
              seg_plot_height_unit="inches",
              xlims=xlims2,
              limit_to_longest_dna_seg=FALSE,
              dna_seg_scale=TRUE,
              main="Random plots for the same segment in 4 Bartonella genomes")

}
\keyword{ data }

\name{gene_types}
\alias{gene_types}
\title{
  Gene types
}
\description{
  Returns a vector containing the available gene types. In addition to
  these gene types, the user can provide graphical functions that return
  a list or a single \code{grob} object. 
}
\usage{
gene_types(auto = TRUE)
}
\arguments{
  \item{auto}{
    Logical. Should type "auto" be added?
  }
}
\value{
  A character vector.
}
\details{
  \code{dna_seg}s may contain one character column
  \code{gene_type}. Elements in this column should either be one of the
  predefined gene types, or refer to a graphical function that has
  exactly the same name and that returns a \code{grob} or a \code{gList}
  object.

  A gene object (i.e. a single row of a \code{dna_seg}) is passed to the
  graphical function, as well as the contents of the \dots. The start
  and line width of an element can thus be accessed via
  \code{gene$start} and \code{gene$lwd}. Extra columns that would be
  added in the \code{dna_seg} can be used similarly. Extra arguments can
  also be globally passed via \dots when calling \code{plot_gene_map}.
}
\author{
  Lionel Guy
}
\seealso{
  \code{\link{plot_gene_map}}, \code{\link{dna_seg}} 
}
\examples{
## To view pre-coded gene types:
gene_types()

## Load data
data(barto)
n <- length(gene_types(auto=FALSE))

## Get a small subset from the barto dataset
dna_seg <- barto$dna_segs[[3]][1:n,]
plot_gene_map(list(dna_seg))

## Change gene_types and plot again
dna_seg$gene_type <- gene_types(auto=FALSE)
dna_seg$fill <- rainbow(n)
dna_seg_r <- dna_seg
dna_seg_r$strand <- -dna_seg$strand

## Add an annotation
annot <- annotation(middle(dna_seg), text=dna_seg$gene_type, rot=45,
                    col=dna_seg$col)

## Plot
plot_gene_map(list(dna_seg, dna_seg_r), annotations=list(annot, annot),
              annotation_height=5, dna_seg_line=grey(0.7))


## Using home-made graphical functions
## Data
data(three_genes)

## Functions returning grobs.
## Creates a triangle
triangleGrob <- function(gene, ...) {
  x <- c(gene$start, (gene$start+gene$end)/2, gene$end)
  y1 <- 0.5 + 0.4*gene$strand
  y <- c(y1, 0.5, y1)
  polygonGrob(x, y, gp=gpar(fill=gene$fill, col=gene$col, lty=gene$lty,
                      lwd=gene$lwd), default.units="native")
}

## Draws a star. Note that the limits of the dna_seg region are
## voluntarily not respected
starGrob <- function(gene, ...){
  ## Coordinates for the star
  x <- sin(((0:5)/2.5)*pi)*(gene$end-gene$start)/2 + (gene$end+gene$start)/2
  y <- cos(((0:5)/2.5)*pi)*gene$strand*2 + 0.5
  idx <- c(1, 3, 5, 2, 4, 1)
  ## Attribute line_col only if present in the gene
  line_col <- if (!is.null(gene$line_col)) gene$line_col else gene$col
  ## Having a conditional transparency, depending on a length cut-off
  ## passed via dots
  length_cutoff <- list(...)$length_cutoff
  if (!is.null(length_cutoff)){
    alpha <- if ((gene$end-gene$start) < length_cutoff)  0.3 else  0.8
  } else alpha <- 1
  
  ## Grobs
  g <- polygonGrob(x[idx], y[idx], gp=gpar(fill=gene$col, col=line_col,
                                     lty=gene$lty, lwd=gene$lwd, alpha=alpha),
                   default.units="native")
  t <- textGrob(label="***", x=(gene$end+gene$start)/2, y=0.5,
                default.units="native")
  gList(g, t)
}

## Replacing the standard types
dna_segs[[1]]$gene_type <- "triangleGrob"
dna_segs[[2]]$gene_type <- "starGrob"
## Adding more variables
dna_segs[[2]]$line_col <- c("black", grey(0.3), "blue")
## Mix of several types on the same line
dna_segs[[3]]$gene_type <- c("starGrob", "triangleGrob", "arrows")

## Plot
plot_gene_map(dna_segs, comparisons, length_cutoff=600)


}
\keyword{ data }

\name{c.dna_seg}
\alias{c.dna_seg}
\title{
  Concatenate dna_seg objects
}
\description{
  Concatenate dna_seg objects.
}
\usage{
\method{c}{dna_seg}(...)
}
\arguments{
  \item{\dots}{
    \code{dna_seg}s to be concatenated.
  }
}
\value{
  A \code{dna_seg} object
}
\author{
  Lionel Guy
}
\seealso{
  \code{\link{dna_seg}}
}
\examples{
## load data
data(three_genes)

dna_segs[1:2]
c(dna_segs[[1]], dna_segs[[2]])

}
\keyword{ data }

\name{trim}
\alias{trim}
\alias{trim.default}
\alias{trim.dna_seg}
\alias{trim.comparison}
\alias{trim.annotation}
\alias{trim.seg_plot}
\title{
  Trimming data frames or more complex objects with >= 2 numeric columns 
}
\description{
  Trims data frames with 2 or more numeric columns using a
  xlim. \code{xlim}(s) are as used to filter rows whose numeric values are
  included in this interval.
}
\usage{
trim(x, ...)
\method{trim}{default}(x, xlim = NULL, ...)
\method{trim}{dna_seg}(x, xlim = NULL, ...)
\method{trim}{comparison}(x, xlim1 = c(-Inf, Inf), xlim2 = c(-Inf, Inf), ...)
\method{trim}{annotation}(x, xlim = NULL, ...)
\method{trim}{seg_plot}(x, xlim = NULL, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
    An object to trim,. generally a data frame or a matrix, or a
    \code{seg_plot} object.
  }
  \item{xlim}{
    A numeric of length 2. In a general case, the rows whose values are
    included in this interval are returned.
  }
  \item{\dots}{
    Unused.
  }
  \item{xlim1}{
    A numeric of length 2. In the case of comparison, where the
    comparison can be filtered on two sides, the interval to filter the
    first side. 
  }
  \item{xlim2}{
    A numeric of length 2. The interval to filter the second side.
  }
}
\details{
  In the case where \code{x} is a \code{seg_plot} object, the function
  uses the \code{xargs} argument to define what are the vectors defining
  the x position (they should be the same length). Then, all the
  arguments (including those inside an eventual \code{gp} argument) that
  are the same length as the x vectors are trimmed, so that only the
  rows for which the x values are inside the \code{xlim} argument are kept.
}
\value{
  Returns the same object as input, with the rows (or subset)
  corresponding to the given interval.
}
\author{
  Lionel Guy
}
\seealso{
  \code{\link{dna_seg}}, \code{\link{comparison}}, \code{\link{seg_plot}}.

}
\examples{
## Load
data(barto)
xlim_ref <- c(10000, 45000)
## Seg 2 (ref)
barto$dna_segs[[2]] <- trim(barto$dna_segs[[2]], xlim=xlim_ref)
## Seg 1
barto$comparisons[[1]] <- trim(barto$comparisons[[1]], xlim2=xlim_ref)
xlim1 <- range(barto$comparisons[[1]], overall=FALSE)$xlim1
barto$dna_segs[[1]] <- trim(barto$dna_segs[[1]], xlim=xlim1)
## Seg 3
barto$comparisons[[2]] <- trim(barto$comparisons[[2]], xlim1=xlim_ref)
xlim3 <- range(barto$comparisons[[2]], overall=FALSE)$xlim2
barto$dna_segs[[3]] <- trim(barto$dna_segs[[3]], xlim=xlim3)
## Seg 4
barto$comparisons[[3]] <- trim(barto$comparisons[[3]], xlim1=xlim3)
xlim4 <- range(barto$comparisons[[3]], overall=FALSE)$xlim2
barto$dna_segs[[4]] <- trim(barto$dna_segs[[4]], xlim=xlim4)
## Plot
plot_gene_map(barto$dna_segs, barto$comparisons)

## With seg_plot
x <- 1:20
y <- rnorm(20)
sp <- seg_plot(func=pointsGrob, args=list(x=x, y=y,
                                  gp=gpar(col=1:20, cex=1:3)))
## Trim 
sp_trim <- trim(sp, c(3, 10))
str(sp_trim)
range(sp_trim$arg$x)
}
\keyword{ data }

\name{three_genes}
\docType{data}
\alias{three_genes}
\alias{dna_segs}
\alias{comparisons}
\title{Three genes data set}
\description{
  A set of three made-up genes, compared in three chromosomes.
}
\usage{
data(three_genes)
}
\format{
  Two dataframes, representing the three genes in three DNA
  segments:
  \itemize{
    \item{\code{dna_segs}}{which is a list of three \code{dna_seg}
      objects, containing each three rows (or genes).}
    \item{\code{comparisons}}{which is a list of two \code{comparison}
      objects.}
  }
}
\examples{
data(three_genes)
plot_gene_map(dna_segs, comparisons)
}
\keyword{datasets}

\name{barto}
\docType{data}
\alias{barto}
\title{Comparison of 4 Bartonella genomes}
\description{
  Comparison of 4 Bartonella genomes by BLAST. 
}
\usage{
data(barto)
}
\format{
  \code{barto}, a list of three dataframes, representing the four genomes
  and their pairwise comparisons:
  \itemize{
    \item{\code{dna_segs}}{which is a list of 4 \code{dna_seg}
      objects, containing all the protein genes for each genome. Obtained
      by reading ptt files downloaded from NCBI with
      \code{read_dna_seg_from_ptt}}.
    \item{\code{comparisons}}{which is a list of 3 \code{comparison}
      objects, obtained by doing genome-to-genome (fasta files) BLASTS,
      and then reading the resulting tab files with
      \code{read_comparison_from_blast}.}
    \item{\code{rnt_segs}}{which is a list of 4 \code{dna_seg} objects,
      containing all the RNA genes of the four genomes. Obtained by
      reading rnt files downloaded from NCBI with
      \code{read_dna_seg_from_ptt}}.
  }
  A bash script to obtain the same file as in the dataset is
  available in the \code{extdata} folder of the package. Find its
  location by running 
  \code{system.file('extdata/barto.sh', package = 'genoPlotR')}.
}
\references{
  BLAST: http://www.ncbi.nlm.nih.gov/blast/
}
\examples{
data(barto)
plot_gene_map(barto$rnt_segs, barto$comparisons, gene_type="blocks")
}
\keyword{datasets}

\name{plot_gene_map}
\alias{plot_gene_map}
\title{
  Plot gene and genome maps
}
\description{
  This plotting function represents linearly DNA segments and their
  comparisons. It will plot one line per DNA segment, eventually
  separated by the comparisons. In addition, a tree can be plotted on
  the left of the plot, and annotations on the top row. Since this is a
  grid plot, it can be placed into other graphics, or modified subsequently.
}
\usage{
plot_gene_map(dna_segs,
              comparisons = NULL,
              tree = NULL,
              tree_width = NULL,
              tree_branch_labels_cex = NULL,
              tree_scale = FALSE,
              legend = NULL, 
              annotations = NULL, 
              annotation_height = 1, 
              annotation_cex = 0.8, 
              seg_plots=NULL,    # user-defined plots
              seg_plot_height=3, # height of plots (in lines)
              seg_plot_height_unit="lines", # unit of preceding
              seg_plot_yaxis=3, # if non-null or non false, ticks
              seg_plot_yaxis_cex=scale_cex,
              xlims = NULL,
              offsets = NULL,
              minimum_gap_size = 0.05,
              fixed_gap_length = FALSE,
              limit_to_longest_dna_seg = TRUE,
              main = NULL, 
              main_pos = "centre", 
              dna_seg_labels = NULL, 
              dna_seg_label_cex=1,
              dna_seg_label_col="black",
              gene_type = NULL,
              arrow_head_len = 200,
              dna_seg_line = TRUE,
              scale = TRUE, 
              dna_seg_scale = !scale,
              n_scale_ticks=7,
              scale_cex=0.6,
              global_color_scheme = c("auto", "auto", "blue_red", 0.5),
              override_color_schemes = FALSE,
              plot_new=TRUE,
              debug = 0,
              ...)
}
\arguments{
  \item{dna_segs}{
    A list of \code{dna_seg} objects. Mandatory.
  }
  \item{comparisons}{
    A list of \code{comparison} objects. Optional. 
  }
  \item{tree}{
    A tree, under the form of a \code{\link[ade4]{phylog}} object. If
    specified, takes place at the left of the tags. See details below
    for more information.
  }
  \item{tree_width}{
    Numeric. The width of the tree area in the plot, in inches. By
    default, takes 20 percent of the total plot.
  }
  \item{tree_branch_labels_cex}{
    Numeric or \code{NULL} (default). If the tree provided contains
    node annotations, they will be displayed with this \code{cex}. If
    equal to 0, node annotations are not displayed.
  }
  \item{tree_scale}{
    Logical. Plot a scale for the tree? Default is FALSE.
  }
  \item{legend}{
    Yet unimplemented.
  }
  \item{annotations}{
    An \code{annotation} object or a list of annotation objects. See
    details. Optional.
  }
  \item{annotation_height}{
    Numeric. The height, in lines, of the annotation line. One by
    default, if \code{annotation is defined}.
  }
  \item{annotation_cex}{
    Numeric. The \code{cex} (i.e. the character expansion) of the
    annotation line. 
  }
  \item{seg_plots}{
    A list of \code{seg_plot} objects of the length as \code{dna_segs},
    a single \code{seg_plot} or \code{NULL} (default). To draw plots
    associated to a \code{dna_seg}. See \code{\link{seg_plot}} for more
    information and some examples.
  }
  \item{seg_plot_height}{
    The height of the \code{seg_plot} regions. By default, equals to 3
    (lines, see next argument).
  }
  \item{seg_plot_height_unit}{
    The unit of the height of the \code{seg_plot} regions. Should be a
    valid \code{unit}. See the \code{grid} documentation for more
    information. If equals to "null", then the height will be calculated
    as a proportion of the comparison region (i.e. 0.5 means the
    \code{seg_plot} region will be half the size of the \code{comparison}. 
  }
  \item{seg_plot_yaxis}{
    Can be \code{NULL}, \code{FALSE} or a numercic. In the first two
    cases, no y-axis is drawn for the \code{seg_plot}s. If numeric, a
    axis is drawn with approximately that number of ticks. 
  }
  \item{seg_plot_yaxis_cex}{
    The character expansion of the \code{seg_plot_yaxis} labels. Equals
    to \code{scale_cex} by default.
  }
  \item{xlims}{
    A list with as many elements as there are \code{dna_seg}s, or
    \code{NULL}. If \code{NULL}, the whole segment will be
    represented. If a list, each element of the list is a numeric
    vector, representing pairs of  left and right limits for each
    subsegment. See details. 
  }
  \item{offsets}{
    A list or a vector with as many elements as there are
    \code{dna_seg}s, or \code{NULL}. If is a numeric vector, gives the
    offset of the first subsegment. If is a list, each element should
    have the same length as there are subsegments (see
    \code{xlims}). Gives then the length of each gap. If \code{NULL},
    the size of the gaps is optimized to minimize the lengths of the
    comparisons. See details.
  }
  \item{minimum_gap_size}{
    A numeric. How much of the plotting region should a gap be, at
    least. Default is 0.05 (20\% of the plotting region).
  }
  \item{fixed_gap_length}{
    Should the gaps have a fixed length? Otherwise, the gap length will
    be optimized to minimize the size of comparisons. \code{FALSE} by
    default. 
  }
  \item{limit_to_longest_dna_seg}{
    A logical. Should the plot be restricted to the longest
    \code{dna_seg}? If no, the other segments can be extended to better
    fit comparisons.
  }
  \item{main}{
    A character. Main title of the plot.
  }
  \item{main_pos}{
    Position of the main title. One of \code{centre}, \code{left} or
    \code{right}.
  }
  \item{dna_seg_labels}{
    A character, same length as \code{dna_segs}. The names of the
    segments. If \code{NULL}, the names of \code{dna_segs} will be
    used, if available. Else, no name are plotted. If a \code{tree} is
    given, names must exist either in \code{dna_seg_labels} or in the
    names of \code{dna_segs}.
  }
  \item{dna_seg_label_cex}{
    A numeric. The character size for the DNA segments labels, or tree
    labels. Default is 1.
  }
  \item{dna_seg_label_col}{
    A color, of length 1 or of the same length as \code{dna_segs}. Gives
    the color of the labels. Default is black.
  }
  \item{gene_type}{
    A character. Describes the type of representation of genes or
    \code{dna_seg} elements. See details.
  }
  \item{arrow_head_len}{
    A numeric. Gives the length of arrow heads for gene type
    "arrows". The arrow head extends at maximum at half of the gene. Set
    to \code{Inf} to have all arrow heads covering the half of the
    gene. 200 by default.
  }
  \item{dna_seg_line}{
    A vector, either logical or giving colors, of length 1 or of same
    length as \code{dna_segs}. Should the line in the middle of the
    segments be drawn, and if yes, in what color. \code{TRUE} by
    default, which gives black lines. \code{FALSE} (logical, or as a
    string) results in no plotting.
  }
  \item{scale}{
    A logical. Should the scale be displayed on the plot. \code{TRUE} by
    default. 
  }
  \item{dna_seg_scale}{
    A logical, of length one or of the same length as
    \code{dna_segs}. Should a scale be displayed below each or all dna
    segments, respectively. \code{!scale} by default. 
  }
  \item{n_scale_ticks}{
    A integer. The (approximate) number of ticks on the longest
    segment. Default: 7. 
  }
  \item{scale_cex}{
    A numeric. The character size for the scale labels. Default is 1.
  }
  \item{global_color_scheme}{
    A character of length 4. If no \code{col} column is present on any
    comparison or is \code{override_color_schemes} is set, apply a
    global color scheme over all comparions. See below for more details.
    \code{c("auto", "auto", "blue_red")} by default. 
  }
  \item{override_color_schemes}{
    A logical. If \code{TRUE}, apply a global color scheme even if there
    are comparisons that have \code{col} columns. \code{FALSE} by
    default. 
  }
  \item{plot_new}{
    Logical. Produce a new plot? If \code{TRUE}, uses
    \code{grid.newpage} before plotting.
  }
  \item{debug}{
    A numeric. If > 0, only that number of element will be plotted for
    each \code{dna_seg} and \code{comparison}.
  }
  \item{\dots}{
    Further arguments to be passed to user-defined graphical functions.
  }
}
\details{
  One line is plotted per \code{\link{dna_seg}}. Eventually, the space 
  between the lines will be filled with the
  \code{\link{comparison}}s. \code{dna_seg}s can be annotated with
  \code{\link{annotation}}s, and accompagnying data can be plotted using
  \code{\link{seg_plot}}. 
  
  A phylogenetic tree (a \code{phylog} object from package \code{ade4})
  can be drawn at the left of the plot. The tree does not need to be
  ordered as the \code{dna_seg_labels}, but a permutation of the tree
  with that order should exist. If the tree is large, the number of
  permutations become too large, and the function will stop (>100000
  permutations). The solution is then to provide segments that are
  ordered in the same manner as the tree labels (or vice-versa).

  There is an (experimental) support for branch annotations. These are
  given in the Newick tree, directly after the parenthesis closing a
  node. They can be characters or integers, but so far
  \code{newick2phylog} doesn't support decimal values. Tags will be
  ignored if they start with "I", and trimmed if they start with "X".

  The format of the elements of \code{dna_segs} is previously determined
  in the object or can be globally set by \code{gene_type}. See the
  function \code{\link{gene_types}} to return the available types. Gene
  type can also be user-defined, using a function returning a
  \code{grob}. See \code{\link{gene_types}} for more details.

  \code{xlims} allow the user to plot subsegments of a
  \code{dna_seg}. \code{xlims} consists of a list composed of as many
  numeric vectors as there are segments. Each of these numeric vectors
  give pairs of left and right borders, and gives the
  direction. For example, c(1,2,6,4) will plot two subsegments, segment
  1 to 2 which is plotted left to right and segment 4 to 6, plotted
  right to left. \code{-Inf} and \code{Inf} values are
  accepted. \code{NULL} values will result in plotting the whole
  segment.

  \code{offsets} allows to user to define the placement of the
  subsegments. If a list is provided, each element of the list should
  have as many elements as there are subsegments. It will give the size
  of the gaps, including the first one from the border of the plot to
  the first subsegment.

  A main title (\code{main}) can also be added at the top of the plot,
  at the position defined by \code{main_pos}. A general scale
  can be added at the bottom right of the plot (\code{scale}).
  
  \code{dna_seg_scale} gives the ability to plot scales on one, some or
  every segment. \code{c(TRUE, FALSE, TRUE)} will add scales to the
  first and third segments.

  The four elements of \code{global_color_scheme} are (i) which column
  serves as scale to apply the color  scheme, or "auto" (default);
  (ii) if the scale is "increasing" or "decreasing" (see
  \code{apply_color_scheme} for more details), or "auto" (default);
  (iii) the color scheme to apply; (iv) the transparency to apply (0.5
  by default).
}
\value{
  %% Change this when/if would return gTree
  Nothing. A lattice graphic is plotted on the current device.
}
\author{
  Lionel Guy \email{lionel.guy@ebc.uu.se}, Jens Roat Kultima
}
\note{
  This plotting function has been tested as far as possible, but given
  its complexity and that the package is young, bugs or strange behaviors
  are possible. Please report them to the author.

  As of 10/3/2010, support for viewing exons/introns is only available
  using genbank and embl formats, not when importing ptt files.
}
\seealso{
  \code{\link{dna_seg}} and \code{\link{comparison}} for the base objects;
  \code{\link{read_dna_seg_from_tab}}, \code{\link{read_dna_seg_from_ptt}},
  \code{\link{read_comparison_from_tab}} and
  \code{\link{read_comparison_from_blast}} to read from files;
  \code{\link{annotation}} to annotate \code{dna_seg}s;
  \code{\link{seg_plot}} to draw plots next to \code{dna_seg}s;
  \code{\link{gene_types}} for \code{gene_type} argument;
  \code{\link{apply_color_scheme}} for color schemes;
}
\examples{
old.par <- par(no.readonly=TRUE)
data("three_genes")

## Segments only
plot_gene_map(dna_segs=dna_segs) 

## With comparisons
plot_gene_map(dna_segs=dna_segs, comparisons=comparisons) 

## Tree
names <- c("A_aaa", "B_bbb", "C_ccc")
names(dna_segs) <- names
tree <- newick2phylog("(((A_aaa:4.2,B_bbb:3.9):3.1,C_ccc:7.3):1);")
plot_gene_map(dna_segs=dna_segs, comparisons=comparisons,
              tree=tree)
## Increasing tree width
plot_gene_map(dna_segs=dna_segs, comparisons=comparisons,
              tree=tree, tree_width=3)
## Annotations on the tree
tree2 <- newick2phylog("(((A_aaa:4.2,B_bbb:3.9)97:3.1,C_ccc:7.3)78:1);")
plot_gene_map(dna_segs=dna_segs, comparisons=comparisons,
              tree=tree2, tree_width=3)
plot_gene_map(dna_segs=dna_segs, comparisons=comparisons,
              tree=tree2, tree_width=3, tree_branch_labels_cex=0.5)
plot_gene_map(dna_segs=dna_segs, comparisons=comparisons,
              tree=tree2, tree_width=3, tree_branch_labels_cex=0)

## Annotation
## Calculating middle positions
mid_pos <- middle(dna_segs[[1]])

# Create first annotation
annot1 <- annotation(x1=mid_pos, text=dna_segs[[1]]$name)
plot_gene_map(dna_segs=dna_segs, comparisons=comparisons, annotations=annot1)

## Exploring options
annot2 <- annotation(x1=c(mid_pos[1], dna_segs[[1]]$end[2]),
                     x2=c(NA, dna_segs[[1]]$end[3]),
                     text=c(dna_segs[[1]]$name[1], "region1"),
                     rot=c(30, 0), col=c("grey", "black"))
plot_gene_map(dna_segs=dna_segs, comparisons=comparisons,
              annotations=annot2, annotation_height=1.3)

## xlims
## Just returning a segment
plot_gene_map(dna_segs, comparisons,
              xlims=list(NULL, NULL, c(Inf,-Inf)),
              dna_seg_scale=TRUE)
## Removing one gene
plot_gene_map(dna_segs, comparisons,
              xlims=list(NULL, NULL, c(-Inf,2800)),
              dna_seg_scale=TRUE)

## offsets
offsets <- c(0, 0, 0)  
plot_gene_map(dna_segs=dna_segs, comparisons=comparisons, offsets=offsets)
offsets <- c(200, 400, 0)  
plot_gene_map(dna_segs=dna_segs, comparisons=comparisons, offsets=offsets)

## main
plot_gene_map(dna_segs=dna_segs, comparisons=comparisons,
              main="Comparison of A, B and C")
plot_gene_map(dna_segs=dna_segs, comparisons=comparisons,
              main="Comparison of A, B and C", main_pos="left")

## dna_seg_labels
plot_gene_map(dna_segs=dna_segs, comparisons=comparisons,
              dna_seg_labels=c("Huey", "Dewey", "Louie"))

## dna_seg_labels size
plot_gene_map(dna_segs=dna_segs, comparisons=comparisons,
              dna_seg_labels=c("Huey", "Dewey", "Louie"),
              dna_seg_label_cex=2)

## dna_seg_line
plot_gene_map(dna_segs=dna_segs, comparisons=comparisons,
              dna_seg_line=c("FALSE", "red", grey(0.6)))

## gene_type
plot_gene_map(dna_segs=dna_segs, comparisons=comparisons,
              gene_type="side_blocks")

##
## From here on, using a bigger dataset from a 4-genome comparison
##
data("barto")
## Adding a tree
tree <- newick2phylog("(BB:2.5,(BG:1.8,(BH:1,BQ:0.8):1.9):3);")
## Showing only subsegments
xlims1 <- list(c(1380000, 1445000),
               c(10000, 83000),
               c(15000, 98000),
               c(5000, 82000))
## Reducing dataset size for speed purpose
for (i in 1:length(barto$dna_segs)){
    barto$dna_segs[[i]] <- trim(barto$dna_segs[[i]], xlim=xlims1[[i]])
    if (i < length(barto$dna_segs))
        barto$comparisons[[i]] <- trim(barto$comparisons[[i]],
                                      xlim1=xlims1[[i]], xlims1[[i+1]])   
}
plot_gene_map(barto$dna_segs, barto$comparisons, tree=tree,
              xlims=xlims1,
              dna_seg_scale=TRUE)
## Showing several subsegments per genome
xlims2 <- list(c(1445000, 1415000, 1380000, 1412000),
               c(  10000,   45000,   50000,   83000, 90000, 120000),
               c(  15000,   36000,   90000,  120000, 74000,  98000),
               c(   5000,    82000))

## dna_seg_scale, global_color_scheme, size, number, color of dna_seg_scale,
## size of dna_seg_scale labels
plot_gene_map(barto$dna_segs, barto$comparisons, tree=tree, xlims=xlims2,
              dna_seg_scale=c(TRUE, FALSE, FALSE, TRUE), scale=FALSE,
              dna_seg_label_cex=1.7,
              dna_seg_label_col=c("black", "grey", "blue", "red"),
              global_color_scheme=c("e_value", "auto", "grey", "0.7"),
              n_scale_ticks=3, scale_cex=1)

## Hand-made offsets: size of all gaps
offsets2 <- list(c(10000, 10000),
                 c(2000, 2000, 2000),
                 c(10000, 5000, 2000),
                 c(10000))
plot_gene_map(barto$dna_segs, barto$comparisons, tree=tree,
              #annotations=annots,
              xlims=xlims2,
              offsets=offsets2,
              dna_seg_scale=TRUE)
##
## Exploring and modifying a previously plotted gene map plot
##
## View viewports
current.vpTree()
## Go down to one of the viewports, add an xaxis, go back up to root viewport
downViewport("dna_seg_scale.3.2")
grid.rect()
upViewport(0)
## Get all the names of the objects
grobNames <- getNames()
grobNames
## Change the color ot the scale line
grid.edit("scale.lines", gp=gpar(col="grey"))
## Remove first dna_seg_lines
grid.remove("dna_seg_line.1.1")

##
## Plot genoPlotR logo
##
col <- c("#B2182B", "#D6604D", "#F4A582", "#FDDBC7",
         "#D1E5F0", "#92C5DE", "#4393C3", "#2166AC")
cex <- 2.3
## First segment 
start1 <- c(150, 390, 570)
end1   <- c(  1, 490, 690)
genoR <- c(270, 530)
## Second segment
start2 <- c(100, 520, 550)
end2   <- c(240, 420, 650)
Plot <- c(330)
## dna_segs
ds1 <- as.dna_seg(data.frame(name=c("", "", ""),
                             start=start1, end=end1, strand=rep(1, 3),
                             col=col[c(2, 6, 1)], stringsAsFactor=FALSE))
ds_genoR <- as.dna_seg(data.frame(name=c("geno", "R"),
                             start=genoR, end=genoR, strand=rep(1, 2),
                             col=c(col[8], "black"),
                             stringsAsFactor=FALSE), cex=cex, gene_type="text")
ds2 <- as.dna_seg(data.frame(name=c("", "", ""),
                             start=start2, end=end2, strand=rep(1, 3),
                             col=col[c(5, 3, 7)],
                             stringsAsFactor=FALSE))
ds_Plot <- as.dna_seg(data.frame(name="Plot",
                             start=Plot, end=Plot, strand=1,
                             col=col[c(1)],
                             stringsAsFactor=FALSE), cex=cex, gene_type="text")
## comparison
c1 <- as.comparison(data.frame(start1=start1, end1=end1,
                               start2=start2, end2=end2,
                               col=grey(c(0.6, 0.8, 0.5))))
## Generate genoPlotR logo
\dontrun{
pdf("logo.pdf", h=0.7, w=3)
}
par(fin=c(0.7, 3))
plot_gene_map(dna_segs=list(c(ds1, ds_genoR), c(ds2, ds_Plot)),
              comparisons=list(c1), scale=FALSE, dna_seg_scale=FALSE,
              dna_seg_line=grey(0.7), offsets=c(-20,160))
\dontrun{
dev.off()
}
par(old.par)
}
\keyword{ hplot }
